@@ -87,6 +87,7 @@
 #include <QQmlEngine>
 #include <QtGui/qevent.h>
 #include <QtGui/qguiapplication.h>
+#include <QProcessEnvironment>
 
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 #include <QtQuick/private/qquickrendercontrol_p.h>
@@ -104,7 +105,9 @@
 #endif
 
 #ifdef QUICK3D_PARTICLES_MODULE
-#include <QtQuick3DParticles/private/qquick3dparticlesystem_p.h>
+#include <QtQuick3DParticles/private/qquick3dparticle_p.h>
+#include <QtQuick3DParticles/private/qquick3dparticleaffector_p.h>
+#include <QtQuick3DParticles/private/qquick3dparticleemitter_p.h>
 #endif
 
 #ifdef IMPORT_QUICK3D_ASSETS
@@ -150,12 +153,6 @@ static bool imageHasContent(const QImage &image)
     return false;
 }
 
-static bool isQuick3DMode()
-{
-    static bool mode3D = qEnvironmentVariableIsSet("QMLDESIGNER_QUICK3D_MODE");
-    return mode3D;
-}
-
 static QObjectList toObjectList(const QVariant &variantList)
 {
     QObjectList objList;
@@ -216,7 +213,7 @@ void Qt5InformationNodeInstanceServer::createAuxiliaryQuickView(const QUrl &url,
 
 void Qt5InformationNodeInstanceServer::updateLockedAndHiddenStates(const QSet<ServerNodeInstance> &instances)
 {
-    if (!isQuick3DMode())
+    if (!ViewConfig::isQuick3DMode())
         return;
 
     // We only want to update the topmost parents in the set
@@ -328,7 +325,7 @@ void Qt5InformationNodeInstanceServer::updateRotationBlocks(const QVector<Proper
     if (helper) {
         QSet<QQuick3DNode *> blockedNodes;
         QSet<QQuick3DNode *> unblockedNodes;
-        const PropertyName propName = "rotBlocked@internal";
+        const PropertyName propName = "rotBlocked@Internal";
         for (const auto &container : valueChanges) {
             if (container.name() == propName) {
                 ServerNodeInstance instance = instanceForId(container.instanceId());
@@ -406,9 +403,122 @@ void Qt5InformationNodeInstanceServer::createEditView3D()
 #endif
 }
 
+#ifdef QUICK3D_PARTICLES_MODULE
+void Qt5InformationNodeInstanceServer::resetParticleSystem()
+{
+    if (!m_targetParticleSystem)
+        return;
+    m_targetParticleSystem->reset();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 2)
+    m_targetParticleSystem->setEditorTime(0);
+#endif
+    if (m_particleAnimationDriver)
+        m_particleAnimationDriver->reset();
+}
+
+void Qt5InformationNodeInstanceServer::handleParticleSystemSelected(QQuick3DParticleSystem* targetParticleSystem)
+{
+    if (!m_particleAnimationDriver || targetParticleSystem == m_targetParticleSystem)
+        return;
+
+    m_particleAnimationDriver->reset();
+    // stop the previously selected from animating
+    resetParticleSystem();
+
+    m_targetParticleSystem = targetParticleSystem;
+
+    resetParticleSystem();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 2)
+    QObject::disconnect(m_particleAnimationConnection);
+    m_particleAnimationConnection = connect(m_particleAnimationDriver, &AnimationDriver::advanced, [this] () {
+        if (m_targetParticleSystem)
+            m_targetParticleSystem->setEditorTime(m_particleAnimationDriver->elapsed());
+    });
+#endif
+    if (m_particleAnimationPlaying && m_targetParticleSystem->visible())
+        m_particleAnimationDriver->restart();
+    QObject::connect(m_targetParticleSystem, &QQuick3DNode::visibleChanged, [this] () {
+        if (m_particleAnimationPlaying && m_targetParticleSystem->visible()) {
+            m_particleAnimationDriver->restart();
+        } else {
+            m_particleAnimationDriver->reset();
+            resetParticleSystem();
+        }
+    });
+
+    const auto anim = animations();
+    for (auto a : anim)
+        a->restart();
+}
+
+static QString baseProperty(const QString &property)
+{
+    int index = property.indexOf('.');
+    if (index > 0)
+        return property.left(index);
+    return property;
+}
+
+template <typename T>
+static QQuick3DParticleSystem *systemProperty(QObject *object)
+{
+    return qobject_cast<T>(object) ? qobject_cast<T>(object)->system() : nullptr;
+}
+
+static QQuick3DParticleSystem *getSystemOrSystemProperty(QObject *selectedObject)
+{
+    QQuick3DParticleSystem *system = nullptr;
+    system = qobject_cast<QQuick3DParticleSystem *>(selectedObject);
+    if (system)
+        return system;
+    system = systemProperty<QQuick3DParticle *>(selectedObject);
+    if (system)
+        return system;
+    system = systemProperty<QQuick3DParticleAffector *>(selectedObject);
+    if (system)
+        return system;
+    system = systemProperty<QQuick3DParticleEmitter *>(selectedObject);
+    if (system)
+        return system;
+    return nullptr;
+}
+
+static QQuick3DParticleSystem *parentParticleSystem(QObject *selectedObject)
+{
+    auto *ps = getSystemOrSystemProperty(selectedObject);
+    if (ps)
+        return ps;
+    QObject *parent = selectedObject->parent();
+    while (parent) {
+        ps = getSystemOrSystemProperty(parent);
+        if (ps)
+            return ps;
+        parent = parent->parent();
+    }
+    return nullptr;
+}
+
+void Qt5InformationNodeInstanceServer::handleParticleSystemDeselected(QObject *selectedObject)
+{
+    m_targetParticleSystem = nullptr;
+    const auto anim = animations();
+    int i = 0;
+    for (auto a : anim) {
+        a->stop();
+        QQuickPropertyAnimation *panim = qobject_cast<QQuickPropertyAnimation *>(a);
+        if (panim)
+            panim->target()->setProperty(qPrintable(baseProperty(panim->property())), animationDefaultValue(i));
+        i++;
+    }
+}
+#endif
+
 // The selection has changed in the edit view 3D. Empty list indicates selection is cleared.
 void Qt5InformationNodeInstanceServer::handleSelectionChanged(const QVariant &objs)
 {
+#ifdef QUICK3D_PARTICLES_MODULE
+    resetParticleSystem();
+#endif
     QList<ServerNodeInstance> instanceList;
     const QVariantList varObjs = objs.value<QVariantList>();
     for (const auto &object : varObjs) {
@@ -822,13 +932,31 @@ void Qt5InformationNodeInstanceServer::doRender3DEditView()
         // Key number is selected so that it is unlikely to conflict other ImageContainer use.
         auto imgContainer = ImageContainer(-1, renderImage, 2100000000);
 
-        // send the rendered image to creator process
-        nodeInstanceClient()->handlePuppetToCreatorCommand({PuppetToCreatorCommand::Render3DView,
-                                                            QVariant::fromValue(imgContainer)});
+        // If we have only one or no render queued, send the result to the creator side.
+        // Otherwise, we'll hold on that until we have rendered all pending frames to ensure sent
+        // results are correct.
+        if (m_need3DEditViewRender <= 1) {
+            nodeInstanceClient()->handlePuppetToCreatorCommand({PuppetToCreatorCommand::Render3DView,
+                                                                QVariant::fromValue(imgContainer)});
+#ifdef QUICK3D_PARTICLES_MODULE
+            if (m_need3DEditViewRender == 0 && ViewConfig::isParticleViewMode()
+                    && m_particleAnimationDriver && m_particleAnimationDriver->isAnimating()) {
+                m_need3DEditViewRender = 1;
+            }
+#endif
+        }
+
         if (m_need3DEditViewRender > 0) {
-            m_render3DEditViewTimer.start(0);
+            // We queue another render even if the requested render count was one, because another
+            // render is needed to ensure gizmo geometries are properly updated.
+            // Note that while in theory this seems that we shouldn't need to send the image to
+            // creator side when m_need3DEditViewRender is one, we need to do it to ensure
+            // smooth operation when objects are moved via drag, which triggers new renders
+            // continueously.
+            m_render3DEditViewTimer.start(17); // 16.67ms = ~60fps, rounds up to 17
             --m_need3DEditViewRender;
         }
+
 #ifdef FPS_COUNTER
         // Force constant rendering for accurate fps count
         if (!m_render3DEditViewTimer.isActive())
@@ -870,18 +998,17 @@ void Qt5InformationNodeInstanceServer::doRenderModelNodeImageView()
 void Qt5InformationNodeInstanceServer::doRenderModelNode3DImageView()
 {
 #ifdef QUICK3D_MODULE
+    m_modelNode3DImageViewAsyncData.cleanup();
     if (m_modelNode3DImageViewData.rootItem) {
+        QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "destroyView");
         if (!m_modelNode3DImageViewData.contentItem)
             m_modelNode3DImageViewData.contentItem = getContentItemForRendering(m_modelNode3DImageViewData.rootItem);
 
-        // Key number is selected so that it is unlikely to conflict other ImageContainer use.
-        auto imgContainer = ImageContainer(m_modelNodePreviewImageCommand.instanceId(), {}, 2100000001);
-        QImage renderImage;
         if (m_modelNodePreviewImageCache.contains(m_modelNodePreviewImageCommand.componentPath())) {
-            renderImage = m_modelNodePreviewImageCache[m_modelNodePreviewImageCommand.componentPath()];
+            m_modelNode3DImageViewAsyncData.renderImage
+                    = m_modelNodePreviewImageCache[m_modelNodePreviewImageCommand.componentPath()];
+            modelNode3DImageViewSendImageToCreator();
         } else {
-            QObject *instanceObj = nullptr;
-            bool createdFromComponent = false;
             ServerNodeInstance instance = instanceForId(m_modelNodePreviewImageCommand.instanceId());
             if (!m_modelNodePreviewImageCommand.componentPath().isEmpty()
                     && instance.isSubclassOf("QQuick3DNode")) {
@@ -890,14 +1017,15 @@ void Qt5InformationNodeInstanceServer::doRenderModelNode3DImageView()
                 // wouldn't want the children of the Node to appear in the preview.
                 QQmlComponent component(engine());
                 component.loadUrl(QUrl::fromLocalFile(m_modelNodePreviewImageCommand.componentPath()));
-                instanceObj = qobject_cast<QQuick3DObject *>(component.create());
-                if (!instanceObj) {
+                m_modelNode3DImageViewAsyncData.instanceObj = qobject_cast<QQuick3DObject *>(component.create());
+                if (!m_modelNode3DImageViewAsyncData.instanceObj) {
                     qWarning() << "Could not create preview component: " << component.errors();
+                    m_modelNode3DImageViewAsyncData.cleanup();
                     return;
                 }
-                createdFromComponent = true;
+                m_modelNode3DImageViewAsyncData.createdFromComponent = true;
             } else {
-                instanceObj = instance.internalObject();
+                m_modelNode3DImageViewAsyncData.instanceObj = instance.internalObject();
             }
             QSize renderSize = m_modelNodePreviewImageCommand.size();
             if (Internal::QuickItemNodeInstance::unifiedRenderPathOrQt6()) {
@@ -916,53 +1044,73 @@ void Qt5InformationNodeInstanceServer::doRenderModelNode3DImageView()
             m_modelNode3DImageViewData.window->resize(renderSize);
             m_modelNode3DImageViewData.rootItem->setSize(renderSize);
 
-            QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "createViewForObject",
-                                      Q_ARG(QVariant, objectToVariant(instanceObj)));
+            QMetaObject::invokeMethod(
+                        m_modelNode3DImageViewData.rootItem, "createViewForObject",
+                        Q_ARG(QVariant, objectToVariant(m_modelNode3DImageViewAsyncData.instanceObj)));
 
-            bool ready = false;
-            int count = 0; // Ensure we don't ever get stuck in an infinite loop
-            while (!ready && ++count < 10) {
-                updateNodesRecursive(m_modelNode3DImageViewData.contentItem);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-                if (Internal::QuickItemNodeInstance::unifiedRenderPath()) {
-                    renderImage = m_modelNode3DImageViewData.window->grabWindow();
-                } else {
-                    // Fake render loop signaling to update things like QML items as 3D textures
-                    m_modelNode3DImageViewData.window->beforeSynchronizing();
-                    m_modelNode3DImageViewData.window->beforeRendering();
-
-                    QSizeF size = qobject_cast<QQuickItem *>(m_modelNode3DImageViewData.contentItem)->size();
-                    QRectF renderRect(QPointF(0., 0.), size);
-                    renderImage = designerSupport()->renderImageForItem(m_modelNode3DImageViewData.contentItem,
-                                                                        renderRect, size.toSize());
-
-                    m_modelNode3DImageViewData.window->afterRendering();
-                }
-#else
-                renderImage = grabRenderControl(m_modelNode3DImageViewData);
-#endif
-                QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "afterRender");
-                ready = QQmlProperty::read(m_modelNode3DImageViewData.rootItem, "ready").value<bool>();
-            }
-            QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "destroyView");
-            if (createdFromComponent) {
-                // If component changes, puppet will need a reset anyway, so we can cache the image
-                m_modelNodePreviewImageCache.insert(m_modelNodePreviewImageCommand.componentPath(), renderImage);
-                delete instanceObj;
-            }
-        }
-
-        if (!renderImage.isNull()) {
-            imgContainer.setImage(renderImage);
-
-            // send the rendered image to creator process
-            nodeInstanceClient()->handlePuppetToCreatorCommand({PuppetToCreatorCommand::RenderModelNodePreviewImage,
-                                                                QVariant::fromValue(imgContainer)});
+            // Selection box geometry updates have an asynchronous step, so we need to do rendering
+            // in asynchronous steps as well, since we are adjusting the selection box geometry
+            // while finding correct zoom level.
+            m_modelNode3DImageViewAsyncData.timer.start();
         }
     }
 #endif
 }
 
+void Qt5InformationNodeInstanceServer::modelNode3DImageViewSendImageToCreator()
+{
+    if (!m_modelNode3DImageViewAsyncData.renderImage.isNull()) {
+        // Key number is selected so that it is unlikely to conflict other ImageContainer use.
+        ImageContainer imgContainer(m_modelNodePreviewImageCommand.instanceId(), {}, 2100000001);
+        imgContainer.setImage(m_modelNode3DImageViewAsyncData.renderImage);
+
+        // send the rendered image to creator process
+        nodeInstanceClient()->handlePuppetToCreatorCommand(
+                    {PuppetToCreatorCommand::RenderModelNodePreviewImage,
+                     QVariant::fromValue(imgContainer)});
+
+        m_modelNode3DImageViewAsyncData.cleanup();
+    }
+}
+
+void Qt5InformationNodeInstanceServer::modelNode3DImageViewRenderStep()
+{
+    ++m_modelNode3DImageViewAsyncData.count;
+
+    updateNodesRecursive(m_modelNode3DImageViewData.contentItem);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    if (Internal::QuickItemNodeInstance::unifiedRenderPath()) {
+        m_modelNode3DImageViewAsyncData.renderImage = m_modelNode3DImageViewData.window->grabWindow();
+    } else {
+        // Fake render loop signaling to update things like QML items as 3D textures
+        m_modelNode3DImageViewData.window->beforeSynchronizing();
+        m_modelNode3DImageViewData.window->beforeRendering();
+
+        QSizeF size = qobject_cast<QQuickItem *>(m_modelNode3DImageViewData.contentItem)->size();
+        QRectF renderRect(QPointF(0., 0.), size);
+        m_modelNode3DImageViewAsyncData.renderImage
+                = designerSupport()->renderImageForItem(m_modelNode3DImageViewData.contentItem,
+                                                        renderRect, size.toSize());
+        m_modelNode3DImageViewData.window->afterRendering();
+    }
+#else
+    m_modelNode3DImageViewAsyncData.renderImage = grabRenderControl(m_modelNode3DImageViewData);
+#endif
+    QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "afterRender");
+    const bool ready = QQmlProperty::read(m_modelNode3DImageViewData.rootItem, "ready").value<bool>();
+    if (ready || m_modelNode3DImageViewAsyncData.count >= 10) {
+        QMetaObject::invokeMethod(m_modelNode3DImageViewData.rootItem, "destroyView");
+        if (m_modelNode3DImageViewAsyncData.createdFromComponent) {
+            // If component changes, puppet will need a reset anyway, so we can cache the image
+            m_modelNodePreviewImageCache.insert(m_modelNodePreviewImageCommand.componentPath(),
+                                                m_modelNode3DImageViewAsyncData.renderImage);
+        }
+        modelNode3DImageViewSendImageToCreator();
+    } else {
+        m_modelNode3DImageViewAsyncData.timer.start();
+    }
+}
+
 static QRectF itemBoundingRect(QQuickItem *item)
 {
     QRectF itemRect;
@@ -1077,6 +1225,7 @@ Qt5InformationNodeInstanceServer::Qt5InformationNodeInstanceServer(NodeInstanceC
     m_render3DEditViewTimer.setSingleShot(true);
     m_inputEventTimer.setSingleShot(true);
     m_renderModelNodeImageViewTimer.setSingleShot(true);
+    m_modelNode3DImageViewAsyncData.timer.setSingleShot(true);
 
 #ifdef FPS_COUNTER
     if (!_fpsTimer) {
@@ -1084,17 +1233,24 @@ Qt5InformationNodeInstanceServer::Qt5InformationNodeInstanceServer(NodeInstanceC
         _fpsTimer->start();
     }
 #endif
+#ifdef QUICK3D_PARTICLES_MODULE
+    if (ViewConfig::isParticleViewMode()) {
+        m_particleAnimationDriver = new AnimationDriver(this);
+        m_particleAnimationDriver->setInterval(17);
+    }
+#endif
 }
 
 Qt5InformationNodeInstanceServer::~Qt5InformationNodeInstanceServer()
 {
     m_editView3DSetupDone = false;
 
-    m_propertyChangeTimer.stop();
     m_propertyChangeTimer.stop();
     m_selectionChangeTimer.stop();
     m_render3DEditViewTimer.stop();
     m_inputEventTimer.stop();
+    m_renderModelNodeImageViewTimer.stop();
+    m_modelNode3DImageViewAsyncData.timer.stop();
 
     if (m_editView3DData.rootItem)
         m_editView3DData.rootItem->disconnect(this);
@@ -1205,7 +1361,7 @@ QList<ServerNodeInstance> Qt5InformationNodeInstanceServer::createInstances(
 void Qt5InformationNodeInstanceServer::initializeAuxiliaryViews()
 {
 #ifdef QUICK3D_MODULE
-    if (isQuick3DMode())
+    if (ViewConfig::isQuick3DMode())
         createEditView3D();
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     createAuxiliaryQuickView(QUrl("qrc:/qtquickplugin/mockfiles/qt6/ModelNode3DImageView.qml"),
@@ -1487,6 +1643,8 @@ void Qt5InformationNodeInstanceServer::setup3DEditView(const QList<ServerNodeIns
                      this, &Qt5InformationNodeInstanceServer::doRender3DEditView);
     QObject::connect(&m_inputEventTimer, &QTimer::timeout,
                      this, &Qt5InformationNodeInstanceServer::handleInputEvents);
+    QObject::connect(&m_modelNode3DImageViewAsyncData.timer, &QTimer::timeout,
+                     this, &Qt5InformationNodeInstanceServer::modelNode3DImageViewRenderStep);
 
     QString lastSceneId;
     auto helper = qobject_cast<QmlDesigner::Internal::GeneralHelper *>(m_3dHelper);
@@ -1665,7 +1823,7 @@ void Qt5InformationNodeInstanceServer::createScene(const CreateSceneCommand &com
     sendChildrenChangedCommand(instanceList);
     nodeInstanceClient()->componentCompleted(createComponentCompletedCommand(instanceList));
 
-    if (isQuick3DMode()) {
+    if (ViewConfig::isQuick3DMode()) {
         setup3DEditView(instanceList, command.edit3dToolStates);
         updateRotationBlocks(command.auxiliaryChanges);
     }
@@ -1762,14 +1920,29 @@ void Qt5InformationNodeInstanceServer::changeSelection(const ChangeSelectionComm
             if (firstSceneRoot && sceneRoot == firstSceneRoot && instance.isSubclassOf("QQuick3DNode"))
                 object = instance.internalObject();
 
+#ifdef QUICK3D_PARTICLES_MODULE
+            auto particlesystem = qobject_cast<QQuick3DParticleSystem *>(instance.internalObject());
+            if (particlesystem) {
+                handleParticleSystemSelected(particlesystem);
+            } else {
+                particlesystem = parentParticleSystem(instance.internalObject());
+                if (particlesystem) {
+                    if (particlesystem != m_targetParticleSystem)
+                        handleParticleSystemSelected(particlesystem);
+                } else {
+                    handleParticleSystemDeselected(instance.internalObject());
+                }
+            }
+#endif
             auto isSelectableAsRoot = [&]() -> bool {
 #ifdef QUICK3D_MODULE
                 if (qobject_cast<QQuick3DModel *>(object)
                     || qobject_cast<QQuick3DCamera *>(object)
+                    || qobject_cast<QQuick3DAbstractLight *>(object)
 #ifdef QUICK3D_PARTICLES_MODULE
                     || qobject_cast<QQuick3DParticleSystem *>(object)
 #endif
-                    || qobject_cast<QQuick3DAbstractLight *>(object)) {
+                ) {
                     return true;
                 }
                 // Node is a component if it has node children that have no instances
@@ -1888,8 +2061,8 @@ void Qt5InformationNodeInstanceServer::view3DAction(const View3DActionCommand &c
         break;
     case View3DActionCommand::CameraToggle:
         updatedState.insert("usePerspective", command.isEnabled());
-        // It can take a couple frames to properly update icon gizmo positions, so render 3 frames
-        renderCount = 3;
+        // It can take a couple frames to properly update icon gizmo positions
+        renderCount = 2;
         break;
     case View3DActionCommand::OrientationToggle:
         updatedState.insert("globalOrientation", command.isEnabled());
@@ -1900,6 +2073,31 @@ void Qt5InformationNodeInstanceServer::view3DAction(const View3DActionCommand &c
     case View3DActionCommand::ShowGrid:
         updatedState.insert("showGrid", command.isEnabled());
         break;
+#ifdef QUICK3D_PARTICLES_MODULE
+    case View3DActionCommand::ParticlesPlay:
+        m_particleAnimationPlaying = command.isEnabled();
+        updatedState.insert("particlePlay", command.isEnabled());
+        if (m_particleAnimationPlaying) {
+            m_particleAnimationDriver->reset();
+            m_particleAnimationDriver->restart();
+            m_particleAnimationDriver->setSeekerEnabled(false);
+            m_particleAnimationDriver->setSeekerPosition(0);
+        } else {
+            m_particleAnimationDriver->reset();
+            m_particleAnimationDriver->setSeekerEnabled(true);
+        }
+        break;
+    case View3DActionCommand::ParticlesRestart:
+        resetParticleSystem();
+        m_particleAnimationPlaying = true;
+        m_particleAnimationDriver->restart();
+        m_particleAnimationDriver->setSeekerEnabled(false);
+        m_particleAnimationDriver->setSeekerPosition(0);
+        break;
+    case View3DActionCommand::ParticlesSeek:
+        m_particleAnimationDriver->setSeekerPosition(static_cast<const View3DSeekActionCommand &>(command).position());
+        break;
+#endif
     default:
         break;
     }
@@ -1915,6 +2113,7 @@ void Qt5InformationNodeInstanceServer::view3DAction(const View3DActionCommand &c
 
 void Qt5InformationNodeInstanceServer::requestModelNodePreviewImage(const RequestModelNodePreviewImageCommand &command)
 {
+    m_modelNode3DImageViewAsyncData.timer.stop();
     m_modelNodePreviewImageCommand = command;
     renderModelNodeImageView();
 }
@@ -1977,7 +2176,7 @@ void Qt5InformationNodeInstanceServer::handleInstanceLocked(const ServerNodeInst
                                                             bool enable, bool checkAncestors)
 {
 #ifdef QUICK3D_MODULE
-    if (!isQuick3DMode())
+    if (!ViewConfig::isQuick3DMode())
         return;
 
     bool edit3dLocked = enable;
@@ -2015,7 +2214,7 @@ void Qt5InformationNodeInstanceServer::handleInstanceHidden(const ServerNodeInst
                                                             bool enable, bool checkAncestors)
 {
 #ifdef QUICK3D_MODULE
-    if (!isQuick3DMode())
+    if (!ViewConfig::isQuick3DMode())
         return;
 
     bool edit3dHidden = enable;
@@ -2110,8 +2309,8 @@ void Qt5InformationNodeInstanceServer::update3DViewState(const Update3dViewState
             auto helper = qobject_cast<QmlDesigner::Internal::GeneralHelper *>(m_3dHelper);
             if (helper)
                 helper->storeToolState(helper->globalStateId(), helper->rootSizeKey(), QVariant(command.size()), 0);
-            // Queue three renders to make sure icon gizmos update properly
-            render3DEditView(3);
+            // Queue two renders to make sure all gizmos update properly
+            render3DEditView(2);
         }
     }
 #else
