@@ -23,16 +23,18 @@
 **
 ****************************************************************************/
 
+#include "autotestunittests.h"
 #include "autotestconstants.h"
 #include "autotestplugin.h"
-#include "autotestunittests.h"
 #include "testcodeparser.h"
 #include "testsettings.h"
 #include "testtreemodel.h"
 
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolstestcase.h>
-#include <cpptools/projectinfo.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolstestcase.h>
+#include <cppeditor/projectinfo.h>
+
+#include <extensionsystem/pluginmanager.h>
 
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/kitmanager.h>
@@ -46,6 +48,7 @@
 #include <qtsupport/qtkitinformation.h>
 
 using namespace Core;
+using namespace ExtensionSystem;
 using namespace ProjectExplorer;
 using namespace Utils;
 
@@ -78,9 +81,9 @@ void AutoTestUnitTests::initTestCase()
     if (!toolchain)
         QSKIP("This test requires that there is a kit with a toolchain.");
 
-    m_tmpDir = new CppTools::Tests::TemporaryCopiedDir(":/unit_test");
+    m_tmpDir = new CppEditor::Tests::TemporaryCopiedDir(":/unit_test");
 
-    if (!qgetenv("BOOST_INCLUDE_DIR").isEmpty()) {
+    if (!qEnvironmentVariableIsEmpty("BOOST_INCLUDE_DIR")) {
         m_checkBoost = true;
     } else {
         if (Utils::HostOsInfo::isLinuxHost()
@@ -105,9 +108,8 @@ void AutoTestUnitTests::testCodeParser()
     QFETCH(int, expectedUnnamedQuickTestsCount);
     QFETCH(int, expectedDataTagsCount);
 
-    CppTools::Tests::ProjectOpenerAndCloser projectManager;
-    const CppTools::ProjectInfo projectInfo = projectManager.open(projectFilePath, true, m_kit);
-    QVERIFY(projectInfo.isValid());
+    CppEditor::Tests::ProjectOpenerAndCloser projectManager;
+    QVERIFY(projectManager.open(projectFilePath, true, m_kit));
 
     QSignalSpy parserSpy(m_model->parser(), SIGNAL(parsingFinished()));
     QSignalSpy modelUpdateSpy(m_model, SIGNAL(sweepingDone()));
@@ -154,11 +156,10 @@ void AutoTestUnitTests::testCodeParserSwitchStartup()
     QFETCH(QList<int>, expectedUnnamedQuickTestsCount);
     QFETCH(QList<int>, expectedDataTagsCount);
 
-    CppTools::Tests::ProjectOpenerAndCloser projectManager;
+    CppEditor::Tests::ProjectOpenerAndCloser projectManager;
     for (int i = 0; i < projectFilePaths.size(); ++i) {
         qDebug() << "Opening project" << projectFilePaths.at(i);
-        CppTools::ProjectInfo projectInfo = projectManager.open(projectFilePaths.at(i), true, m_kit);
-        QVERIFY(projectInfo.isValid());
+        QVERIFY(projectManager.open(projectFilePaths.at(i), true, m_kit));
 
         QSignalSpy parserSpy(m_model->parser(), SIGNAL(parsingFinished()));
         QSignalSpy modelUpdateSpy(m_model, SIGNAL(sweepingDone()));
@@ -200,20 +201,19 @@ void AutoTestUnitTests::testCodeParserSwitchStartup_data()
 
 void AutoTestUnitTests::testCodeParserGTest()
 {
-    if (qgetenv("GOOGLETEST_DIR").isEmpty())
+    if (qEnvironmentVariableIsEmpty("GOOGLETEST_DIR"))
         QSKIP("This test needs googletest - set GOOGLETEST_DIR (point to googletest repository)");
 
     QFETCH(QString, projectFilePath);
-    CppTools::Tests::ProjectOpenerAndCloser projectManager;
-    CppTools::ProjectInfo projectInfo = projectManager.open(projectFilePath, true, m_kit);
-    QVERIFY(projectInfo.isValid());
+    CppEditor::Tests::ProjectOpenerAndCloser projectManager;
+    QVERIFY(projectManager.open(projectFilePath, true, m_kit));
 
     QSignalSpy parserSpy(m_model->parser(), SIGNAL(parsingFinished()));
     QSignalSpy modelUpdateSpy(m_model, SIGNAL(sweepingDone()));
     QVERIFY(parserSpy.wait(20000));
     QVERIFY(modelUpdateSpy.wait());
 
-    QCOMPARE(m_model->gtestNamesCount(), 7);
+    QCOMPARE(m_model->gtestNamesCount(), 8);
 
     QMultiMap<QString, int> expectedNamesAndSets;
     expectedNamesAndSets.insert(QStringLiteral("FactorialTest"), 3);
@@ -222,6 +222,7 @@ void AutoTestUnitTests::testCodeParserGTest()
     expectedNamesAndSets.insert(QStringLiteral("QueueTest"), 2);
     expectedNamesAndSets.insert(QStringLiteral("DummyTest"), 1); // used as parameterized test
     expectedNamesAndSets.insert(QStringLiteral("DummyTest"), 1); // used as 'normal' test
+    expectedNamesAndSets.insert(QStringLiteral("NumberAsNameStart"), 1);
     expectedNamesAndSets.insert(QStringLiteral("NamespaceTest"), 1);
 
     QMultiMap<QString, int> foundNamesAndSets = m_model->gtestNamesAndSets();
@@ -253,9 +254,10 @@ void AutoTestUnitTests::testCodeParserBoostTest()
 
     QFETCH(QString, projectFilePath);
     QFETCH(QString, extension);
-    CppTools::Tests::ProjectOpenerAndCloser projectManager;
-    CppTools::ProjectInfo projectInfo = projectManager.open(projectFilePath, true, m_kit);
-    QVERIFY(projectInfo.isValid());
+    CppEditor::Tests::ProjectOpenerAndCloser projectManager;
+    const CppEditor::ProjectInfo::ConstPtr projectInfo
+            = projectManager.open(projectFilePath, true, m_kit);
+    QVERIFY(projectInfo);
 
     QSignalSpy parserSpy(m_model->parser(), SIGNAL(parsingFinished()));
     QSignalSpy modelUpdateSpy(m_model, SIGNAL(sweepingDone()));
@@ -264,21 +266,19 @@ void AutoTestUnitTests::testCodeParserBoostTest()
 
     QCOMPARE(m_model->boostTestNamesCount(), 5);
 
-    QString basePath;
-    if (auto project = projectInfo.project())
-        basePath = project->projectFilePath().toFileInfo().absolutePath();
+    const Utils::FilePath basePath = projectInfo->projectRoot();
     QVERIFY(!basePath.isEmpty());
 
     QMap<QString, int> expectedSuitesAndTests;
 
     auto pathConstructor = [basePath, extension](const QString &name, const QString &subPath) {
-        return QString(name + '|' + basePath + subPath + extension);
+        return QString(name + '|' + basePath.pathAppended(subPath + extension).toString());
     };
-    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "/tests/deco/deco"), 2); // decorators w/o suite
-    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "/tests/fix/fix"), 2); // fixtures
-    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "/tests/params/params"), 3); // functions
-    expectedSuitesAndTests.insert(pathConstructor("Suite1", "/tests/deco/deco"), 4);
-    expectedSuitesAndTests.insert(pathConstructor("SuiteOuter", "/tests/deco/deco"), 5); // 2 sub suites + 3 tests
+    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "tests/deco/deco"), 2); // decorators w/o suite
+    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "tests/fix/fix"), 2); // fixtures
+    expectedSuitesAndTests.insert(pathConstructor("Master Test Suite", "tests/params/params"), 3); // functions
+    expectedSuitesAndTests.insert(pathConstructor("Suite1", "tests/deco/deco"), 4);
+    expectedSuitesAndTests.insert(pathConstructor("SuiteOuter", "tests/deco/deco"), 5); // 2 sub suites + 3 tests
 
     QMap<QString, int> foundNamesAndSets = m_model->boostTestSuitesAndTests();
     QCOMPARE(expectedSuitesAndTests.size(), foundNamesAndSets.size());
@@ -303,5 +303,24 @@ void AutoTestUnitTests::testCodeParserBoostTest_data()
         << QString(m_tmpDir->path() + "/simple_boost/simple_boost.qbs") << QString(".qbs");
 }
 
+static int executeScenario(const QString &scenario)
+{
+    const PluginManager::ProcessData data = PluginManager::creatorProcessData();
+    QStringList additionalArgs{ "-scenario", scenario };
+    if (!data.m_args.contains("-settingspath") && !data.m_settingsPath.isEmpty())
+        additionalArgs << "-settingspath" << data.m_settingsPath;
+    return QProcess::execute(data.m_executable, data.m_args + additionalArgs);
+}
+
+void AutoTestUnitTests::testStringTable()
+{
+    QCOMPARE(executeScenario("TestStringTable"), 0);
+}
+
+void AutoTestUnitTests::testModelManagerInterface()
+{
+    QCOMPARE(executeScenario("TestModelManagerInterface"), 0);
+}
+
 } // namespace Internal
 } // namespace Autotest
