@@ -26,6 +26,7 @@
 #include "clangtextmark.h"
 
 #include "clangconstants.h"
+#include "clangdclient.h"
 #include "clangdiagnostictooltipwidget.h"
 #include "clangeditordocumentprocessor.h"
 #include "clangmodelmanagersupport.h"
@@ -33,9 +34,10 @@
 #include "clangutils.h"
 
 #include <coreplugin/icore.h>
-#include <cpptools/clangdiagnosticconfigsmodel.h>
-#include <cpptools/cpptoolsreuse.h>
-#include <cpptools/cppcodemodelsettings.h>
+#include <cppeditor/clangdiagnosticconfigsmodel.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cppcodemodelsettings.h>
 
 #include <utils/fadingindicator.h>
 #include <utils/qtcassert.h>
@@ -48,8 +50,10 @@
 #include <QLayout>
 #include <QString>
 
-using namespace CppTools;
+using namespace CppEditor;
 using namespace ClangCodeModel::Internal;
+using namespace LanguageClient;
+using namespace LanguageServerProtocol;
 using namespace Utils;
 
 namespace ClangCodeModel {
@@ -71,20 +75,20 @@ bool isWarningOrNote(ClangBackEnd::DiagnosticSeverity severity)
     Q_UNREACHABLE();
 }
 
-static Core::Id categoryForSeverity(ClangBackEnd::DiagnosticSeverity severity)
+static Id categoryForSeverity(ClangBackEnd::DiagnosticSeverity severity)
 {
     return isWarningOrNote(severity) ? Constants::CLANG_WARNING : Constants::CLANG_ERROR;
 }
 
 ProjectExplorer::Project *projectForCurrentEditor()
 {
-    const QString filePath = Utils::currentCppEditorDocumentFilePath();
+    const QString filePath = currentCppEditorDocumentFilePath();
     if (filePath.isEmpty())
         return nullptr;
 
     if (auto processor = ClangEditorDocumentProcessor::get(filePath)) {
-        if (ProjectPart::Ptr projectPart = processor->projectPart())
-            return projectPart->project;
+        if (ProjectPart::ConstPtr projectPart = processor->projectPart())
+            return projectForProjectPart(*projectPart);
     }
 
     return nullptr;
@@ -97,8 +101,8 @@ DiagnosticType diagnosticType(const ClangBackEnd::DiagnosticContainer &diagnosti
     if (!diagnostic.disableOption.isEmpty())
         return DiagnosticType::Clang;
 
-    const Utils::DiagnosticTextInfo textInfo(diagnostic.text);
-    if (Utils::DiagnosticTextInfo::isClazyOption(textInfo.option()))
+    const DiagnosticTextInfo textInfo(diagnostic.text);
+    if (DiagnosticTextInfo::isClazyOption(textInfo.option()))
         return DiagnosticType::Clazy;
     return DiagnosticType::Tidy;
 }
@@ -106,8 +110,6 @@ DiagnosticType diagnosticType(const ClangBackEnd::DiagnosticContainer &diagnosti
 void disableDiagnosticInConfig(ClangDiagnosticConfig &config,
                                const ClangBackEnd::DiagnosticContainer &diagnostic)
 {
-    using namespace ClangCodeModel::Utils;
-
     switch (diagnosticType(diagnostic)) {
     case DiagnosticType::Clang:
         config.setClangOptions(config.clangOptions() + QStringList(diagnostic.disableOption));
@@ -127,19 +129,19 @@ void disableDiagnosticInConfig(ClangDiagnosticConfig &config,
     }
 }
 
-ClangDiagnosticConfig diagnosticConfig(ClangProjectSettings &projectSettings,
-                                       CppCodeModelSettings &globalSettings)
+ClangDiagnosticConfig diagnosticConfig(const ClangProjectSettings &projectSettings,
+                                       const CppCodeModelSettings &globalSettings)
 {
     ProjectExplorer::Project *project = projectForCurrentEditor();
     QTC_ASSERT(project, return {});
 
     // Get config id
-    Core::Id currentConfigId = projectSettings.warningConfigId();
+    Id currentConfigId = projectSettings.warningConfigId();
     if (projectSettings.useGlobalConfig())
         currentConfigId = globalSettings.clangDiagnosticConfigId();
 
     // Get config
-    ClangDiagnosticConfigsModel configsModel(globalSettings.clangCustomDiagnosticConfigs());
+    ClangDiagnosticConfigsModel configsModel = CppEditor::diagnosticConfigsModel();
     QTC_ASSERT(configsModel.hasConfigWithId(currentConfigId), return {});
     return configsModel.configWithId(currentConfigId);
 }
@@ -152,10 +154,10 @@ bool isDiagnosticConfigChangable(ProjectExplorer::Project *project,
 
     ClangProjectSettings &projectSettings = ClangModelManagerSupport::instance()->projectSettings(
         project);
-    const QSharedPointer<CppCodeModelSettings> globalSettings = codeModelSettings();
+    const CppCodeModelSettings *globalSettings = codeModelSettings();
     const ClangDiagnosticConfig config = diagnosticConfig(projectSettings, *globalSettings);
 
-    if (config.clangTidyMode() == ClangDiagnosticConfig::TidyMode::File
+    if (config.clangTidyMode() == ClangDiagnosticConfig::TidyMode::UseConfigFile
         && diagnosticType(diagnostic) == DiagnosticType::Tidy) {
         return false;
     }
@@ -170,11 +172,11 @@ void disableDiagnosticInCurrentProjectConfig(const ClangBackEnd::DiagnosticConta
     // Get settings
     ClangProjectSettings &projectSettings = ClangModelManagerSupport::instance()->projectSettings(
         project);
-    const QSharedPointer<CppCodeModelSettings> globalSettings = codeModelSettings();
+    CppCodeModelSettings *globalSettings = codeModelSettings();
 
     // Get config
     ClangDiagnosticConfig config = diagnosticConfig(projectSettings, *globalSettings);
-    ClangDiagnosticConfigsModel configsModel(globalSettings->clangCustomDiagnosticConfigs());
+    ClangDiagnosticConfigsModel configsModel = CppEditor::diagnosticConfigsModel();
 
     // Create copy if needed
     if (config.isReadOnly()) {
@@ -202,9 +204,9 @@ void disableDiagnosticInCurrentProjectConfig(const ClangBackEnd::DiagnosticConta
     const QString text
         = QCoreApplication::translate("ClangDiagnosticConfig",
                                       "Changes applied in Projects Mode > Clang Code Model");
-    ::Utils::FadingIndicator::showText(Core::ICore::mainWindow(),
-                                       text,
-                                       ::Utils::FadingIndicator::SmallText);
+    FadingIndicator::showText(Core::ICore::mainWindow(),
+                              text,
+                              FadingIndicator::SmallText);
 }
 
 } // anonymous namespace
@@ -212,13 +214,16 @@ void disableDiagnosticInCurrentProjectConfig(const ClangBackEnd::DiagnosticConta
 ClangTextMark::ClangTextMark(const FilePath &fileName,
                              const ClangBackEnd::DiagnosticContainer &diagnostic,
                              const RemovedFromEditorHandler &removedHandler,
-                             bool fullVisualization)
+                             bool fullVisualization, const ClangDiagnosticManager *diagMgr)
     : TextEditor::TextMark(fileName,
                            int(diagnostic.location.line),
                            categoryForSeverity(diagnostic.severity))
     , m_diagnostic(diagnostic)
     , m_removedFromEditorHandler(removedHandler)
+    , m_diagMgr(diagMgr)
 {
+    setSettingsPage(CppEditor::Constants::CPP_CODE_MODEL_SETTINGS_ID);
+
     const bool warning = isWarningOrNote(diagnostic.severity);
     setDefaultToolTip(warning ? QApplication::translate("Clang Code Model Marks", "Code Model Warning")
                               : QApplication::translate("Clang Code Model Marks", "Code Model Error"));
@@ -226,15 +231,16 @@ ClangTextMark::ClangTextMark(const FilePath &fileName,
                         : TextEditor::TextMark::HighPriority);
     updateIcon();
     if (fullVisualization) {
-        setLineAnnotation(Utils::diagnosticCategoryPrefixRemoved(diagnostic.text.toString()));
-        setColor(warning ? ::Utils::Theme::CodeModel_Warning_TextMarkColor
-                         : ::Utils::Theme::CodeModel_Error_TextMarkColor);
+        setLineAnnotation(diagnosticCategoryPrefixRemoved(diagnostic.text.toString()));
+        setColor(warning ? Theme::CodeModel_Warning_TextMarkColor
+                         : Theme::CodeModel_Error_TextMarkColor);
     }
 
     // Copy to clipboard action
     QVector<QAction *> actions;
     QAction *action = new QAction();
-    action->setIcon(QIcon::fromTheme("edit-copy", ::Utils::Icons::COPY.icon()));
+    action->setIcon(QIcon::fromTheme("edit-copy", Icons::COPY.icon()));
+    action->setToolTip(QApplication::translate("Clang Code Model Marks", "Copy to Clipboard"));
     QObject::connect(action, &QAction::triggered, [diagnostic]() {
         const QString text = ClangDiagnosticWidget::createText({diagnostic},
                                                                ClangDiagnosticWidget::InfoBar);
@@ -246,7 +252,9 @@ ClangTextMark::ClangTextMark(const FilePath &fileName,
     ProjectExplorer::Project *project = projectForCurrentEditor();
     if (project && isDiagnosticConfigChangable(project, diagnostic)) {
         action = new QAction();
-        action->setIcon(::Utils::Icons::BROKEN.icon());
+        action->setIcon(Icons::BROKEN.icon());
+        action->setToolTip(QApplication::translate("Clang Code Model Marks",
+                                                   "Disable Diagnostic in Current Project"));
         QObject::connect(action, &QAction::triggered, [diagnostic]() {
             disableDiagnosticInCurrentProjectConfig(diagnostic);
         });
@@ -258,7 +266,7 @@ ClangTextMark::ClangTextMark(const FilePath &fileName,
 
 void ClangTextMark::updateIcon(bool valid)
 {
-    using namespace ::Utils::Icons;
+    using namespace Icons;
     if (isWarningOrNote(m_diagnostic.severity))
         setIcon(valid ? CODEMODEL_WARNING.icon() : CODEMODEL_DISABLED_WARNING.icon());
     else
@@ -267,8 +275,13 @@ void ClangTextMark::updateIcon(bool valid)
 
 bool ClangTextMark::addToolTipContent(QLayout *target) const
 {
-    QWidget *widget = ClangDiagnosticWidget::createWidget({m_diagnostic},
-                                                          ClangDiagnosticWidget::ToolTip);
+    const auto canApplyFixIt = [diag = m_diagnostic, diagMgr = m_diagMgr, c = color()] {
+        return c != Utils::Theme::Color::IconsDisabledColor
+                && !diagMgr->diagnosticsInvalidated()
+                && diagMgr->diagnosticsWithFixIts().contains(diag);
+    };
+    QWidget *widget = ClangDiagnosticWidget::createWidget(
+                {m_diagnostic}, ClangDiagnosticWidget::ToolTip, canApplyFixIt, "libclang");
     target->addWidget(widget);
 
     return true;
@@ -280,6 +293,114 @@ void ClangTextMark::removedFromEditor()
     m_removedFromEditorHandler(this);
 }
 
+ClangBackEnd::DiagnosticSeverity convertSeverity(DiagnosticSeverity src)
+{
+    if (src == DiagnosticSeverity::Error)
+        return ClangBackEnd::DiagnosticSeverity::Error;
+    if (src == DiagnosticSeverity::Warning)
+        return ClangBackEnd::DiagnosticSeverity::Warning;
+    return ClangBackEnd::DiagnosticSeverity::Note;
+}
+
+ClangBackEnd::SourceRangeContainer convertRange(const FilePath &filePath, const Range &src)
+{
+    const ClangBackEnd::SourceLocationContainer start(filePath.toString(), src.start().line() + 1,
+                                                      src.start().character() + 1);
+    const ClangBackEnd::SourceLocationContainer end(filePath.toString(), src.end().line() + 1,
+                                                      src.end().character() + 1);
+    return ClangBackEnd::SourceRangeContainer(start, end);
+}
+
+ClangBackEnd::DiagnosticContainer convertDiagnostic(const ClangdDiagnostic &src,
+                                                    const FilePath &filePath)
+{
+    ClangBackEnd::DiagnosticContainer target;
+    target.ranges.append(convertRange(filePath, src.range()));
+    target.location = target.ranges.first().start;
+    target.text = src.message();
+    target.category = src.category();
+    if (src.severity())
+        target.severity = convertSeverity(*src.severity());
+    const Diagnostic::Code code = src.code().value_or(Diagnostic::Code());
+    const QString * const codeString = Utils::get_if<QString>(&code);
+    if (codeString && codeString->startsWith("-W"))
+        target.enableOption = *codeString;
+    for (const CodeAction &codeAction : src.codeActions().value_or(QList<CodeAction>())) {
+        const Utils::optional<WorkspaceEdit> edit = codeAction.edit();
+        if (!edit)
+            continue;
+        const Utils::optional<WorkspaceEdit::Changes> changes = edit->changes();
+        if (!changes)
+            continue;
+        for (auto it = changes->cbegin(); it != changes->cend(); ++it) {
+            for (const TextEdit &textEdit : it.value()) {
+                target.fixIts << ClangBackEnd::FixItContainer(textEdit.newText(),
+                        convertRange(it.key().toFilePath(), textEdit.range()));
+            }
+        }
+    }
+    return target;
+}
+
+ClangdTextMark::ClangdTextMark(const FilePath &filePath,
+                               const Diagnostic &diagnostic,
+                               const Client *client)
+    : TextEditor::TextMark(filePath, int(diagnostic.range().start().line() + 1), client->id())
+    , m_lspDiagnostic(diagnostic)
+    , m_diagnostic(convertDiagnostic(ClangdDiagnostic(diagnostic), filePath))
+    , m_client(client)
+{
+    setSettingsPage(CppEditor::Constants::CPP_CODE_MODEL_SETTINGS_ID);
+
+    const bool isError = diagnostic.severity()
+            && *diagnostic.severity() == DiagnosticSeverity::Error;
+    setDefaultToolTip(isError ? tr("Code Model Error") : tr("Code Model Warning"));
+    setPriority(isError ? TextEditor::TextMark::HighPriority
+                        : TextEditor::TextMark::NormalPriority);
+    setIcon(isError ? Icons::CODEMODEL_ERROR.icon() : Icons::CODEMODEL_WARNING.icon());
+    if (client->project()) {
+        setLineAnnotation(diagnostic.message());
+        setColor(isError ? Theme::CodeModel_Error_TextMarkColor
+                         : Theme::CodeModel_Warning_TextMarkColor);
+        ClangDiagnosticManager::addTask(m_diagnostic);
+    }
+
+    // Copy to clipboard action
+    QVector<QAction *> actions;
+    QAction *action = new QAction();
+    action->setIcon(QIcon::fromTheme("edit-copy", Icons::COPY.icon()));
+    action->setToolTip(tr("Clang Code Model Marks", "Copy to Clipboard"));
+    QObject::connect(action, &QAction::triggered, [diag = m_diagnostic]() {
+        const QString text = ClangDiagnosticWidget::createText({diag},
+                                                               ClangDiagnosticWidget::InfoBar);
+        QApplication::clipboard()->setText(text, QClipboard::Clipboard);
+    });
+    actions << action;
+
+    // Remove diagnostic warning action
+    ProjectExplorer::Project *project = projectForCurrentEditor();
+    if (project && isDiagnosticConfigChangable(project, m_diagnostic)) {
+        action = new QAction();
+        action->setIcon(Icons::BROKEN.icon());
+        action->setToolTip(tr("Disable Diagnostic in Current Project"));
+        QObject::connect(action, &QAction::triggered, [diag = m_diagnostic]() {
+            disableDiagnosticInCurrentProjectConfig(diag);
+        });
+        actions << action;
+    }
+
+    setActions(actions);
+}
+
+bool ClangdTextMark::addToolTipContent(QLayout *target) const
+{
+    const auto canApplyFixIt = [c = m_client, diag = m_lspDiagnostic, fp = fileName()] {
+        return c && c->reachable() && c->hasDiagnostic(DocumentUri::fromFilePath(fp), diag);
+    };
+    target->addWidget(ClangDiagnosticWidget::createWidget({m_diagnostic},
+        ClangDiagnosticWidget::ToolTip, canApplyFixIt, m_client ? m_client->name() : "clangd"));
+    return true;
+}
+
 } // namespace Internal
 } // namespace ClangCodeModel
-
