@@ -24,7 +24,6 @@
 ****************************************************************************/
 
 #include "gerritmodel.h"
-#include "../gitplugin.h"
 #include "../gitclient.h"
 
 #include <coreplugin/progressmanager/progressmanager.h>
@@ -32,8 +31,7 @@
 #include <vcsbase/vcsoutputwindow.h>
 
 #include <utils/algorithm.h>
-#include <utils/asconst.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -54,6 +52,7 @@
 
 enum { debug = 0 };
 
+using namespace Utils;
 using namespace VcsBase;
 
 namespace Gerrit {
@@ -69,7 +68,8 @@ QDebug operator<<(QDebug d, const GerritApproval &a)
 // Sort approvals by type and reviewer
 bool gerritApprovalLessThan(const GerritApproval &a1, const GerritApproval &a2)
 {
-    return a1.type.compare(a2.type) < 0 || a1.reviewer.fullName.compare(a2.reviewer.fullName) < 0;
+    const int compare = a1.type.compare(a2.type);
+    return compare == 0 ? a1.reviewer.fullName.compare(a2.reviewer.fullName) < 0 : compare < 0;
 }
 
 QDebug operator<<(QDebug d, const GerritPatchSet &p)
@@ -123,7 +123,10 @@ QString GerritPatchSet::approvalsToHtml() const
         str << a.reviewer.fullName;
         if (!a.reviewer.email.isEmpty())
             str << " <a href=\"mailto:" << a.reviewer.email << "\">" << a.reviewer.email << "</a>";
-        str << ": " << forcesign << a.approval << noforcesign;
+        str << ": ";
+        if (a.approval >= 0)
+            str << '+';
+        str << a.approval;
     }
     str << "</tr>\n";
     return result;
@@ -141,9 +144,9 @@ static inline void applyApproval(int approval, int *total)
 // by a type character followed by the approval level: "C: -2, S: 1"
 QString GerritPatchSet::approvalsColumn() const
 {
-    typedef QMap<QChar, int> TypeReviewMap;
-    typedef TypeReviewMap::iterator TypeReviewMapIterator;
-    typedef TypeReviewMap::const_iterator TypeReviewMapConstIterator;
+    using TypeReviewMap = QMap<QChar, int>;
+    using TypeReviewMapIterator = TypeReviewMap::iterator;
+    using TypeReviewMapConstIterator = TypeReviewMap::const_iterator;
 
     QString result;
     if (approvals.isEmpty())
@@ -165,7 +168,10 @@ QString GerritPatchSet::approvalsColumn() const
     for (TypeReviewMapConstIterator it = reviews.constBegin(); it != cend; ++it) {
         if (!result.isEmpty())
             str << ' ';
-        str << it.key() << ": " << forcesign << it.value() << noforcesign;
+        str << it.key() << ": ";
+        if (it.value() >= 0)
+            str << '+';
+        str << it.value();
     }
     return result;
 }
@@ -220,7 +226,8 @@ QString GerritChange::fullTitle() const
 // In theory, querying uses a continuation/limit protocol, but we assume
 // we will never reach a limit with those queries.
 
-class QueryContext : public QObject {
+class QueryContext : public QObject
+{
     Q_OBJECT
 public:
     QueryContext(const QString &query,
@@ -228,7 +235,7 @@ public:
                  const GerritServer &server,
                  QObject *parent = nullptr);
 
-    ~QueryContext();
+    ~QueryContext() override;
     void start();
     void terminate();
 
@@ -239,14 +246,14 @@ signals:
 
 private:
     void processError(QProcess::ProcessError);
-    void processFinished(int exitCode, QProcess::ExitStatus);
+    void processFinished();
     void timeout();
 
     void errorTermination(const QString &msg);
 
-    QProcess m_process;
+    QtcProcess m_process;
     QTimer m_timer;
-    QString m_binary;
+    FilePath m_binary;
     QByteArray m_output;
     QString m_error;
     QFutureInterface<void> m_progress;
@@ -277,20 +284,19 @@ QueryContext::QueryContext(const QString &query,
                 + "&o=CURRENT_REVISION&o=DETAILED_LABELS&o=DETAILED_ACCOUNTS";
         m_arguments = server.curlArguments() << url;
     }
-    connect(&m_process, &QProcess::readyReadStandardError, this, [this] {
+    connect(&m_process, &QtcProcess::readyReadStandardError, this, [this] {
         const QString text = QString::fromLocal8Bit(m_process.readAllStandardError());
         VcsOutputWindow::appendError(text);
         m_error.append(text);
     });
-    connect(&m_process, &QProcess::readyReadStandardOutput, this, [this] {
+    connect(&m_process, &QtcProcess::readyReadStandardOutput, this, [this] {
         m_output.append(m_process.readAllStandardOutput());
     });
-    connect(&m_process, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
-            this, &QueryContext::processFinished);
-    connect(&m_process, &QProcess::errorOccurred, this, &QueryContext::processError);
+    connect(&m_process, &QtcProcess::finished, this, &QueryContext::processFinished);
+    connect(&m_process, &QtcProcess::errorOccurred, this, &QueryContext::processError);
     connect(&m_watcher, &QFutureWatcherBase::canceled, this, &QueryContext::terminate);
     m_watcher.setFuture(m_progress.future());
-    m_process.setProcessEnvironment(Git::Internal::GitPlugin::client()->processEnvironment());
+    m_process.setEnvironment(Git::Internal::GitClient::instance()->processEnvironment());
     m_progress.setProgressRange(0, 1);
 
     m_timer.setInterval(timeOutMS);
@@ -315,11 +321,10 @@ void QueryContext::start()
     fp->setKeepOnFinish(Core::FutureProgress::HideOnFinish);
     m_progress.reportStarted();
     // Order: synchronous call to error handling if something goes wrong.
-    VcsOutputWindow::appendCommand(
-                m_process.workingDirectory(), Utils::FileName::fromString(m_binary), m_arguments);
+    VcsOutputWindow::appendCommand(m_process.workingDirectory(), {m_binary, m_arguments});
     m_timer.start();
-    m_process.start(m_binary, m_arguments);
-    m_process.closeWriteChannel();
+    m_process.setCommand({m_binary, m_arguments});
+    m_process.start();
 }
 
 void QueryContext::errorTermination(const QString &msg)
@@ -333,28 +338,28 @@ void QueryContext::errorTermination(const QString &msg)
 
 void QueryContext::terminate()
 {
-    Utils::SynchronousProcess::stopProcess(m_process);
+    m_process.stopProcess();
 }
 
 void QueryContext::processError(QProcess::ProcessError e)
 {
-    const QString msg = tr("Error running %1: %2").arg(m_binary, m_process.errorString());
+    const QString msg = tr("Error running %1: %2").arg(m_binary.toUserOutput(), m_process.errorString());
     if (e == QProcess::FailedToStart)
         errorTermination(msg);
     else
         VcsOutputWindow::appendError(msg);
 }
 
-void QueryContext::processFinished(int exitCode, QProcess::ExitStatus es)
+void QueryContext::processFinished()
 {
     if (m_timer.isActive())
         m_timer.stop();
     emit errorText(m_error);
-    if (es != QProcess::NormalExit) {
-        errorTermination(tr("%1 crashed.").arg(m_binary));
+    if (m_process.exitStatus() != QProcess::NormalExit) {
+        errorTermination(tr("%1 crashed.").arg(m_binary.toUserOutput()));
         return;
-    } else if (exitCode) {
-        errorTermination(tr("%1 returned %2.").arg(m_binary).arg(exitCode));
+    } else if (m_process.exitCode()) {
+        errorTermination(tr("%1 returned %2.").arg(m_binary.toUserOutput()).arg(m_process.exitCode()));
         return;
     }
     emit resultRetrieved(m_output);
@@ -377,8 +382,7 @@ void QueryContext::timeout()
                     arg(timeOutMS / 1000), QMessageBox::NoButton, parent);
     QPushButton *terminateButton = box.addButton(tr("Terminate"), QMessageBox::YesRole);
     box.addButton(tr("Keep Running"), QMessageBox::NoRole);
-    connect(&m_process, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
-            &box, &QDialog::reject);
+    connect(&m_process, &QtcProcess::finished, &box, &QDialog::reject);
     box.exec();
     if (m_process.state() != QProcess::Running)
         return;
@@ -399,8 +403,7 @@ GerritModel::GerritModel(const QSharedPointer<GerritParameters> &p, QObject *par
     setHorizontalHeaderLabels(headers);
 }
 
-GerritModel::~GerritModel()
-{ }
+GerritModel::~GerritModel() = default;
 
 QVariant GerritModel::data(const QModelIndex &index, int role) const
 {
@@ -464,7 +467,7 @@ QString GerritModel::toHtml(const QModelIndex& index) const
         << dependencyHtml(dependsOnHeader, c->dependsOnNumber, serverPrefix)
         << dependencyHtml(neededByHeader, c->neededByNumber, serverPrefix)
         << "<tr><td>" << statusHeader << "</td><td>" << c->status
-        << ", " << c->lastUpdated.toString(Qt::DefaultLocaleShortDate) << "</td></tr>"
+        << ", " << QLocale::system().toString(c->lastUpdated, QLocale::ShortFormat) << "</td></tr>"
         << "<tr><td>" << patchSetHeader << "</td><td>" << "</td></tr>" << c->currentPatchSet.patchSetNumber << "</td></tr>"
         << c->currentPatchSet.approvalsToHtml()
         << "<tr><td>" << urlHeader << "</td><td><a href=\"" << c->url << "\">" << c->url << "</a></td></tr>"
@@ -481,19 +484,19 @@ static QStandardItem *numberSearchRecursion(QStandardItem *item, int number)
         if (QStandardItem *i = numberSearchRecursion(item->child(r, 0), number))
             return i;
     }
-    return 0;
+    return nullptr;
 }
 
 QStandardItem *GerritModel::itemForNumber(int number) const
 {
     if (!number)
-        return 0;
+        return nullptr;
     const int numRows = rowCount();
     for (int r = 0; r < numRows; ++r) {
         if (QStandardItem *i = numberSearchRecursion(item(r, 0), number))
             return i;
     }
-    return 0;
+    return nullptr;
 }
 
 void GerritModel::refresh(const QSharedPointer<GerritServer> &server, const QString &query)
@@ -599,7 +602,7 @@ static GerritChangePtr parseSshOutput(const QJsonObject &object)
     change->branch = object.value("branch").toString();
     change->status =  object.value("status").toString();
     if (const int timeT = object.value("lastUpdated").toInt())
-        change->lastUpdated = QDateTime::fromTime_t(uint(timeT));
+        change->lastUpdated = QDateTime::fromSecsSinceEpoch(timeT);
     // Read out dependencies
     const QJsonValue dependsOnValue = object.value("dependsOn");
     if (dependsOnValue.isArray()) {
@@ -844,9 +847,9 @@ QList<QStandardItem *> GerritModel::changeToRow(const GerritChangePtr &c) const
 {
     QList<QStandardItem *> row;
     const QVariant filterV = QVariant(c->filterString());
-    const QVariant changeV = qVariantFromValue(c);
+    const QVariant changeV = QVariant::fromValue(c);
     for (int i = 0; i < GerritModel::ColumnCount; ++i) {
-        QStandardItem *item = new QStandardItem;
+        auto item = new QStandardItem;
         item->setData(changeV, GerritModel::GerritChangeRole);
         item->setData(filterV, GerritModel::FilterRole);
         item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
@@ -857,8 +860,8 @@ QList<QStandardItem *> GerritModel::changeToRow(const GerritChangePtr &c) const
     row[OwnerColumn]->setText(c->owner.fullName);
     // Shorten columns: Display time if it is today, else date
     const QString dateString = c->lastUpdated.date() == QDate::currentDate() ?
-                c->lastUpdated.time().toString(Qt::SystemLocaleShortDate) :
-                c->lastUpdated.date().toString(Qt::SystemLocaleShortDate);
+                QLocale::system().toString(c->lastUpdated.time(), QLocale::ShortFormat) :
+                QLocale::system().toString(c->lastUpdated.date(), QLocale::ShortFormat);
     row[DateColumn]->setData(dateString, Qt::DisplayRole);
     row[DateColumn]->setData(c->lastUpdated, SortRole);
 
@@ -933,7 +936,7 @@ void GerritModel::resultRetrieved(const QByteArray &output)
     std::stable_sort(changes.begin(), changes.end(), gerritChangeLessThan);
     numberIndexHash.clear();
 
-    for (const GerritChangePtr &c : Utils::asConst(changes)) {
+    for (const GerritChangePtr &c : qAsConst(changes)) {
         // Avoid duplicate entries for example in the (unlikely)
         // case people do self-reviews.
         if (!itemForNumber(c->number)) {
