@@ -35,12 +35,15 @@
 #include "toolchain.h"
 #include "toolchainmanager.h"
 
-#include <coreplugin/icore.h>
+#include <docker/dockerconstants.h>
+
 #include <ssh/sshconnection.h>
+
 #include <utils/algorithm.h>
 #include <utils/elidinglabel.h>
 #include <utils/environment.h>
 #include <utils/environmentdialog.h>
+#include <utils/layoutbuilder.h>
 #include <utils/macroexpander.h>
 #include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
@@ -48,14 +51,14 @@
 
 #include <QCheckBox>
 #include <QComboBox>
-#include <QDir>
-#include <QFileInfo>
 #include <QFontMetrics>
 #include <QGridLayout>
 #include <QLabel>
 #include <QPushButton>
 #include <QVBoxLayout>
 
+using namespace Utils;
+
 namespace ProjectExplorer {
 
 const char KITINFORMATION_ID_V1[] = "PE.Profile.ToolChain";
@@ -74,7 +77,7 @@ class SysRootKitAspectWidget : public KitAspectWidget
 public:
     SysRootKitAspectWidget(Kit *k, const KitAspect *ki) : KitAspectWidget(k, ki)
     {
-        m_chooser = new Utils::PathChooser;
+        m_chooser = createSubWidget<PathChooser>();
         m_chooser->setExpectedKind(Utils::PathChooser::ExistingDirectory);
         m_chooser->setHistoryCompleter(QLatin1String("PE.SysRoot.History"));
         m_chooser->setFilePath(SysRootKitAspect::sysRoot(k));
@@ -86,8 +89,12 @@ public:
 
 private:
     void makeReadOnly() override { m_chooser->setReadOnly(true); }
-    QWidget *buttonWidget() const override { return m_chooser->buttonAtIndex(0); }
-    QWidget *mainWidget() const override { return m_chooser->lineEdit(); }
+
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_chooser);
+        builder.addItem(Layouting::Span(2, m_chooser));
+    }
 
     void refresh() override
     {
@@ -124,18 +131,16 @@ Tasks SysRootKitAspect::validate(const Kit *k) const
     if (dir.isEmpty())
         return result;
 
-    if (dir.toString().startsWith("target:") || dir.toString().startsWith("remote:"))
+    if (dir.startsWith("target:") || dir.startsWith("remote:"))
         return result;
 
-    const QFileInfo fi = dir.toFileInfo();
-
-    if (!fi.exists()) {
+    if (!dir.exists()) {
         result << BuildSystemTask(Task::Warning,
                     tr("Sys Root \"%1\" does not exist in the file system.").arg(dir.toUserOutput()));
-    } else if (!fi.isDir()) {
+    } else if (!dir.isDir()) {
         result << BuildSystemTask(Task::Warning,
                     tr("Sys Root \"%1\" is not a directory.").arg(dir.toUserOutput()));
-    } else if (QDir(dir.toString()).entryList(QDir::AllEntries | QDir::NoDotAndDotDot).isEmpty()) {
+    } else if (dir.dirEntries(QDir::AllEntries | QDir::NoDotAndDotDot).isEmpty()) {
         result << BuildSystemTask(Task::Warning,
                     tr("Sys Root \"%1\" is empty.").arg(dir.toUserOutput()));
     }
@@ -158,8 +163,8 @@ void SysRootKitAspect::addToMacroExpander(Kit *kit, Utils::MacroExpander *expand
 {
     QTC_ASSERT(kit, return);
 
-    expander->registerFileVariables("SysRoot", tr("Sys Root"), [kit]() -> QString {
-        return SysRootKitAspect::sysRoot(kit).toString();
+    expander->registerFileVariables("SysRoot", tr("Sys Root"), [kit] {
+        return SysRootKitAspect::sysRoot(kit);
     });
 }
 
@@ -214,7 +219,7 @@ class ToolChainKitAspectWidget final : public KitAspectWidget
 public:
     ToolChainKitAspectWidget(Kit *k, const KitAspect *ki) : KitAspectWidget(k, ki)
     {
-        m_mainWidget = new QWidget;
+        m_mainWidget = createSubWidget<QWidget>();
         m_mainWidget->setContentsMargins(0, 0, 0, 0);
 
         auto layout = new QGridLayout(m_mainWidget);
@@ -244,10 +249,7 @@ public:
 
         refresh();
 
-        m_manageButton = new QPushButton(KitAspectWidget::msgManage());
-        m_manageButton->setContentsMargins(0, 0, 0, 0);
-        connect(m_manageButton, &QAbstractButton::clicked,
-                this, &ToolChainKitAspectWidget::manageToolChains);
+        m_manageButton = createManageButton(Constants::TOOLCHAIN_SETTINGS_PAGE_ID);
     }
 
     ~ToolChainKitAspectWidget() override
@@ -257,8 +259,12 @@ public:
     }
 
 private:
-    QWidget *mainWidget() const override { return m_mainWidget; }
-    QWidget *buttonWidget() const override { return m_manageButton; }
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_mainWidget);
+        builder.addItem(m_mainWidget);
+        builder.addItem(m_manageButton);
+    }
 
     void refresh() override
     {
@@ -289,11 +295,6 @@ private:
         }
     }
 
-    void manageToolChains()
-    {
-        Core::ICore::showOptionsDialog(Constants::TOOLCHAIN_SETTINGS_PAGE_ID, buttonWidget());
-    }
-
     void currentToolChainChanged(Utils::Id language, int idx)
     {
         if (m_ignoreChanges || idx < 0)
@@ -319,7 +320,7 @@ private:
     }
 
     QWidget *m_mainWidget = nullptr;
-    QPushButton *m_manageButton = nullptr;
+    QWidget *m_manageButton = nullptr;
     QHash<Utils::Id, QComboBox *> m_languageComboboxMap;
     bool m_ignoreChanges = false;
     bool m_isReadOnly = false;
@@ -477,14 +478,17 @@ void ToolChainKitAspect::setup(Kit *k)
     QTC_ASSERT(k, return);
 
     QVariantMap value = k->value(id()).toMap();
+    bool lockToolchains = k->isSdkProvided() && !value.isEmpty();
     if (value.empty())
         value = defaultToolChainValue().toMap();
 
     for (auto i = value.constBegin(); i != value.constEnd(); ++i) {
         Utils::Id l = findLanguage(i.key());
 
-        if (!l.isValid())
+        if (!l.isValid()) {
+            lockToolchains = false;
             continue;
+        }
 
         const QByteArray id = i.value().toByteArray();
         ToolChain *tc = ToolChainManager::findToolChain(id);
@@ -492,15 +496,24 @@ void ToolChainKitAspect::setup(Kit *k)
             continue;
 
         // ID is not found: Might be an ABI string...
+        lockToolchains = false;
         const QString abi = QString::fromUtf8(id);
-        tc = ToolChainManager::toolChain([abi, l](const ToolChain *t) {
-                 return t->targetAbi().toString() == abi && t->language() == l;
-             });
-        if (tc)
-            setToolChain(k, tc);
+        QList<ToolChain *> possibleTcs = ToolChainManager::toolChains(
+            [abi, l](const ToolChain *t) {
+                return t->targetAbi().toString() == abi && t->language() == l;
+            });
+        ToolChain *bestTc = nullptr;
+        for (ToolChain *tc : possibleTcs) {
+            if (!bestTc || tc->priority() > bestTc->priority())
+                bestTc = tc;
+        }
+        if (bestTc)
+            setToolChain(k, bestTc);
         else
             clearToolChain(k, l);
     }
+
+    k->setSticky(id(), lockToolchains);
 }
 
 KitAspectWidget *ToolChainKitAspect::createConfigWidget(Kit *k) const
@@ -521,7 +534,7 @@ KitAspect::ItemList ToolChainKitAspect::toUserOutput(const Kit *k) const
     return {{tr("Compiler"), tc ? tc->displayName() : tr("None")}};
 }
 
-void ToolChainKitAspect::addToEnvironment(const Kit *k, Utils::Environment &env) const
+void ToolChainKitAspect::addToBuildEnvironment(const Kit *k, Utils::Environment &env) const
 {
     ToolChain *tc = cxxToolChain(k);
     if (tc)
@@ -542,9 +555,10 @@ void ToolChainKitAspect::addToMacroExpander(Kit *kit, Utils::MacroExpander *expa
     expander->registerVariable("Compiler:Executable", tr("Path to the compiler executable"),
                                [kit] {
                                    const ToolChain *tc = cxxToolChain(kit);
-                                   return tc ? tc->compilerCommand().toString() : QString();
+                                   return tc ? tc->compilerCommand().path() : QString();
                                });
 
+    // After 4.2
     expander->registerPrefix("Compiler:Name", tr("Compiler for different languages"),
                              [kit](const QString &ls) {
                                  const ToolChain *tc = toolChain(kit, findLanguage(ls));
@@ -553,7 +567,7 @@ void ToolChainKitAspect::addToMacroExpander(Kit *kit, Utils::MacroExpander *expa
     expander->registerPrefix("Compiler:Executable", tr("Compiler executable for different languages"),
                              [kit](const QString &ls) {
                                  const ToolChain *tc = toolChain(kit, findLanguage(ls));
-                                 return tc ? tc->compilerCommand().toString() : QString();
+                                 return tc ? tc->compilerCommand().path() : QString();
                              });
 }
 
@@ -610,11 +624,10 @@ QList<ToolChain *> ToolChainKitAspect::toolChains(const Kit *k)
 
     const QVariantMap value = k->value(ToolChainKitAspect::id()).toMap();
     const QList<ToolChain *> tcList
-            = Utils::transform<QList>(ToolChainManager::allLanguages(),
-                               [&value](Utils::Id l) -> ToolChain * {
-                                   return ToolChainManager::findToolChain(value.value(l.toString()).toByteArray());
-                               });
-    return Utils::filtered(tcList, [](ToolChain *tc) { return tc; });
+            = transform<QList>(ToolChainManager::allLanguages(), [&value](Id l) {
+                return ToolChainManager::findToolChain(value.value(l.toString()).toByteArray());
+            });
+    return filtered(tcList, [](ToolChain *tc) { return tc; });
 }
 
 void ToolChainKitAspect::setToolChain(Kit *k, ToolChain *tc)
@@ -648,7 +661,7 @@ void ToolChainKitAspect::setAllToolChainsToMatch(Kit *k, ToolChain *tc)
     QVariantMap result = k->value(ToolChainKitAspect::id()).toMap();
     result.insert(tc->language().toString(), tc->id());
 
-    for (Utils::Id l : ToolChainManager::allLanguages()) {
+    for (const Id l : ToolChainManager::allLanguages()) {
         if (l == tc->language())
             continue;
 
@@ -676,7 +689,7 @@ void ToolChainKitAspect::setAllToolChainsToMatch(Kit *k, ToolChain *tc)
     k->setValue(id(), result);
 }
 
-void ToolChainKitAspect::clearToolChain(Kit *k, Utils::Id language)
+void ToolChainKitAspect::clearToolChain(Kit *k, Id language)
 {
     QTC_ASSERT(language.isValid(), return);
     QTC_ASSERT(k, return);
@@ -694,7 +707,7 @@ Abi ToolChainKitAspect::targetAbi(const Kit *k)
     QHash<Abi, int> abiCount;
     foreach (ToolChain *tc, tcList) {
         Abi ta = tc->targetAbi();
-        if (tc->language() == Utils::Id(Constants::CXX_LANGUAGE_ID))
+        if (tc->language() == Id(Constants::CXX_LANGUAGE_ID))
             cxxAbi = tc->targetAbi();
         abiCount[ta] = (abiCount.contains(ta) ? abiCount[ta] + 1 : 1);
     }
@@ -760,7 +773,7 @@ class DeviceTypeKitAspectWidget final : public KitAspectWidget
 
 public:
     DeviceTypeKitAspectWidget(Kit *workingCopy, const KitAspect *ki)
-        : KitAspectWidget(workingCopy, ki), m_comboBox(new QComboBox)
+        : KitAspectWidget(workingCopy, ki), m_comboBox(createSubWidget<QComboBox>())
     {
         for (IDeviceFactory *factory : IDeviceFactory::allDeviceFactories())
             m_comboBox->addItem(factory->displayName(), factory->deviceType().toSetting());
@@ -773,12 +786,17 @@ public:
     ~DeviceTypeKitAspectWidget() override { delete m_comboBox; }
 
 private:
-    QWidget *mainWidget() const override { return m_comboBox; }
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_comboBox);
+        builder.addItem(m_comboBox);
+    }
+
     void makeReadOnly() override { m_comboBox->setEnabled(false); }
 
     void refresh() override
     {
-        Utils::Id devType = DeviceTypeKitAspect::deviceTypeId(m_kit);
+        Id devType = DeviceTypeKitAspect::deviceTypeId(m_kit);
         if (!devType.isValid())
             m_comboBox->setCurrentIndex(-1);
         for (int i = 0; i < m_comboBox->count(); ++i) {
@@ -791,7 +809,7 @@ private:
 
     void currentTypeChanged(int idx)
     {
-        Utils::Id type = idx < 0 ? Utils::Id() : Utils::Id::fromSetting(m_comboBox->itemData(idx));
+        Id type = idx < 0 ? Id() : Id::fromSetting(m_comboBox->itemData(idx));
         DeviceTypeKitAspect::setDeviceTypeId(m_kit, type);
     }
 
@@ -830,7 +848,7 @@ KitAspectWidget *DeviceTypeKitAspect::createConfigWidget(Kit *k) const
 KitAspect::ItemList DeviceTypeKitAspect::toUserOutput(const Kit *k) const
 {
     QTC_ASSERT(k, return {});
-    Utils::Id type = deviceTypeId(k);
+    Id type = deviceTypeId(k);
     QString typeDisplayName = tr("Unknown device type");
     if (type.isValid()) {
         if (IDeviceFactory *factory = IDeviceFactory::find(type))
@@ -839,33 +857,33 @@ KitAspect::ItemList DeviceTypeKitAspect::toUserOutput(const Kit *k) const
     return {{tr("Device type"), typeDisplayName}};
 }
 
-const Utils::Id DeviceTypeKitAspect::id()
+const Id DeviceTypeKitAspect::id()
 {
     return "PE.Profile.DeviceType";
 }
 
-const Utils::Id DeviceTypeKitAspect::deviceTypeId(const Kit *k)
+const Id DeviceTypeKitAspect::deviceTypeId(const Kit *k)
 {
-    return k ? Utils::Id::fromSetting(k->value(DeviceTypeKitAspect::id())) : Utils::Id();
+    return k ? Id::fromSetting(k->value(DeviceTypeKitAspect::id())) : Id();
 }
 
-void DeviceTypeKitAspect::setDeviceTypeId(Kit *k, Utils::Id type)
+void DeviceTypeKitAspect::setDeviceTypeId(Kit *k, Id type)
 {
     QTC_ASSERT(k, return);
     k->setValue(DeviceTypeKitAspect::id(), type.toSetting());
 }
 
-QSet<Utils::Id> DeviceTypeKitAspect::supportedPlatforms(const Kit *k) const
+QSet<Id> DeviceTypeKitAspect::supportedPlatforms(const Kit *k) const
 {
     return {deviceTypeId(k)};
 }
 
-QSet<Utils::Id> DeviceTypeKitAspect::availableFeatures(const Kit *k) const
+QSet<Id> DeviceTypeKitAspect::availableFeatures(const Kit *k) const
 {
-    Utils::Id id = DeviceTypeKitAspect::deviceTypeId(k);
+    Id id = DeviceTypeKitAspect::deviceTypeId(k);
     if (id.isValid())
         return {id.withPrefix("DeviceType.")};
-    return QSet<Utils::Id>();
+    return {};
 }
 
 // --------------------------------------------------------------------------
@@ -878,12 +896,13 @@ class DeviceKitAspectWidget final : public KitAspectWidget
 
 public:
     DeviceKitAspectWidget(Kit *workingCopy, const KitAspect *ki)
-        : KitAspectWidget(workingCopy, ki), m_comboBox(new QComboBox),
+        : KitAspectWidget(workingCopy, ki),
+        m_comboBox(createSubWidget<QComboBox>()),
         m_model(new DeviceManagerModel(DeviceManager::instance()))
     {
         m_comboBox->setSizePolicy(QSizePolicy::Ignored, m_comboBox->sizePolicy().verticalPolicy());
         m_comboBox->setModel(m_model);
-        m_manageButton = new QPushButton(KitAspectWidget::msgManage());
+        m_manageButton = createManageButton(Constants::DEVICE_SETTINGS_PAGE_ID);
         refresh();
         m_comboBox->setToolTip(ki->description());
 
@@ -893,8 +912,6 @@ public:
                 this, &DeviceKitAspectWidget::modelReset);
         connect(m_comboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
                 this, &DeviceKitAspectWidget::currentDeviceChanged);
-        connect(m_manageButton, &QAbstractButton::clicked,
-                this, &DeviceKitAspectWidget::manageDevices);
     }
 
     ~DeviceKitAspectWidget() override
@@ -905,8 +922,13 @@ public:
     }
 
 private:
-    QWidget *mainWidget() const override { return m_comboBox; }
-    QWidget *buttonWidget() const override { return m_manageButton; }
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_comboBox);
+        builder.addItem(m_comboBox);
+        builder.addItem(m_manageButton);
+    }
+
     void makeReadOnly() override { m_comboBox->setEnabled(false); }
 
     void refresh() override
@@ -915,11 +937,6 @@ private:
         m_comboBox->setCurrentIndex(m_model->indexOf(DeviceKitAspect::device(m_kit)));
     }
 
-    void manageDevices()
-    {
-        Core::ICore::showOptionsDialog(Constants::DEVICE_SETTINGS_PAGE_ID, buttonWidget());
-    }
-
     void modelAboutToReset()
     {
         m_selectedId = m_model->deviceId(m_comboBox->currentIndex());
@@ -941,7 +958,7 @@ private:
 
     bool m_ignoreChange = false;
     QComboBox *m_comboBox;
-    QPushButton *m_manageButton;
+    QWidget *m_manageButton;
     DeviceManagerModel *m_model;
     Utils::Id m_selectedId;
 };
@@ -985,6 +1002,9 @@ Tasks DeviceKitAspect::validate(const Kit *k) const
     else if (!dev->isCompatibleWith(k))
         result.append(BuildSystemTask(Task::Error, tr("Device is incompatible with this kit.")));
 
+    if (dev)
+        result.append(dev->validate());
+
     return result;
 }
 
@@ -1047,7 +1067,7 @@ void DeviceKitAspect::addToMacroExpander(Kit *kit, Utils::MacroExpander *expande
     expander->registerVariable("Device:KeyFile", tr("Private key file"),
         [kit]() -> QString {
             const IDevice::ConstPtr device = DeviceKitAspect::device(kit);
-            return device ? device->sshParameters().privateKeyFile : QString();
+            return device ? device->sshParameters().privateKeyFile.toString() : QString();
     });
     expander->registerVariable("Device:Name", tr("Device name"),
         [kit]() -> QString {
@@ -1119,6 +1139,238 @@ void DeviceKitAspect::devicesChanged()
         setup(k); // Set default device if necessary
 }
 
+// --------------------------------------------------------------------------
+// BuildDeviceKitAspect:
+// --------------------------------------------------------------------------
+namespace Internal {
+class BuildDeviceKitAspectWidget final : public KitAspectWidget
+{
+    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::BuildDeviceKitAspect)
+
+public:
+    BuildDeviceKitAspectWidget(Kit *workingCopy, const KitAspect *ki)
+        : KitAspectWidget(workingCopy, ki),
+        m_comboBox(createSubWidget<QComboBox>()),
+        m_model(new DeviceManagerModel(DeviceManager::instance()))
+    {
+        m_comboBox->setSizePolicy(QSizePolicy::Ignored, m_comboBox->sizePolicy().verticalPolicy());
+        m_comboBox->setModel(m_model);
+        m_manageButton = createManageButton(Constants::DEVICE_SETTINGS_PAGE_ID);
+        refresh();
+        m_comboBox->setToolTip(ki->description());
+
+        connect(m_model, &QAbstractItemModel::modelAboutToBeReset,
+                this, &BuildDeviceKitAspectWidget::modelAboutToReset);
+        connect(m_model, &QAbstractItemModel::modelReset,
+                this, &BuildDeviceKitAspectWidget::modelReset);
+        connect(m_comboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
+                this, &BuildDeviceKitAspectWidget::currentDeviceChanged);
+    }
+
+    ~BuildDeviceKitAspectWidget() override
+    {
+        delete m_comboBox;
+        delete m_model;
+        delete m_manageButton;
+    }
+
+private:
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_comboBox);
+        builder.addItem(m_comboBox);
+        builder.addItem(m_manageButton);
+    }
+
+    void makeReadOnly() override { m_comboBox->setEnabled(false); }
+
+    void refresh() override
+    {
+        QList<Utils::Id> blackList;
+        const DeviceManager *dm = DeviceManager::instance();
+        for (int i = 0; i < dm->deviceCount(); ++i) {
+            IDevice::ConstPtr device = dm->deviceAt(i);
+            if (!(device->type() == Constants::DESKTOP_DEVICE_TYPE
+                    || device->type() == Docker::Constants::DOCKER_DEVICE_TYPE))
+                blackList.append(device->id());
+        }
+
+        m_model->setFilter(blackList);
+        m_comboBox->setCurrentIndex(m_model->indexOf(BuildDeviceKitAspect::device(m_kit)));
+    }
+
+    void modelAboutToReset()
+    {
+        m_selectedId = m_model->deviceId(m_comboBox->currentIndex());
+        m_ignoreChange = true;
+    }
+
+    void modelReset()
+    {
+        m_comboBox->setCurrentIndex(m_model->indexForId(m_selectedId));
+        m_ignoreChange = false;
+    }
+
+    void currentDeviceChanged()
+    {
+        if (m_ignoreChange)
+            return;
+        BuildDeviceKitAspect::setDeviceId(m_kit, m_model->deviceId(m_comboBox->currentIndex()));
+    }
+
+    bool m_ignoreChange = false;
+    QComboBox *m_comboBox;
+    QWidget *m_manageButton;
+    DeviceManagerModel *m_model;
+    Id m_selectedId;
+};
+} // namespace Internal
+
+BuildDeviceKitAspect::BuildDeviceKitAspect()
+{
+    setObjectName("BuildDeviceInformation");
+    setId(BuildDeviceKitAspect::id());
+    setDisplayName(tr("Build device"));
+    setDescription(tr("The device used to build applications on."));
+    setPriority(31900);
+
+    connect(KitManager::instance(), &KitManager::kitsLoaded,
+            this, &BuildDeviceKitAspect::kitsWereLoaded);
+}
+
+QVariant BuildDeviceKitAspect::defaultValue(const Kit *k) const
+{
+    Q_UNUSED(k);
+    IDevice::ConstPtr defaultDevice = DeviceManager::defaultDesktopDevice();
+    return defaultDevice->id().toString();
+}
+
+Tasks BuildDeviceKitAspect::validate(const Kit *k) const
+{
+    IDevice::ConstPtr dev = BuildDeviceKitAspect::device(k);
+    Tasks result;
+    if (dev.isNull())
+        result.append(BuildSystemTask(Task::Warning, tr("No build device set.")));
+
+    return result;
+}
+
+KitAspectWidget *BuildDeviceKitAspect::createConfigWidget(Kit *k) const
+{
+    QTC_ASSERT(k, return nullptr);
+    return new Internal::BuildDeviceKitAspectWidget(k, this);
+}
+
+QString BuildDeviceKitAspect::displayNamePostfix(const Kit *k) const
+{
+    IDevice::ConstPtr dev = device(k);
+    return dev.isNull() ? QString() : dev->displayName();
+}
+
+KitAspect::ItemList BuildDeviceKitAspect::toUserOutput(const Kit *k) const
+{
+    IDevice::ConstPtr dev = device(k);
+    return {{tr("Build device"), dev.isNull() ? tr("Unconfigured") : dev->displayName()}};
+}
+
+void BuildDeviceKitAspect::addToMacroExpander(Kit *kit, Utils::MacroExpander *expander) const
+{
+    QTC_ASSERT(kit, return);
+    expander->registerVariable("BuildDevice:HostAddress", tr("Build host address"),
+        [kit]() -> QString {
+            const IDevice::ConstPtr device = BuildDeviceKitAspect::device(kit);
+            return device ? device->sshParameters().host() : QString();
+    });
+    expander->registerVariable("BuildDevice:SshPort", tr("Build SSH port"),
+        [kit]() -> QString {
+            const IDevice::ConstPtr device = BuildDeviceKitAspect::device(kit);
+            return device ? QString::number(device->sshParameters().port()) : QString();
+    });
+    expander->registerVariable("BuildDevice:UserName", tr("Build user name"),
+        [kit]() -> QString {
+            const IDevice::ConstPtr device = BuildDeviceKitAspect::device(kit);
+            return device ? device->sshParameters().userName() : QString();
+    });
+    expander->registerVariable("BuildDevice:KeyFile", tr("Build private key file"),
+        [kit]() -> QString {
+            const IDevice::ConstPtr device = BuildDeviceKitAspect::device(kit);
+            return device ? device->sshParameters().privateKeyFile.toString() : QString();
+    });
+    expander->registerVariable("BuildDevice:Name", tr("Build device name"),
+        [kit]() -> QString {
+            const IDevice::ConstPtr device = BuildDeviceKitAspect::device(kit);
+            return device ? device->displayName() : QString();
+    });
+}
+
+Utils::Id BuildDeviceKitAspect::id()
+{
+    return "PE.Profile.BuildDevice";
+}
+
+IDevice::ConstPtr BuildDeviceKitAspect::device(const Kit *k)
+{
+    QTC_ASSERT(DeviceManager::instance()->isLoaded(), return IDevice::ConstPtr());
+    IDevice::ConstPtr dev = DeviceManager::instance()->find(deviceId(k));
+    // Use the "run" device as fallback if no build device is present.
+    // FIXME: Think about whether this shouldn't be the other way round.
+    if (!dev)
+        dev = DeviceKitAspect::device(k);
+    return dev;
+}
+
+Utils::Id BuildDeviceKitAspect::deviceId(const Kit *k)
+{
+    return k ? Utils::Id::fromSetting(k->value(BuildDeviceKitAspect::id())) : Utils::Id();
+}
+
+void BuildDeviceKitAspect::setDevice(Kit *k, IDevice::ConstPtr dev)
+{
+    setDeviceId(k, dev ? dev->id() : Utils::Id());
+}
+
+void BuildDeviceKitAspect::setDeviceId(Kit *k, Utils::Id id)
+{
+    QTC_ASSERT(k, return);
+    k->setValue(BuildDeviceKitAspect::id(), id.toSetting());
+}
+
+void BuildDeviceKitAspect::kitsWereLoaded()
+{
+    foreach (Kit *k, KitManager::kits())
+        fix(k);
+
+    DeviceManager *dm = DeviceManager::instance();
+    connect(dm, &DeviceManager::deviceListReplaced, this, &BuildDeviceKitAspect::devicesChanged);
+    connect(dm, &DeviceManager::deviceAdded, this, &BuildDeviceKitAspect::devicesChanged);
+    connect(dm, &DeviceManager::deviceRemoved, this, &BuildDeviceKitAspect::devicesChanged);
+    connect(dm, &DeviceManager::deviceUpdated, this, &BuildDeviceKitAspect::deviceUpdated);
+
+    connect(KitManager::instance(), &KitManager::kitUpdated,
+            this, &BuildDeviceKitAspect::kitUpdated);
+    connect(KitManager::instance(), &KitManager::unmanagedKitUpdated,
+            this, &BuildDeviceKitAspect::kitUpdated);
+}
+
+void BuildDeviceKitAspect::deviceUpdated(Utils::Id id)
+{
+    foreach (Kit *k, KitManager::kits()) {
+        if (deviceId(k) == id)
+            notifyAboutUpdate(k);
+    }
+}
+
+void BuildDeviceKitAspect::kitUpdated(Kit *k)
+{
+    setup(k); // Set default device if necessary
+}
+
+void BuildDeviceKitAspect::devicesChanged()
+{
+    foreach (Kit *k, KitManager::kits())
+        setup(k); // Set default device if necessary
+}
+
 // --------------------------------------------------------------------------
 // EnvironmentKitAspect:
 // --------------------------------------------------------------------------
@@ -1130,9 +1382,9 @@ class EnvironmentKitAspectWidget final : public KitAspectWidget
 public:
     EnvironmentKitAspectWidget(Kit *workingCopy, const KitAspect *ki)
         : KitAspectWidget(workingCopy, ki),
-          m_summaryLabel(new Utils::ElidingLabel),
-          m_manageButton(new QPushButton),
-          m_mainWidget(new QWidget)
+          m_summaryLabel(createSubWidget<Utils::ElidingLabel>()),
+          m_manageButton(createSubWidget<QPushButton>()),
+          m_mainWidget(createSubWidget<QWidget>())
     {
         auto *layout = new QVBoxLayout;
         layout->setContentsMargins(0, 0, 0, 0);
@@ -1147,8 +1399,13 @@ public:
     }
 
 private:
-    QWidget *mainWidget() const override { return m_mainWidget; }
-    QWidget *buttonWidget() const override { return m_manageButton; }
+    void addToLayout(LayoutBuilder &builder) override
+    {
+        addMutableAction(m_mainWidget);
+        builder.addItem(m_mainWidget);
+        builder.addItem(m_manageButton);
+    }
+
     void makeReadOnly() override { m_manageButton->setEnabled(false); }
 
     void refresh() override
@@ -1254,7 +1511,7 @@ void EnvironmentKitAspect::fix(Kit *k)
     }
 }
 
-void EnvironmentKitAspect::addToEnvironment(const Kit *k, Utils::Environment &env) const
+void EnvironmentKitAspect::addToBuildEnvironment(const Kit *k, Environment &env) const
 {
     const QStringList values
             = Utils::transform(Utils::EnvironmentItem::toStringList(environmentChanges(k)),
@@ -1262,6 +1519,11 @@ void EnvironmentKitAspect::addToEnvironment(const Kit *k, Utils::Environment &en
     env.modify(Utils::EnvironmentItem::fromStringList(values));
 }
 
+void EnvironmentKitAspect::addToRunEnvironment(const Kit *k, Environment &env) const
+{
+    addToBuildEnvironment(k, env);
+}
+
 KitAspectWidget *EnvironmentKitAspect::createConfigWidget(Kit *k) const
 {
     QTC_ASSERT(k, return nullptr);
