@@ -26,7 +26,8 @@
 #include "projectwizardpage.h"
 #include "ui_projectwizardpage.h"
 
-#include "projectexplorer.h"
+#include "project.h"
+#include "projectmodels.h"
 #include "session.h"
 
 #include <coreplugin/icore.h>
@@ -67,8 +68,8 @@ public:
     AddNewTree(FolderNode *node, QList<AddNewTree *> children, const QString &displayName);
     AddNewTree(FolderNode *node, QList<AddNewTree *> children, const FolderNode::AddNewInformation &info);
 
-    QVariant data(int column, int role) const;
-    Qt::ItemFlags flags(int column) const;
+    QVariant data(int column, int role) const override;
+    Qt::ItemFlags flags(int column) const override;
 
     QString displayName() const { return m_displayName; }
     FolderNode *node() const { return m_node; }
@@ -77,41 +78,36 @@ public:
 private:
     QString m_displayName;
     QString m_toolTip;
-    FolderNode *m_node;
-    bool m_canAdd;
-    int m_priority;
+    FolderNode *m_node = nullptr;
+    bool m_canAdd = true;
+    int m_priority = -1;
 };
 
-AddNewTree::AddNewTree(const QString &displayName)
-    : m_displayName(displayName),
-      m_node(0),
-      m_canAdd(true),
-      m_priority(-1)
-{
-}
+AddNewTree::AddNewTree(const QString &displayName) :
+    m_displayName(displayName)
+{ }
 
 // FIXME: potentially merge the following two functions.
 // Note the different handling of 'node' and m_canAdd.
-AddNewTree::AddNewTree(FolderNode *node, QList<AddNewTree *> children, const QString &displayName)
-    : m_displayName(displayName),
-      m_node(node),
-      m_canAdd(false),
-      m_priority(-1)
+AddNewTree::AddNewTree(FolderNode *node, QList<AddNewTree *> children, const QString &displayName) :
+    m_displayName(displayName),
+    m_node(node),
+    m_canAdd(false)
 {
     if (node)
-        m_toolTip = ProjectExplorerPlugin::directoryFor(node);
+        m_toolTip = node->directory().toString();
     foreach (AddNewTree *child, children)
         appendChild(child);
 }
 
-AddNewTree::AddNewTree(FolderNode *node, QList<AddNewTree *> children, const FolderNode::AddNewInformation &info)
-    : m_displayName(info.displayName),
-      m_node(node),
-      m_canAdd(true),
-      m_priority(info.priority)
+AddNewTree::AddNewTree(FolderNode *node, QList<AddNewTree *> children,
+                       const FolderNode::AddNewInformation &info) :
+    m_displayName(info.displayName),
+    m_node(node),
+    m_priority(info.priority)
 {
     if (node)
-        m_toolTip = ProjectExplorerPlugin::directoryFor(node);
+        m_toolTip = node->directory().toString();
     foreach (AddNewTree *child, children)
         appendChild(child);
 }
@@ -119,13 +115,16 @@ AddNewTree::AddNewTree(FolderNode *node, QList<AddNewTree *> children, const Fol
 
 QVariant AddNewTree::data(int, int role) const
 {
-    if (role == Qt::DisplayRole)
+    switch (role) {
+    case Qt::DisplayRole:
         return m_displayName;
-    if (role == Qt::ToolTipRole)
+    case Qt::ToolTipRole:
         return m_toolTip;
-    if (role == Qt::UserRole)
+    case Qt::UserRole:
         return QVariant::fromValue(static_cast<void*>(node()));
-    return QVariant();
+    default:
+        return QVariant();
+    }
 }
 
 Qt::ItemFlags AddNewTree::flags(int) const
@@ -142,29 +141,26 @@ Qt::ItemFlags AddNewTree::flags(int) const
 class BestNodeSelector
 {
 public:
-    BestNodeSelector(const QString &commonDirectory, const QStringList &files);
+    BestNodeSelector(const QString &commonDirectory, const FilePaths &files);
     void inspect(AddNewTree *tree, bool isContextNode);
     AddNewTree *bestChoice() const;
     bool deploys();
     QString deployingProjects() const;
+
 private:
     QString m_commonDirectory;
-    QStringList m_files;
-    bool m_deploys;
+    FilePaths m_files;
+    bool m_deploys = false;
     QString m_deployText;
-    AddNewTree *m_bestChoice;
-    int m_bestMatchLength;
-    int m_bestMatchPriority;
+    AddNewTree *m_bestChoice = nullptr;
+    int m_bestMatchLength = -1;
+    int m_bestMatchPriority = -1;
 };
 
-BestNodeSelector::BestNodeSelector(const QString &commonDirectory, const QStringList &files)
-    : m_commonDirectory(commonDirectory),
-      m_files(files),
-      m_deploys(false),
-      m_deployText(QCoreApplication::translate("ProjectWizard", "The files are implicitly added to the projects:") + QLatin1Char('\n')),
-      m_bestChoice(0),
-      m_bestMatchLength(-1),
-      m_bestMatchPriority(-1)
+BestNodeSelector::BestNodeSelector(const QString &commonDirectory, const FilePaths &files) :
+    m_commonDirectory(commonDirectory),
+    m_files(files),
+    m_deployText(QCoreApplication::translate("ProjectWizard", "The files are implicitly added to the projects:") + QLatin1Char('\n'))
 { }
 
 // Find the project the new files should be added
@@ -175,7 +171,7 @@ BestNodeSelector::BestNodeSelector(const QString &commonDirectory, const QString
 void BestNodeSelector::inspect(AddNewTree *tree, bool isContextNode)
 {
     FolderNode *node = tree->node();
-    if (node->nodeType() == ProjectNodeType) {
+    if (node->isProjectNodeType()) {
         if (static_cast<ProjectNode *>(node)->deploysFolder(m_commonDirectory)) {
             m_deploys = true;
             m_deployText += tree->displayName() + QLatin1Char('\n');
@@ -184,7 +180,7 @@ void BestNodeSelector::inspect(AddNewTree *tree, bool isContextNode)
     if (m_deploys)
         return;
 
-    const QString projectDirectory = ProjectExplorerPlugin::directoryFor(node);
+    const QString projectDirectory = node->directory().toString();
     const int projectDirectorySize = projectDirectory.size();
     if (m_commonDirectory != projectDirectory
             && !m_commonDirectory.startsWith(projectDirectory + QLatin1Char('/'))
@@ -206,7 +202,7 @@ void BestNodeSelector::inspect(AddNewTree *tree, bool isContextNode)
 AddNewTree *BestNodeSelector::bestChoice() const
 {
     if (m_deploys)
-        return 0;
+        return nullptr;
     return m_bestChoice;
 }
 
@@ -234,132 +230,81 @@ static inline AddNewTree *createNoneNode(BestNodeSelector *selector)
     return new AddNewTree(displayName);
 }
 
-static inline AddNewTree *buildAddProjectTree(ProjectNode *root, const QString &projectPath, Node *contextNode, BestNodeSelector *selector)
+static inline AddNewTree *buildAddProjectTree(ProjectNode *root, const FilePath &projectPath,
+                                              Node *contextNode, BestNodeSelector *selector)
 {
     QList<AddNewTree *> children;
-    foreach (ProjectNode *pn, root->subProjectNodes()) {
-        AddNewTree *child = buildAddProjectTree(pn, projectPath, contextNode, selector);
-        if (child)
-            children.append(child);
+    for (Node *node : root->nodes()) {
+        if (ProjectNode *pn = node->asProjectNode()) {
+            if (AddNewTree *child = buildAddProjectTree(pn, projectPath, contextNode, selector))
+                children.append(child);
+        }
     }
 
-    const QList<ProjectAction> &list = root->supportedActions(root);
-    if (list.contains(AddSubProject) && !list.contains(InheritedFromParent)) {
+    if (root->supportsAction(AddSubProject, root) && !root->supportsAction(InheritedFromParent, root)) {
         if (projectPath.isEmpty() || root->canAddSubProject(projectPath)) {
-            FolderNode::AddNewInformation info = root->addNewInformation(QStringList() << projectPath, contextNode);
-            AddNewTree *item = new AddNewTree(root, children, info);
+            FolderNode::AddNewInformation info = root->addNewInformation({projectPath}, contextNode);
+            auto item = new AddNewTree(root, children, info);
             selector->inspect(item, root == contextNode);
             return item;
         }
     }
 
     if (children.isEmpty())
-        return 0;
+        return nullptr;
     return new AddNewTree(root, children, root->displayName());
 }
 
-static inline AddNewTree *buildAddProjectTree(SessionNode *root, const QString &projectPath, Node *contextNode, BestNodeSelector *selector)
+static AddNewTree *buildAddFilesTree(FolderNode *root, const FilePaths &files,
+                                     Node *contextNode, BestNodeSelector *selector)
 {
     QList<AddNewTree *> children;
-    foreach (ProjectNode *pn, root->projectNodes()) {
-        AddNewTree *child = buildAddProjectTree(pn, projectPath, contextNode, selector);
-        if (child)
-            children.append(child);
-    }
-    children.prepend(createNoneNode(selector));
-    return new AddNewTree(root, children, root->displayName());
-}
-
-static inline AddNewTree *buildAddFilesTree(FolderNode *root, const QStringList &files,
-                                            Node *contextNode, BestNodeSelector *selector)
-{
-    QList<AddNewTree *> children;
-    foreach (FolderNode *fn, root->subFolderNodes()) {
+    foreach (FolderNode *fn, root->folderNodes()) {
         AddNewTree *child = buildAddFilesTree(fn, files, contextNode, selector);
         if (child)
             children.append(child);
     }
 
-    const QList<ProjectAction> &list = root->supportedActions(root);
-    if (list.contains(AddNewFile) && !list.contains(InheritedFromParent)) {
+    if (root->supportsAction(AddNewFile, root) && !root->supportsAction(InheritedFromParent, root)) {
         FolderNode::AddNewInformation info = root->addNewInformation(files, contextNode);
-        AddNewTree *item = new AddNewTree(root, children, info);
+        auto item = new AddNewTree(root, children, info);
         selector->inspect(item, root == contextNode);
         return item;
     }
     if (children.isEmpty())
-        return 0;
+        return nullptr;
     return new AddNewTree(root, children, root->displayName());
 }
 
-static inline AddNewTree *buildAddFilesTree(SessionNode *root, const QStringList &files,
-                                            Node *contextNode, BestNodeSelector *selector)
-{
-    QList<AddNewTree *> children;
-    foreach (ProjectNode *pn, root->projectNodes()) {
-        AddNewTree *child = buildAddFilesTree(pn, files, contextNode, selector);
-        if (child)
-            children.append(child);
-    }
-    children.prepend(createNoneNode(selector));
-    return new AddNewTree(root, children, root->displayName());
-}
-
-static inline AddNewTree *getChoices(const QStringList &generatedFiles,
-                                     IWizardFactory::WizardKind wizardKind,
-                                     Node *contextNode,
-                                     BestNodeSelector *selector)
-{
-    if (wizardKind == IWizardFactory::ProjectWizard)
-        return buildAddProjectTree(SessionManager::sessionNode(), generatedFiles.first(), contextNode, selector);
-    else
-        return buildAddFilesTree(SessionManager::sessionNode(), generatedFiles, contextNode, selector);
-}
-
 // --------------------------------------------------------------------
 // ProjectWizardPage:
 // --------------------------------------------------------------------
 
-ProjectWizardPage::ProjectWizardPage(QWidget *parent) :
-    WizardPage(parent),
-    m_ui(new Ui::WizardPage),
-    m_model(0),
-    m_repositoryExists(false)
+ProjectWizardPage::ProjectWizardPage(QWidget *parent) : WizardPage(parent),
+    m_ui(new Ui::WizardPage)
 {
     m_ui->setupUi(this);
     m_ui->vcsManageButton->setText(ICore::msgShowOptionsDialog());
-    connect(m_ui->projectComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(m_ui->projectComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &ProjectWizardPage::projectChanged);
-    connect(m_ui->addToVersionControlComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    connect(m_ui->addToVersionControlComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &ProjectWizardPage::versionControlChanged);
     connect(m_ui->vcsManageButton, &QAbstractButton::clicked, this, &ProjectWizardPage::manageVcs);
     setProperty(SHORT_TITLE_PROPERTY, tr("Summary"));
 
     connect(VcsManager::instance(), &VcsManager::configurationChanged,
             this, &ProjectExplorer::Internal::ProjectWizardPage::initializeVersionControls);
+
+    m_ui->projectComboBox->setModel(&m_model);
 }
 
 ProjectWizardPage::~ProjectWizardPage()
 {
-    disconnect(m_ui->projectComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+    disconnect(m_ui->projectComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
                this, &ProjectWizardPage::projectChanged);
-    delete m_model;
     delete m_ui;
 }
 
-void ProjectWizardPage::setModel(TreeModel *model)
-{
-    delete m_model;
-    m_model = model;
-
-    // TODO see OverViewCombo and OverView for click event filter
-    m_ui->projectComboBox->setModel(model);
-    bool enabled = m_model->rowCount(QModelIndex()) > 1;
-    m_ui->projectComboBox->setEnabled(enabled);
-
-    expandTree(QModelIndex());
-}
-
 bool ProjectWizardPage::expandTree(const QModelIndex &root)
 {
     bool expand = false;
@@ -367,9 +312,9 @@ bool ProjectWizardPage::expandTree(const QModelIndex &root)
         expand = true;
 
     // Check children
-    int count = m_model->rowCount(root);
+    int count = m_model.rowCount(root);
     for (int i = 0; i < count; ++i) {
-        if (expandTree(m_model->index(i, 0, root)))
+        if (expandTree(m_model.index(i, 0, root)))
             expand = true;
     }
 
@@ -380,7 +325,7 @@ bool ProjectWizardPage::expandTree(const QModelIndex &root)
         m_ui->projectComboBox->view()->collapse(root);
 
     // if we are a high priority node, our *parent* needs to be expanded
-    AddNewTree *tree = static_cast<AddNewTree *>(root.internalPointer());
+    auto tree = static_cast<AddNewTree *>(root.internalPointer());
     if (tree && tree->priority() >= 100)
         expand = true;
 
@@ -389,7 +334,7 @@ bool ProjectWizardPage::expandTree(const QModelIndex &root)
 
 void ProjectWizardPage::setBestNode(AddNewTree *tree)
 {
-    QModelIndex index = tree ? m_model->indexForItem(tree) : QModelIndex();
+    QModelIndex index = tree ? m_model.indexForItem(tree) : QModelIndex();
     m_ui->projectComboBox->setCurrentIndex(index);
 
     while (index.isValid()) {
@@ -401,7 +346,7 @@ void ProjectWizardPage::setBestNode(AddNewTree *tree)
 FolderNode *ProjectWizardPage::currentNode() const
 {
     QVariant v = m_ui->projectComboBox->currentData(Qt::UserRole);
-    return v.isNull() ? 0 : static_cast<FolderNode *>(v.value<void *>());
+    return v.isNull() ? nullptr : static_cast<FolderNode *>(v.value<void *>());
 }
 
 void ProjectWizardPage::setAddingSubProject(bool addingSubProject)
@@ -423,7 +368,7 @@ void ProjectWizardPage::initializeVersionControls()
     if (versionControls.isEmpty())
         hideVersionControlUiElements();
 
-    IVersionControl *currentSelection = 0;
+    IVersionControl *currentSelection = nullptr;
     int currentIdx = versionControlIndex() - 1;
     if (currentIdx >= 0 && currentIdx <= m_activeVersionControls.size() - 1)
         currentSelection = m_activeVersionControls.at(currentIdx);
@@ -432,7 +377,8 @@ void ProjectWizardPage::initializeVersionControls()
 
     QStringList versionControlChoices = QStringList(tr("<None>"));
     if (!m_commonDirectory.isEmpty()) {
-        IVersionControl *managingControl = VcsManager::findVersionControlForDirectory(m_commonDirectory);
+        IVersionControl *managingControl =
+                VcsManager::findVersionControlForDirectory(FilePath::fromString(m_commonDirectory));
         if (managingControl) {
             // Under VCS
             if (managingControl->supportsOperation(IVersionControl::AddOperation)) {
@@ -474,7 +420,7 @@ bool ProjectWizardPage::runVersionControl(const QList<GeneratedFile> &files, QSt
     // Create repository?
     if (!m_repositoryExists) {
         QTC_ASSERT(versionControl->supportsOperation(IVersionControl::CreateRepositoryOperation), return false);
-        if (!versionControl->vcsCreateRepository(m_commonDirectory)) {
+        if (!versionControl->vcsCreateRepository(FilePath::fromString(m_commonDirectory))) {
             *errorMessage = tr("A version control system repository could not be created in \"%1\".").arg(m_commonDirectory);
             return false;
         }
@@ -482,7 +428,7 @@ bool ProjectWizardPage::runVersionControl(const QList<GeneratedFile> &files, QSt
     // Add files if supported.
     if (versionControl->supportsOperation(IVersionControl::AddOperation)) {
         foreach (const GeneratedFile &generatedFile, files) {
-            if (!versionControl->vcsAdd(generatedFile.path())) {
+            if (!versionControl->vcsAdd(generatedFile.filePath())) {
                 *errorMessage = tr("Failed to add \"%1\" to the version control system.").arg(generatedFile.path());
                 return false;
             }
@@ -491,19 +437,42 @@ bool ProjectWizardPage::runVersionControl(const QList<GeneratedFile> &files, QSt
     return true;
 }
 
-void ProjectWizardPage::initializeProjectTree(Node *context, const QStringList &paths,
+void ProjectWizardPage::initializeProjectTree(Node *context, const FilePaths &paths,
                                               IWizardFactory::WizardKind kind,
                                               ProjectAction action)
 {
     BestNodeSelector selector(m_commonDirectory, paths);
-    AddNewTree *tree = getChoices(paths, kind, context, &selector);
+
+    TreeItem *root = m_model.rootItem();
+    root->removeChildren();
+    for (Project *project : SessionManager::projects()) {
+        if (ProjectNode *pn = project->rootProjectNode()) {
+            if (kind == IWizardFactory::ProjectWizard) {
+                if (AddNewTree *child = buildAddProjectTree(pn, paths.first(), context, &selector))
+                    root->appendChild(child);
+            } else {
+                if (AddNewTree *child = buildAddFilesTree(pn, paths, context, &selector))
+                    root->appendChild(child);
+            }
+        }
+    }
+    root->sortChildren([](const TreeItem *ti1, const TreeItem *ti2) {
+        return compareNodes(static_cast<const AddNewTree *>(ti1)->node(),
+                            static_cast<const AddNewTree *>(ti2)->node());
+    });
+    root->prependChild(createNoneNode(&selector));
+
+    // Set combobox to context node if that appears in the tree:
+    auto predicate = [context](TreeItem *ti) { return static_cast<AddNewTree*>(ti)->node() == context; };
+    TreeItem *contextItem = root->findAnyChild(predicate);
+    if (contextItem)
+        m_ui->projectComboBox->setCurrentIndex(m_model.indexForItem(contextItem));
 
     setAdditionalInfo(selector.deployingProjects());
-
-    TreeModel *model = new TreeModel(tree);
-    setModel(model);
     setBestNode(selector.bestChoice());
     setAddingSubProject(action == AddSubProject);
+
+    m_ui->projectComboBox->setEnabled(m_model.rowCount(QModelIndex()) > 1);
 }
 
 void ProjectWizardPage::setNoneLabel(const QString &label)
@@ -537,7 +506,7 @@ IVersionControl *ProjectWizardPage::currentVersionControl()
 {
     int index = m_ui->addToVersionControlComboBox->currentIndex() - 1; // Subtract "<None>"
     if (index < 0 || index > m_activeVersionControls.count())
-        return 0; // <None>
+        return nullptr; // <None>
     return m_activeVersionControls.at(index);
 }
 
@@ -559,7 +528,9 @@ void ProjectWizardPage::setFiles(const QStringList &fileNames)
             formattedFiles = fileNames;
         } else {
             str << QDir::toNativeSeparators(m_commonDirectory) << ":\n\n";
-            const int prefixSize = m_commonDirectory.size() + 1;
+            int prefixSize = m_commonDirectory.size();
+            if (!m_commonDirectory.endsWith('/'))
+                ++prefixSize;
             formattedFiles = Utils::transform(fileNames, [prefixSize](const QString &f)
                                                          { return f.mid(prefixSize); });
         }
@@ -569,7 +540,7 @@ void ProjectWizardPage::setFiles(const QStringList &fileNames)
             const bool filePath2HasDir = filePath2.contains(QLatin1Char('/'));
 
             if (filePath1HasDir == filePath2HasDir)
-                return FileName::fromString(filePath1) < FileName::fromString(filePath2);
+                return FilePath::fromString(filePath1) < FilePath::fromString(filePath2);
             return filePath1HasDir;
         }
 );
