@@ -28,15 +28,16 @@
 #include "pythonconstants.h"
 
 #include <coreplugin/dialogs/ioptionspage.h>
-#include "coreplugin/icore.h"
+#include <coreplugin/icore.h>
 
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/detailswidget.h>
 #include <utils/environment.h>
 #include <utils/listmodel.h>
+#include <utils/layoutbuilder.h>
 #include <utils/pathchooser.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 #include <utils/treemodel.h>
 
 #include <QDir>
@@ -46,13 +47,13 @@
 #include <QSettings>
 #include <QStackedWidget>
 #include <QTreeView>
-#include <QVBoxLayout>
 #include <QWidget>
 
 namespace Python {
 namespace Internal {
 
 using namespace Utils;
+using namespace Layouting;
 
 class InterpreterDetailsWidget : public QWidget
 {
@@ -61,25 +62,24 @@ public:
         : m_name(new QLineEdit)
         , m_executable(new Utils::PathChooser())
     {
-        auto mainLayout = new QGridLayout();
-        mainLayout->addWidget(new QLabel(PythonSettings::tr("Name:")), 0, 0);
-        mainLayout->addWidget(m_name, 0, 1);
-        mainLayout->addWidget(new QLabel(PythonSettings::tr("Executable")), 1, 0);
-        mainLayout->addWidget(m_executable, 1, 1);
         m_executable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-        setLayout(mainLayout);
+
+        Form {
+            PythonSettings::tr("Name:"), m_name, Break(),
+            PythonSettings::tr("Executable"), m_executable
+        }.attachTo(this, false);
     }
 
     void updateInterpreter(const Interpreter &interpreter)
     {
         m_name->setText(interpreter.name);
-        m_executable->setPath(interpreter.command.toString());
+        m_executable->setFilePath(interpreter.command);
         m_currentId = interpreter.id;
     }
 
     Interpreter toInterpreter()
     {
-        return {m_currentId, m_name->text(), FilePath::fromUserInput(m_executable->filePath().toString())};
+        return {m_currentId, m_name->text(), m_executable->filePath()};
     }
     QLineEdit *m_name = nullptr;
     PathChooser *m_executable = nullptr;
@@ -125,8 +125,6 @@ InterpreterOptionsWidget::InterpreterOptionsWidget(const QList<Interpreter> &int
     });
     m_model.setAllData(interpreters);
 
-    auto mainLayout = new QVBoxLayout();
-    auto layout = new QHBoxLayout();
     m_view.setModel(&m_model);
     m_view.setHeaderHidden(true);
     m_view.setSelectionMode(QAbstractItemView::SingleSelection);
@@ -135,25 +133,31 @@ InterpreterOptionsWidget::InterpreterOptionsWidget(const QList<Interpreter> &int
             &QItemSelectionModel::currentChanged,
             this,
             &InterpreterOptionsWidget::currentChanged);
-    auto buttonLayout = new QVBoxLayout();
+
     auto addButton = new QPushButton(PythonSettings::tr("&Add"));
     connect(addButton, &QPushButton::pressed, this, &InterpreterOptionsWidget::addItem);
+
     m_deleteButton = new QPushButton(PythonSettings::tr("&Delete"));
     m_deleteButton->setEnabled(false);
     connect(m_deleteButton, &QPushButton::pressed, this, &InterpreterOptionsWidget::deleteItem);
+
     m_makeDefaultButton = new QPushButton(PythonSettings::tr("&Make Default"));
     m_makeDefaultButton->setEnabled(false);
     connect(m_makeDefaultButton, &QPushButton::pressed, this, &InterpreterOptionsWidget::makeDefault);
-    mainLayout->addLayout(layout);
-    mainLayout->addWidget(m_detailsWidget);
+
     m_detailsWidget->hide();
-    setLayout(mainLayout);
-    layout->addWidget(&m_view);
-    layout->addLayout(buttonLayout);
-    buttonLayout->addWidget(addButton);
-    buttonLayout->addWidget(m_deleteButton);
-    buttonLayout->addWidget(m_makeDefaultButton);
-    buttonLayout->addStretch(10);
+
+    Column buttons {
+        addButton,
+        m_deleteButton,
+        m_makeDefaultButton,
+        Stretch()
+    };
+
+    Column {
+        Row { &m_view, buttons },
+        m_detailsWidget
+    }.attachTo(this);
 }
 
 void InterpreterOptionsWidget::apply()
@@ -274,13 +278,13 @@ Interpreter::Interpreter(const FilePath &python, const QString &defaultName, boo
     : id(QUuid::createUuid().toString())
     , command(python)
 {
-    SynchronousProcess pythonProcess;
+    QtcProcess pythonProcess;
     pythonProcess.setProcessChannelMode(QProcess::MergedChannels);
     pythonProcess.setTimeoutS(1);
-    SynchronousProcessResponse response = pythonProcess.runBlocking(
-        CommandLine(python, {"--version"}));
-    if (response.result == SynchronousProcessResponse::Finished)
-        name = response.stdOut().trimmed();
+    pythonProcess.setCommand({python, {"--version"}});
+    pythonProcess.runBlocking();
+    if (pythonProcess.result() == QtcProcess::FinishedWithSuccess)
+        name = pythonProcess.stdOut().trimmed();
     if (name.isEmpty())
         name = defaultName;
     if (windowedSuffix)
@@ -387,11 +391,10 @@ static void addPythonsFromRegistry(QList<Interpreter> &pythons)
         regVal = pythonRegistry.value("InstallPath/.");
         if (regVal.isValid()) {
             const FilePath &path = FilePath::fromUserInput(regVal.toString());
-            const FilePath &python = path.pathAppended(HostOsInfo::withExecutableSuffix("python"));
+            const FilePath python = path.pathAppended("python").withExecutableSuffix();
             if (python.exists() && !alreadyRegistered(pythons, python))
                 pythons << Interpreter(python, "Python " + versionGroup);
-            const FilePath &pythonw = path.pathAppended(
-                HostOsInfo::withExecutableSuffix("pythonw"));
+            const FilePath pythonw = path.pathAppended("pythonw").withExecutableSuffix();
             if (pythonw.exists() && !alreadyRegistered(pythons, pythonw))
                 pythons << Interpreter(pythonw, "Python " + versionGroup, true);
         }
