@@ -26,14 +26,18 @@
 #include "designeractionmanager.h"
 
 #include "changestyleaction.h"
-#include "modelnodecontextmenu_helper.h"
-#include <bindingproperty.h>
-#include <nodeproperty.h>
-#include <nodelistproperty.h>
-#include <nodehints.h>
-#include <nodemetainfo.h>
 #include "designeractionmanagerview.h"
+#include "designermcumanager.h"
+#include "formatoperation.h"
+#include "modelnodecontextmenu_helper.h"
 #include "qmldesignerconstants.h"
+#include "rewritingexception.h"
+#include <bindingproperty.h>
+#include <nodehints.h>
+#include <nodelistproperty.h>
+#include <nodemetainfo.h>
+#include <nodeproperty.h>
+#include <theme.h>
 
 #include <formeditortoolbutton.h>
 
@@ -44,15 +48,20 @@
 #include <listmodeleditor/listmodeleditordialog.h>
 #include <listmodeleditor/listmodeleditormodel.h>
 
-#include <QHBoxLayout>
-#include <QGraphicsLinearLayout>
-
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/icore.h>
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
 #include <utils/utilsicons.h>
 
+#include <QGraphicsLinearLayout>
+#include <QHBoxLayout>
+#include <QMessageBox>
+#include <QMimeData>
+
+#include <exception>
+
 namespace QmlDesigner {
 
 static inline QString captionForModelNode(const ModelNode &modelNode)
@@ -85,7 +94,7 @@ DesignerActionToolBar *DesignerActionManager::createToolBar(QWidget *parent) con
         return l->priority() > r->priority();
     });
 
-    for (auto *categoryAction : categories) {
+    for (auto *categoryAction : qAsConst(categories)) {
         QList<ActionInterface* > actions = Utils::filtered(designerActions(), [categoryAction](ActionInterface *action) {
                 return action->category() == categoryAction->menuId();
         });
@@ -96,7 +105,7 @@ DesignerActionToolBar *DesignerActionManager::createToolBar(QWidget *parent) con
 
         bool addSeparator = false;
 
-        for (auto *action : actions) {
+        for (auto *action : qAsConst(actions)) {
             if ((action->type() == ActionInterface::Action || action->type() == ActionInterface::ToolBarAction)
                     && action->action()) {
                 toolBar->registerAction(action);
@@ -159,7 +168,7 @@ QGraphicsWidget *DesignerActionManager::createFormEditorToolBar(QGraphicsItem *p
     layout->setSpacing(0);
     toolbar->setLayout(layout);
 
-    for (ActionInterface *action : actions) {
+    for (ActionInterface *action : qAsConst(actions)) {
         auto button = new FormEditorToolButton(action->action(), toolbar);
         layout->addItem(button);
     }
@@ -194,6 +203,101 @@ void DesignerActionManager::registerAddResourceHandler(const AddResourceHandler
     m_addResourceHandler.append(handler);
 }
 
+void DesignerActionManager::unregisterAddResourceHandlers(const QString &category)
+{
+    for (int i = m_addResourceHandler.size() - 1; i >= 0 ; --i) {
+        const AddResourceHandler &handler = m_addResourceHandler[i];
+        if (handler.category == category)
+            m_addResourceHandler.removeAt(i);
+    }
+}
+
+void DesignerActionManager::registerModelNodePreviewHandler(const ModelNodePreviewImageHandler &handler)
+{
+    m_modelNodePreviewImageHandlers.append(handler);
+}
+
+bool DesignerActionManager::hasModelNodePreviewHandler(const ModelNode &node) const
+{
+    const bool isComponent = node.isComponent();
+    for (const auto &handler : qAsConst(m_modelNodePreviewImageHandlers)) {
+        if ((isComponent || !handler.componentOnly) && node.isSubclassOf(handler.type))
+            return true;
+    }
+    return false;
+}
+
+ModelNodePreviewImageOperation DesignerActionManager::modelNodePreviewOperation(const ModelNode &node) const
+{
+    ModelNodePreviewImageOperation op = nullptr;
+    int prio = -1;
+    const bool isComponent = node.isComponent();
+    for (const auto &handler : qAsConst(m_modelNodePreviewImageHandlers)) {
+        if ((isComponent || !handler.componentOnly) && handler.priority > prio
+                && node.isSubclassOf(handler.type)) {
+            op = handler.operation;
+            prio = handler.priority;
+        }
+    }
+    return op;
+}
+
+bool DesignerActionManager::externalDragHasSupportedAssets(const QMimeData *mimeData) const
+{
+    if (!mimeData->hasUrls())
+        return false;
+
+    QSet<QString> filtersSet;
+    const QList<AddResourceHandler> handlers = addResourceHandler();
+    for (const AddResourceHandler &handler : handlers)
+        filtersSet.insert(handler.filter);
+
+    const QList<QUrl> urls = mimeData->urls();
+    for (const QUrl &url : urls) {
+        QString suffix = "*." + url.fileName().split('.').last().toLower();
+        if (filtersSet.contains(suffix)) // accept drop if it contains a valid file
+            return true;
+    }
+
+    return false;
+}
+
+QHash<QString, QStringList> DesignerActionManager::handleExternalAssetsDrop(const QMimeData *mimeData) const
+{
+    const QList<AddResourceHandler> handlers = addResourceHandler();
+    // create suffix to categry and category to operation hashes
+    QHash<QString, QString> suffixCategory;
+    QHash<QString, AddResourceOperation> categoryOperation;
+    for (const AddResourceHandler &handler : handlers) {
+        suffixCategory.insert(handler.filter, handler.category);
+        categoryOperation.insert(handler.category, handler.operation);
+    }
+
+    // add files grouped by categories (so that files under same category run under 1 operation)
+    QHash<QString, QStringList> categoryFiles;
+    const QList<QUrl> urls = mimeData->urls();
+    for (const QUrl &url : urls) {
+        QString suffix = "*." + url.fileName().split('.').last().toLower();
+        QString category = suffixCategory.value(suffix);
+        if (!category.isEmpty())
+            categoryFiles[category].append(url.toLocalFile());
+    }
+
+    QHash<QString, QStringList> addedCategoryFiles;
+
+    // run operations
+    const QStringList categories = categoryFiles.keys();
+    for (const QString &category : categories) {
+        AddResourceOperation operation = categoryOperation.value(category);
+        QStringList files = categoryFiles.value(category);
+        AddFilesResult result = operation(files, {});
+        if (result == AddFilesResult::Succeeded)
+            addedCategoryFiles.insert(category, files);
+    }
+
+    return addedCategoryFiles;
+}
+
 class VisiblityModelNodeAction : public ModelNodeContextMenuAction
 {
 public:
@@ -305,28 +409,30 @@ public:
                     && !selectionContext().currentSingleSelectedNode().isRootNode()
                     && selectionContext().currentSingleSelectedNode().hasParentProperty()) {
 
-                ActionTemplate *selectionAction = new ActionTemplate(QString(), &ModelNodeOperations::select);
-                selectionAction->setParent(menu());
-
                 parentNode = selectionContext().currentSingleSelectedNode().parentProperty().parentModelNode();
 
-                selectionAction->setText(QString(QT_TRANSLATE_NOOP("QmlDesignerContextMenu", "Select parent: %1")).arg(
-                                             captionForModelNode(parentNode)));
+                if (!ModelNode::isThisOrAncestorLocked(parentNode)) {
+                    ActionTemplate *selectionAction = new ActionTemplate("SELECTION", {}, &ModelNodeOperations::select);
+                    selectionAction->setParent(menu());
+                    selectionAction->setText(QString(QT_TRANSLATE_NOOP("QmlDesignerContextMenu", "Select parent: %1")).arg(
+                                                 captionForModelNode(parentNode)));
 
-                SelectionContext nodeSelectionContext = selectionContext();
-                nodeSelectionContext.setTargetNode(parentNode);
-                selectionAction->setSelectionContext(nodeSelectionContext);
+                    SelectionContext nodeSelectionContext = selectionContext();
+                    nodeSelectionContext.setTargetNode(parentNode);
+                    selectionAction->setSelectionContext(nodeSelectionContext);
 
-                menu()->addAction(selectionAction);
+                    menu()->addAction(selectionAction);
+                }
             }
-            foreach (const ModelNode &node, selectionContext().view()->allModelNodes()) {
+            for (const ModelNode &node : selectionContext().view()->allModelNodes()) {
                 if (node != selectionContext().currentSingleSelectedNode()
                         && node != parentNode
                         && contains(node, selectionContext().scenePosition())
-                        && !node.isRootNode()) {
+                        && !node.isRootNode()
+                        && !ModelNode::isThisOrAncestorLocked(node)) {
                     selectionContext().setTargetNode(node);
                     QString what = QString(QT_TRANSLATE_NOOP("QmlDesignerContextMenu", "Select: %1")).arg(captionForModelNode(node));
-                    ActionTemplate *selectionAction = new ActionTemplate(what, &ModelNodeOperations::select);
+                    ActionTemplate *selectionAction = new ActionTemplate("SELECT", what, &ModelNodeOperations::select);
 
                     SelectionContext nodeSelectionContext = selectionContext();
                     nodeSelectionContext.setTargetNode(node);
@@ -335,10 +441,19 @@ public:
                     menu()->addAction(selectionAction);
                 }
             }
+
+            if (menu()->isEmpty())
+                action()->setEnabled(false);
         }
     }
 };
 
+class DocumentError : public std::exception
+{
+public:
+    const char *what() const noexcept override { return "Current document contains errors."; }
+};
+
 class EditListModelAction final : public ModelNodeContextMenuAction
 {
 public:
@@ -357,8 +472,10 @@ public:
     static bool isListViewInBaseState(const SelectionContext &selectionState)
     {
         return selectionState.isInBaseState() && selectionState.singleNodeIsSelected()
-               && selectionState.currentSingleSelectedNode().metaInfo().isSubclassOf(
-                   "QtQuick.ListView");
+               && (selectionState.currentSingleSelectedNode().metaInfo().isSubclassOf(
+                       "QtQuick.ListView")
+                   || selectionState.currentSingleSelectedNode().metaInfo().isSubclassOf(
+                       "QtQuick.GridView"));
     }
 
     bool isEnabled(const SelectionContext &) const override { return true; }
@@ -384,6 +501,24 @@ public:
                                        return view->createModelNode(elementMetaInfo.typeName(),
                                                                     elementMetaInfo.majorVersion(),
                                                                     elementMetaInfo.minorVersion());
+                                   },
+                                   [&](const ModelNode &node) {
+                                       bool isNowInComponent = ModelNodeOperations::goIntoComponent(
+                                           node);
+
+                                       Model *currentModel = QmlDesignerPlugin::instance()
+                                                                 ->currentDesignDocument()
+                                                                 ->currentModel();
+
+                                       if (currentModel->rewriterView()
+                                           && currentModel->rewriterView()->inErrorState()) {
+                                           throw DocumentError{};
+                                       }
+
+                                       if (isNowInComponent)
+                                           return view->rootModelNode();
+
+                                       return node;
                                    }};
 
         model.setListView(targetNode);
@@ -391,7 +526,22 @@ public:
         ListModelEditorDialog dialog{Core::ICore::mainWindow()};
         dialog.setModel(&model);
 
-        dialog.exec();
+        try {
+            dialog.exec();
+        } catch (const DocumentError &) {
+            QMessageBox::warning(
+                Core::ICore::mainWindow(),
+                QCoreApplication::translate("DesignerActionManager", "Document Has Errors"),
+                QCoreApplication::translate("DesignerActionManager",
+                                            "The document which contains the list model "
+                                            "contains errors. So we cannot edit it."));
+        } catch (const RewritingException &) {
+            QMessageBox::warning(
+                Core::ICore::mainWindow(),
+                QCoreApplication::translate("DesignerActionManager", "Document Cannot Be Written"),
+                QCoreApplication::translate("DesignerActionManager",
+                                            "An error occurred during a write attemp."));
+        }
     }
 };
 
@@ -465,7 +615,7 @@ public:
             for (const QmlFlowItemNode &node : QmlFlowViewNode(selectionContext().rootNode()).flowItems()) {
                 if (node != selectionContext().currentSingleSelectedNode().parentProperty().parentModelNode()) {
                     QString what = QString(QT_TRANSLATE_NOOP("QmlDesignerContextMenu", "Connect: %1")).arg(captionForModelNode(node));
-                    ActionTemplate *connectionAction = new ActionTemplate(what, &ModelNodeOperations::addTransition);
+                    ActionTemplate *connectionAction = new ActionTemplate("CONNECT", what, &ModelNodeOperations::addTransition);
 
                     SelectionContext nodeSelectionContext = selectionContext();
                     nodeSelectionContext.setTargetNode(node);
@@ -483,6 +633,7 @@ const char yProperty[] = "y";
 const char zProperty[] = "z";
 const char widthProperty[] = "width";
 const char heightProperty[] = "height";
+const char triggerSlot[] = "trigger";
 
 using namespace SelectionContextFunctors;
 
@@ -491,11 +642,6 @@ bool multiSelection(const SelectionContext &context)
     return !singleSelection(context) && selectionNotEmpty(context);
 }
 
-bool singleSelectionAndInBaseState(const SelectionContext &context)
-{
-    return singleSelection(context) && inBaseState(context);
-}
-
 bool multiSelectionAndInBaseState(const SelectionContext &context)
 {
     return multiSelection(context) && inBaseState(context);
@@ -511,6 +657,11 @@ bool selectionHasSameParentAndInBaseState(const SelectionContext &context)
     return selectionHasSameParent(context) && inBaseState(context);
 }
 
+bool multiSelectionAndHasSameParent(const SelectionContext &context)
+{
+    return multiSelection(context) && selectionHasSameParent(context);
+}
+
 bool isNotInLayout(const SelectionContext &context)
 {
     if (selectionNotEmpty(context)) {
@@ -540,6 +691,11 @@ bool selectionCanBeLayoutedAndQtQuickLayoutPossible(const SelectionContext &cont
     return selectionCanBeLayouted(context) && context.view()->majorQtQuickVersion() > 1;
 }
 
+bool selectionCanBeLayoutedAndQtQuickLayoutPossibleAndNotMCU(const SelectionContext &context)
+{
+    return selectionCanBeLayoutedAndQtQuickLayoutPossible(context) && !DesignerMcuManager::instance().isMCUProject();
+}
+
 bool selectionNotEmptyAndHasZProperty(const SelectionContext &context)
 {
     return selectionNotEmpty(context) && selectionHasProperty(context, zProperty);
@@ -563,6 +719,14 @@ bool selectionNotEmptyAndHasXorYProperty(const SelectionContext &context)
         && selectionHasProperty1or2(context, xProperty, yProperty);
 }
 
+bool singleSelectionAndHasSlotTrigger(const SelectionContext &context)
+{
+    if (!singleSelection(context))
+        return false;
+
+    return selectionHasSlot(context, triggerSlot);
+}
+
 bool singleSelectionAndInQtQuickLayout(const SelectionContext &context)
 {
     if (!singleSelection(context))
@@ -728,14 +892,14 @@ bool isPositioner(const SelectionContext &context)
 
 bool layoutOptionVisible(const SelectionContext &context)
 {
-    return multiSelectionAndInBaseState(context)
+    return selectionCanBeLayoutedAndQtQuickLayoutPossible(context)
             || singleSelectionAndInQtQuickLayout(context)
             || isLayout(context);
 }
 
 bool positionOptionVisible(const SelectionContext &context)
 {
-    return multiSelectionAndInBaseState(context)
+    return selectionCanBeLayouted(context)
             || isPositioner(context);
 }
 
@@ -745,6 +909,11 @@ bool studioComponentsAvailable(const SelectionContext &context)
     return context.view()->model()->isImportPossible(import, true, true);
 }
 
+bool studioComponentsAvailableAndSelectionCanBeLayouted(const SelectionContext &context)
+{
+    return selectionCanBeLayouted(context) && studioComponentsAvailable(context);
+}
+
 bool singleSelectedAndUiFile(const SelectionContext &context)
 {
     if (!singleSelection(context))
@@ -755,8 +924,7 @@ bool singleSelectedAndUiFile(const SelectionContext &context)
     if (!designDocument)
         return false;
 
-    return designDocument->fileName().toFileInfo().completeSuffix()
-            == QLatin1String("ui.qml");
+    return designDocument->fileName().completeSuffix() == QLatin1String("ui.qml");
 }
 
 bool lowerAvailable(const SelectionContext &selectionState)
@@ -799,20 +967,28 @@ bool raiseAvailable(const SelectionContext &selectionState)
     return parentProperty.indexOf(modelNode) < parentProperty.count() - 1;
 }
 
+bool anchorsMenuEnabled(const SelectionContext &context)
+{
+    return singleSelectionItemIsNotAnchoredAndSingleSelectionNotRoot(context)
+           || singleSelectionItemIsAnchored(context);
+}
+
+
 void DesignerActionManager::createDefaultDesignerActions()
 {
     using namespace SelectionContextFunctors;
     using namespace ComponentCoreConstants;
     using namespace ModelNodeOperations;
+    using namespace FormatOperation;
 
     const Utils::Icon prevIcon({
-        {QLatin1String(":/utils/images/prev.png"), Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
+        {":/utils/images/prev.png", Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
 
     const Utils::Icon nextIcon({
-        {QLatin1String(":/utils/images/next.png"), Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
+        {":/utils/images/next.png", Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
 
     const Utils::Icon addIcon({
-        {QLatin1String(":/utils/images/plus.png"), Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
+        {":/utils/images/plus.png", Utils::Theme::QmlDesigner_FormEditorForegroundColor}}, Utils::Icon::MenuTintedStyle);
 
     addDesignerAction(new SelectionModelNodeAction(
                           selectionCategoryDisplayName,
@@ -820,8 +996,8 @@ void DesignerActionManager::createDefaultDesignerActions()
                           prioritySelectionCategory));
 
     addDesignerAction(new ActionGroup(
-                          stackCategoryDisplayName,
-                          stackCategory,
+                          arrangeCategoryDisplayName,
+                          arrangeCategory,
                           priorityStackCategory,
                           &selectionNotEmpty));
 
@@ -829,28 +1005,19 @@ void DesignerActionManager::createDefaultDesignerActions()
                           toFrontCommandId,
                           toFrontDisplayName,
                           {},
-                          stackCategory,
+                          arrangeCategory,
                           QKeySequence(),
                           200,
                           &toFront,
-                          &singleSelection));
+                          &raiseAvailable));
 
     addDesignerAction(new ModelNodeContextMenuAction(
-                          toBackCommandId,
-                          toBackDisplayName,
-                          {},
-                          stackCategory,
+                          raiseCommandId,
+                          raiseDisplayName,
+                          Utils::Icon({{":/qmldesigner/icon/designeractions/images/raise.png", Utils::Theme::IconsBaseColor}}).icon(),
+                          arrangeCategory,
                           QKeySequence(),
                           180,
-                          &toBack,
-                          &singleSelection));
-
-    addDesignerAction(new ModelNodeContextMenuAction(
-                          raiseCommandId, raiseDisplayName,
-                          Utils::Icon({{":/qmldesigner/icon/designeractions/images/raise.png", Utils::Theme::IconsBaseColor}}).icon(),
-                          stackCategory,
-                          QKeySequence(),
-                          160,
                           &raise,
                           &raiseAvailable));
 
@@ -858,37 +1025,97 @@ void DesignerActionManager::createDefaultDesignerActions()
                           lowerCommandId,
                           lowerDisplayName,
                           Utils::Icon({{":/qmldesigner/icon/designeractions/images/lower.png", Utils::Theme::IconsBaseColor}}).icon(),
-                          stackCategory,
+                          arrangeCategory,
                           QKeySequence(),
-                          140,
+                          160,
                           &lower,
                           &lowerAvailable));
 
-    addDesignerAction(new SeperatorDesignerAction(stackCategory, 120));
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          toBackCommandId,
+                          toBackDisplayName,
+                          {},
+                          arrangeCategory,
+                          QKeySequence(),
+                          140,
+                          &toBack,
+                          &lowerAvailable));
 
     addDesignerAction(new ModelNodeContextMenuAction(
-                          resetZCommandId,
-                          resetZDisplayName,
+                          reverseCommandId,
+                          reverseDisplayName,
                           {},
-                          stackCategory,
+                          arrangeCategory,
                           QKeySequence(),
                           100,
-                          &resetZ,
-                          &selectionNotEmptyAndHasZProperty));
+                          &reverse,
+                          &multiSelectionAndHasSameParent));
 
     addDesignerAction(new ActionGroup(editCategoryDisplayName, editCategory, priorityEditCategory, &selectionNotEmpty));
 
     addDesignerAction(new SeperatorDesignerAction(editCategory, 220));
 
-    addDesignerAction(new ModelNodeAction(
-                          resetPositionCommandId,
-                          resetPositionDisplayName,
-                          Utils::Icon({{":/utils/images/pan.png", Utils::Theme::IconsBaseColor},
-                                      {":/utils/images/iconoverlay_reset.png", Utils::Theme::IconsStopToolBarColor}}).icon(),
-                          resetPositionTooltip, editCategory, QKeySequence("Ctrl+d"),
-                          200,
-                          &resetPosition,
-                          &selectionNotEmptyAndHasXorYProperty));
+    addDesignerAction(
+        new ModelNodeAction(resetPositionCommandId,
+                            resetPositionDisplayName,
+                            Utils::Icon({{":/utils/images/pan.png", Utils::Theme::IconsBaseColor},
+                                         {":/utils/images/iconoverlay_reset.png",
+                                          Utils::Theme::IconsStopToolBarColor}})
+                                .icon(),
+                            resetPositionTooltip,
+                            editCategory,
+                            QKeySequence("Ctrl+d"),
+                            200,
+                            &resetPosition,
+                            &selectionNotEmptyAndHasXorYProperty));
+
+    const QString fontName = "qtds_propertyIconFont.ttf";
+    const QColor iconColorDefault(Theme::getColor(Theme::IconsBaseColor));
+    const QColor iconColorDisabled(Theme::getColor(Theme::IconsDisabledColor));
+    const QString copyUnicode = Theme::getIconUnicode(Theme::Icon::copyStyle);
+    const QString pasteUnicode = Theme::getIconUnicode(Theme::Icon::pasteStyle);
+
+    const auto copyDefault = Utils::StyleHelper::IconFontHelper(copyUnicode,
+                                                                iconColorDefault,
+                                                                QSize(28, 28),
+                                                                QIcon::Normal);
+    const auto copyDisabled = Utils::StyleHelper::IconFontHelper(copyUnicode,
+                                                                 iconColorDisabled,
+                                                                 QSize(28, 28),
+                                                                 QIcon::Disabled);
+    const QIcon copyIcon = Utils::StyleHelper::getIconFromIconFont(fontName,
+                                                                   {copyDefault, copyDisabled});
+
+    const auto pasteDefault = Utils::StyleHelper::IconFontHelper(pasteUnicode,
+                                                                 iconColorDefault,
+                                                                 QSize(28, 28),
+                                                                 QIcon::Normal);
+    const auto pasteDisabled = Utils::StyleHelper::IconFontHelper(pasteUnicode,
+                                                                  iconColorDisabled,
+                                                                  QSize(28, 28),
+                                                                  QIcon::Disabled);
+    const QIcon pasteIcon = Utils::StyleHelper::getIconFromIconFont(fontName,
+                                                                    {pasteDefault, pasteDisabled});
+
+    addDesignerAction(new ModelNodeAction(copyFormatCommandId,
+                                          copyFormatDisplayName,
+                                          copyIcon,
+                                          copyFormatTooltip,
+                                          editCategory,
+                                          QKeySequence(),
+                                          120,
+                                          &copyFormat,
+                                          &propertiesCopyable));
+
+    addDesignerAction(new ModelNodeAction(applyFormatCommandId,
+                                          applyFormatDisplayName,
+                                          pasteIcon,
+                                          applyFormatTooltip,
+                                          editCategory,
+                                          QKeySequence(),
+                                          120,
+                                          &applyFormat,
+                                          &propertiesApplyable));
 
     addDesignerAction(new ModelNodeAction(
                           resetSizeCommandId,
@@ -913,11 +1140,10 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &setVisible,
                           &singleSelectedItem));
 
-    addDesignerAction(new ActionGroup(
-                          anchorsCategoryDisplayName,
-                          anchorsCategory,
-                          priorityAnchorsCategory,
-                          &singleSelectionAndInBaseState));
+    addDesignerAction(new ActionGroup(anchorsCategoryDisplayName,
+                                      anchorsCategory,
+                                      priorityAnchorsCategory,
+                                      &anchorsMenuEnabled));
 
     addDesignerAction(new ModelNodeAction(
                           anchorsFillCommandId,
@@ -959,7 +1185,7 @@ void DesignerActionManager::createDefaultDesignerActions()
     addDesignerAction(new ActionGroup(groupCategoryDisplayName,
                                       groupCategory,
                                       priorityGroupCategory,
-                                      &studioComponentsAvailable));
+                                      &studioComponentsAvailableAndSelectionCanBeLayouted));
 
     addDesignerAction(new ActionGroup(
         flowCategoryDisplayName,
@@ -1197,7 +1423,7 @@ void DesignerActionManager::createDefaultDesignerActions()
                           QKeySequence("shift+g"),
                           60,
                           &layoutGridLayout,
-                          &selectionCanBeLayoutedAndQtQuickLayoutPossible));
+                          &selectionCanBeLayoutedAndQtQuickLayoutPossibleAndNotMCU));
 
     addDesignerAction(new SeperatorDesignerAction(layoutCategory, 50));
 
@@ -1233,6 +1459,17 @@ void DesignerActionManager::createDefaultDesignerActions()
                           &goIntoComponentOperation,
                           &selectionIsComponent));
 
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          editAnnotationCommandId,
+                          editAnnotationDisplayName,
+                          {},
+                          rootCategory,
+                          QKeySequence(),
+                          (priorityLast+6),
+                          &editAnnotation,
+                          &singleSelection,
+                          &singleSelection));
+
     addDesignerAction(new ModelNodeContextMenuAction(
                           goToImplementationCommandId,
                           goToImplementationDisplayName,
@@ -1281,29 +1518,89 @@ void DesignerActionManager::createDefaultDesignerActions()
     addDesignerAction(new ChangeStyleAction());
 
     addDesignerAction(new EditListModelAction);
+
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          openSignalDialogCommandId,
+                          openSignalDialogDisplayName,
+                          {},
+                          rootCategory,
+                          QKeySequence(),
+                          66,
+                          &openSignalDialog,
+                          &singleSelectionAndHasSlotTrigger));
+
+    addDesignerAction(new ModelNodeContextMenuAction(
+                          update3DAssetCommandId,
+                          update3DAssetDisplayName,
+                          {},
+                          rootCategory,
+                          QKeySequence(),
+                          priorityGenericToolBar,
+                          &updateImported3DAsset,
+                          &selectionIsImported3DAsset,
+                          &selectionIsImported3DAsset));
 }
 
 void DesignerActionManager::createDefaultAddResourceHandler()
 {
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addImagesDisplayString,
-                                                  "*.png",
-                                                  ModelNodeOperations::addImageToProject));
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addImagesDisplayString,
-                                                  "*.jpg",
-                                                  ModelNodeOperations::addImageToProject));
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addImagesDisplayString,
-                                                  "*.bmp",
-                                                  ModelNodeOperations::addImageToProject));
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addImagesDisplayString,
-                                                  "*.svg",
-                                                  ModelNodeOperations::addImageToProject));
+    auto registerHandlers = [this](const QStringList &exts, AddResourceOperation op,
+                                   const QString &category) {
+        for (const QString &ext : exts)
+            registerAddResourceHandler(AddResourceHandler(category, ext, op));
+    };
 
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addFontsDisplayString,
-                                                  "*.ttf",
-                                                  ModelNodeOperations::addFontToProject));
-    registerAddResourceHandler(AddResourceHandler(ComponentCoreConstants::addFontsDisplayString,
-                                                  "*.otf",
-                                                  ModelNodeOperations::addFontToProject));
+    // The filters will be displayed in reverse order to these lists in file dialog,
+    // so declare most common types last
+    registerHandlers({"*.webp",  "*.hdr", "*.svg", "*.bmp", "*.jpg", "*.png"},
+                     ModelNodeOperations::addImageToProject,
+                     ComponentCoreConstants::addImagesDisplayString);
+    registerHandlers({"*.otf", "*.ttf"},
+                     ModelNodeOperations::addFontToProject,
+                     ComponentCoreConstants::addFontsDisplayString);
+    registerHandlers({"*.wav"},
+                     ModelNodeOperations::addSoundToProject,
+                     ComponentCoreConstants::addSoundsDisplayString);
+    registerHandlers({"*.glsl", "*.glslv", "*.glslf", "*.vsh", "*.fsh", "*.vert", "*.frag"},
+                     ModelNodeOperations::addShaderToProject,
+                     ComponentCoreConstants::addShadersDisplayString);
+}
+
+void DesignerActionManager::createDefaultModelNodePreviewImageHandlers()
+{
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick.Image",
+                                             ModelNodeOperations::previewImageDataForImageNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick.BorderImage",
+                                             ModelNodeOperations::previewImageDataForImageNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("Qt.SafeRenderer.SafeRendererImage",
+                                             ModelNodeOperations::previewImageDataForImageNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("Qt.SafeRenderer.SafeRendererPicture",
+                                             ModelNodeOperations::previewImageDataForImageNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick3D.Texture",
+                                             ModelNodeOperations::previewImageDataForImageNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick3D.Material",
+                                             ModelNodeOperations::previewImageDataForGenericNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick3D.Model",
+                                             ModelNodeOperations::previewImageDataForGenericNode));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick3D.Node",
+                                             ModelNodeOperations::previewImageDataForGenericNode,
+                                             true));
+    registerModelNodePreviewHandler(
+                ModelNodePreviewImageHandler("QtQuick.Item",
+                                             ModelNodeOperations::previewImageDataForGenericNode,
+                                             true));
+
+    // TODO - Disabled until QTBUG-86616 is fixed
+//    registerModelNodePreviewHandler(
+//                ModelNodePreviewImageHandler("QtQuick3D.Effect",
+//                                             ModelNodeOperations::previewImageDataFor3DNode));
 }
 
 void DesignerActionManager::addDesignerAction(ActionInterface *newAction)
@@ -1317,6 +1614,16 @@ void DesignerActionManager::addCreatorCommand(Core::Command *command, const QByt
     addDesignerAction(new CommandAction(command, category, priority, overrideIcon));
 }
 
+QList<QSharedPointer<ActionInterface> > DesignerActionManager::actionsForTargetView(const ActionInterface::TargetView &target)
+{
+    QList<QSharedPointer<ActionInterface> > out;
+    for (auto interface : qAsConst(m_designerActions))
+        if (interface->targetView() == target)
+            out << interface;
+
+    return out;
+}
+
 QList<ActionInterface* > DesignerActionManager::designerActions() const
 {
     return Utils::transform(m_designerActions, [](const QSharedPointer<ActionInterface> &pointer) {
@@ -1324,6 +1631,14 @@ QList<ActionInterface* > DesignerActionManager::designerActions() const
     });
 }
 
+ActionInterface *DesignerActionManager::actionByMenuId(const QByteArray &id)
+{
+    for (const auto &action : m_designerActions)
+        if (action->menuId() == id)
+            return action.data();
+    return nullptr;
+}
+
 DesignerActionManager::DesignerActionManager(DesignerActionManagerView *designerActionManagerView)
     : m_designerActionManagerView(designerActionManagerView)
 {
