@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,17 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -36,7 +36,8 @@ namespace Internal {
 DebugViewWidget::DebugViewWidget(QWidget *parent) : QWidget(parent)
 {
     m_ui.setupUi(this);
-    connect(m_ui.enabledCheckBox, SIGNAL(toggled(bool)), this, SLOT(enabledCheckBoxToggled(bool)));
+    connect(m_ui.enabledCheckBox, &QAbstractButton::toggled,
+            this, &DebugViewWidget::enabledCheckBoxToggled);
 }
 
 void DebugViewWidget::addLogMessage(const QString &topic, const QString &message, bool highlight)
@@ -67,19 +68,30 @@ void DebugViewWidget::addErrorMessage(const QString &topic, const QString &messa
 void DebugViewWidget::addLogInstanceMessage(const QString &topic, const QString &message, bool highlight)
 {
     if (highlight) {
-        m_ui.instanceLog->appendHtml(QStringLiteral("<b><font color=\"blue\">")
-                                  + topic
-                                  + QStringLiteral("</b><br>")
-                                  + message);
+        m_ui.instanceLog->appendHtml("<b><font color=\"blue\">"
+                                     + topic
+                                     + "</b><br>"
+                                     + "<p>"
+                                     + message
+                                     + "</p>"
+                                     + "<br>");
 
     } else {
-        m_ui.instanceLog->appendHtml(QStringLiteral("<b>")
-                                  + topic
-                                  + QStringLiteral("</b><br>")
-                                  + message);
+        m_ui.instanceLog->appendHtml("<b>"
+                                     + topic
+                                     + "</b><br>"
+                                     + "<p>"
+                                     + message
+                                     + "</p>"
+                                     + "<br>");
     }
 }
 
+void DebugViewWidget::setPuppetStatus(const QString &text)
+{
+    m_ui.instanceStatus->setPlainText(text);
+}
+
 void DebugViewWidget::setDebugViewEnabled(bool b)
 {
     if (m_ui.enabledCheckBox->isChecked() != b)
@@ -88,9 +100,7 @@ void DebugViewWidget::setDebugViewEnabled(bool b)
 
 void DebugViewWidget::enabledCheckBoxToggled(bool b)
 {
-    DesignerSettings settings = QmlDesignerPlugin::instance()->settings();
-    settings.warningsInDesigner = b;
-    QmlDesignerPlugin::instance()->setSettings(settings);
+    DesignerSettings::setValue(DesignerSettingsKey::WARNING_FOR_FEATURES_IN_DESIGNER, b);
 }
 
 } //namespace Internal
