@@ -34,21 +34,22 @@ class NavigatorTreeModel;
 class IconCheckboxItemDelegate : public QStyledItemDelegate
 {
 public:
-    explicit IconCheckboxItemDelegate(QObject *parent,
-                                      const QPixmap &checkedPixmap,
-                                      const QPixmap &uncheckedPixmap,
-                                      NavigatorTreeModel *treeModel);
+    explicit IconCheckboxItemDelegate(QObject *parent, const QIcon &icon);
 
     QSize sizeHint(const QStyleOptionViewItem &option,
-                   const QModelIndex &index) const;
+                   const QModelIndex &index) const override;
 
     void paint(QPainter *painter,
                const QStyleOptionViewItem &option,
-               const QModelIndex &index) const;
+               const QModelIndex &index) const override;
+
+protected:
+    bool editorEvent(QEvent *event,
+                     QAbstractItemModel *model,
+                     const QStyleOptionViewItem &option,
+                     const QModelIndex &index) override;
 
 private:
-    const QPixmap m_checkedPixmap;
-    const QPixmap m_uncheckedPixmap;
-    NavigatorTreeModel *m_navigatorTreeModel;
+    const QIcon m_icon;
 };
 } // namespace QmlDesigner
