@@ -41,7 +41,7 @@ public:
     void indent(int offset, int length) override;
     void indentLines(int startLine, int endLine) override;
 
-    int indentDepth() const override;
+    TextEditor::TabSettings tabSettings() const override;
 
     void startGroup() override;
     void flushGroup() override;
@@ -62,11 +62,14 @@ public:
     { return false; }
 
 private:
+    void handleOriginalTextChanged();
+
     TextModifier *m_originalModifier;
     int m_componentStartOffset;
     int m_componentEndOffset;
     int m_rootStartOffset;
     int m_startLength;
+    QString m_originalText;
 };
 
 } // namespace QmlDesigner
