@@ -25,6 +25,7 @@
 
 #include "nodemetainfo.h"
 #include "model.h"
+#include "model/model_p.h"
 
 #include "metainfo.h"
 #include <enumeration.h>
@@ -41,6 +42,7 @@
 #include <languageutils/fakemetaobject.h>
 
 #include <utils/qtcassert.h>
+#include <utils/algorithm.h>
 
 namespace QmlDesigner {
 
@@ -49,8 +51,8 @@ namespace Internal {
 struct TypeDescription
 {
     QString className;
-    int minorVersion;
-    int majorVersion;
+    int minorVersion{};
+    int majorVersion{};
 };
 
 } //Internal
@@ -76,22 +78,28 @@ namespace Internal {
 
 using namespace QmlJS;
 
-typedef QPair<PropertyName, TypeName> PropertyInfo;
+using PropertyInfo = QPair<PropertyName, TypeName>;
 
 QVector<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &context, bool local = false, int rec = 0);
 
-static TypeName resolveTypeName(const ASTPropertyReference *ref, const ContextPtr &context,  QVector<PropertyInfo> &dotProperties)
+static TypeName resolveTypeName(const ASTPropertyReference *ref, const ContextPtr &context, QVector<PropertyInfo> &dotProperties)
 {
     TypeName type = "unknown";
 
-    if (!ref->ast()->memberType.isEmpty()) {
-        type = ref->ast()->memberType.toUtf8();
+    if (ref->ast()->propertyToken.isValid()) {
+        type = ref->ast()->memberType->name.toUtf8();
+
+        const Value *value = context->lookupReference(ref);
+
+        if (!value)
+            return type;
+
+        if (const CppComponentValue * componentObjectValue = value->asCppComponentValue()) {
+            type = componentObjectValue->className().toUtf8();
+            dotProperties = getObjectTypes(componentObjectValue, context);
+        }
 
         if (type == "alias") {
-            const Value *value = context->lookupReference(ref);
-
-            if (!value)
-                return type;
 
             if (const ASTObjectValue * astObjectValue = value->asAstObjectValue()) {
                 if (astObjectValue->typeName()) {
@@ -285,32 +293,31 @@ public:
         if (ref) {
             QVector<PropertyInfo> dotProperties;
             const TypeName type = resolveTypeName(ref, m_context, dotProperties);
-            m_properties.append(qMakePair(propertyName, type));
+            m_properties.append({propertyName, type});
             if (!dotProperties.isEmpty()) {
-                foreach (const PropertyInfo &propertyInfo, dotProperties) {
+                for (const PropertyInfo &propertyInfo : qAsConst(dotProperties)) {
                     PropertyName dotName = propertyInfo.first;
                     TypeName type = propertyInfo.second;
                     dotName = propertyName + '.' + dotName;
-                    m_properties.append(qMakePair(dotName, type));
+                    m_properties.append({dotName, type});
                 }
             }
         } else {
-
             if (const CppComponentValue * cppComponentValue = value_cast<CppComponentValue>(value)) {
                 TypeName qualifiedTypeName = qualifiedTypeNameForContext(cppComponentValue,
                     m_context->viewerContext(), *m_context->snapshot().importDependencies()).toUtf8();
-                m_properties.append(qMakePair(propertyName, qualifiedTypeName));
+                m_properties.append({propertyName, qualifiedTypeName});
             } else {
                 TypeId typeId;
                 TypeName typeName = typeId(value).toUtf8();
-                if (typeName == "number") {
-                    if (value->asIntValue()) {
-                        typeName = "int";
-                    } else {
-                        typeName = "real";
-                    }
-                }
-                m_properties.append(qMakePair(propertyName, typeName));
+
+                if (typeName == "Function")
+                    return processSlot(name, value);
+
+                if (typeName == "number")
+                    typeName = value->asIntValue() ? "int" : "real";
+
+                m_properties.append({propertyName, typeName});
             }
         }
         return true;
@@ -322,34 +329,62 @@ public:
         return true;
     }
 
+    bool processSlot(const QString &name, const Value * /*value*/) override
+    {
+        m_slots.append(name.toUtf8());
+        return true;
+    }
+
     QVector<PropertyInfo> properties() const { return m_properties; }
 
     PropertyNameList signalList() const { return m_signals; }
+    PropertyNameList slotList() const { return m_slots; }
 
 private:
     QVector<PropertyInfo> m_properties;
     PropertyNameList m_signals;
+    PropertyNameList m_slots;
     const ContextPtr m_context;
 };
 
 static inline bool isValueType(const TypeName &type)
 {
-    static const PropertyTypeList objectValuesList({"QFont", "QPoint", "QPointF",
-        "QSize", "QSizeF", "QVector3D", "QVector2D"});
+    static const PropertyTypeList objectValuesList({"QFont",
+                                                    "QPoint",
+                                                    "QPointF",
+                                                    "QSize",
+                                                    "QSizeF",
+                                                    "QVector2D",
+                                                    "QVector3D",
+                                                    "QVector4D",
+                                                    "vector2d",
+                                                    "vector3d",
+                                                    "vector4d",
+                                                    "font"});
     return objectValuesList.contains(type);
 }
 
 static inline bool isValueType(const QString &type)
 {
-    static const QStringList objectValuesList({"QFont", "QPoint", "QPointF",
-        "QSize", "QSizeF", "QVector3D", "QVector2D"});
+    static const QStringList objectValuesList({"QFont",
+                                               "QPoint",
+                                               "QPointF",
+                                               "QSize",
+                                               "QSizeF",
+                                               "QVector2D",
+                                               "QVector3D",
+                                               "QVector4D",
+                                               "vector2d",
+                                               "vector3d",
+                                               "vector4d",
+                                               "font"});
     return objectValuesList.contains(type);
 }
 
 const CppComponentValue *findQmlPrototype(const ObjectValue *ov, const ContextPtr &context)
 {
     if (!ov)
-        return 0;
+        return nullptr;
 
     const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov);
     if (qmlValue)
@@ -412,7 +447,7 @@ QVector<PropertyInfo> getQmlTypes(const CppComponentValue *objectValue, const Co
     PropertyMemberProcessor processor(context);
     objectValue->processMembers(&processor);
 
-    foreach (const PropertyInfo &property, processor.properties()) {
+    for (const PropertyInfo &property : processor.properties()) {
         const PropertyName name = property.first;
         const QString nameAsString = QString::fromUtf8(name);
         if (!objectValue->isWritable(nameAsString) && objectValue->isPointer(nameAsString)) {
@@ -423,7 +458,7 @@ QVector<PropertyInfo> getQmlTypes(const CppComponentValue *objectValue, const Co
                 foreach (const PropertyInfo &propertyInfo, dotProperties) {
                     const PropertyName dotName = name + '.' + propertyInfo.first;
                     const TypeName type = propertyInfo.second;
-                    propertyList.append(qMakePair(dotName, type));
+                    propertyList.append({dotName, type});
                 }
             }
         }
@@ -434,7 +469,7 @@ QVector<PropertyInfo> getQmlTypes(const CppComponentValue *objectValue, const Co
                 foreach (const PropertyInfo &propertyInfo, dotProperties) {
                     const PropertyName dotName = name + '.' + propertyInfo.first;
                     const TypeName type = propertyInfo.second;
-                    propertyList.append(qMakePair(dotName, type));
+                    propertyList.append({dotName, type});
                 }
             }
         }
@@ -445,7 +480,7 @@ QVector<PropertyInfo> getQmlTypes(const CppComponentValue *objectValue, const Co
         if (type == "unknown" && objectValue->hasProperty(nameAsString))
             type = objectValue->propertyType(nameAsString).toUtf8();
 
-        propertyList.append(qMakePair(name, type));
+        propertyList.append({name, type});
     }
 
     if (!local)
@@ -472,13 +507,41 @@ PropertyNameList getSignals(const ObjectValue *objectValue, const ContextPtr &co
     QList<const ObjectValue *> objects = prototypeIterator.all();
 
     if (!local) {
-        foreach (const ObjectValue *prototype, objects)
+        for (const ObjectValue *prototype : objects)
             signalList.append(getSignals(prototype, context, true));
     }
 
     return signalList;
 }
 
+PropertyNameList getSlots(const ObjectValue *objectValue, const ContextPtr &context, bool local = false)
+{
+    PropertyNameList slotList;
+
+    if (!objectValue)
+        return slotList;
+    if (objectValue->className().isEmpty())
+        return slotList;
+
+    PropertyMemberProcessor processor(context);
+    objectValue->processMembers(&processor);
+
+    if (const ASTObjectValue *astObjectValue = objectValue->asAstObjectValue())
+        astObjectValue->processMembers(&processor);
+
+    slotList.append(processor.slotList());
+
+    PrototypeIterator prototypeIterator(objectValue, context);
+    const QList<const ObjectValue *> objects = prototypeIterator.all();
+
+    if (!local) {
+        for (const ObjectValue *prototype : objects)
+            slotList.append(getSlots(prototype, context, true));
+    }
+
+    return slotList;
+}
+
 QVector<PropertyInfo> getObjectTypes(const ObjectValue *objectValue, const ContextPtr &context, bool local, int rec)
 {
     QVector<PropertyInfo> propertyList;
@@ -494,7 +557,31 @@ QVector<PropertyInfo> getObjectTypes(const ObjectValue *objectValue, const Conte
     PropertyMemberProcessor processor(context);
     objectValue->processMembers(&processor);
 
-    propertyList.append(processor.properties());
+    const auto props = processor.properties();
+
+    for (const PropertyInfo &property : props) {
+        const PropertyName name = property.first;
+        const QString nameAsString = QString::fromUtf8(name);
+
+        if (isValueType(property.second)) {
+            const Value *dotValue = objectValue->lookupMember(nameAsString, context);
+            if (!dotValue)
+                continue;
+
+            if (const Reference *ref = dotValue->asReference())
+                dotValue = context->lookupReference(ref);
+
+            if (const ObjectValue *dotObjectValue = dotValue->asObjectValue()) {
+                const QVector<PropertyInfo> dotProperties = getObjectTypes(dotObjectValue, context, false, rec + 1);
+                for (const PropertyInfo &propertyInfo : dotProperties) {
+                    const PropertyName dotName = name + '.' + propertyInfo.first;
+                    const TypeName type = propertyInfo.second;
+                    propertyList.append({dotName, type});
+                }
+            }
+        }
+        propertyList.append(property);
+    }
 
     if (!local) {
         const ObjectValue* prototype = objectValue->prototype(context);
@@ -516,15 +603,16 @@ QVector<PropertyInfo> getObjectTypes(const ObjectValue *objectValue, const Conte
 class NodeMetaInfoPrivate
 {
 public:
-    typedef QSharedPointer<NodeMetaInfoPrivate> Pointer;
-    NodeMetaInfoPrivate();
-    ~NodeMetaInfoPrivate() {}
+    using Pointer = QSharedPointer<NodeMetaInfoPrivate>;
+    NodeMetaInfoPrivate() = default;
+    ~NodeMetaInfoPrivate() = default;
 
     bool isValid() const;
     bool isFileComponent() const;
     PropertyNameList properties() const;
     PropertyNameList localProperties() const;
     PropertyNameList signalNames() const;
+    PropertyNameList slotNames() const;
     PropertyName defaultPropertyName() const;
     TypeName propertyType(const PropertyName &propertyName) const;
 
@@ -535,7 +623,6 @@ public:
     bool isPropertyPointer(const PropertyName &propertyName) const;
     bool isPropertyList(const PropertyName &propertyName) const;
     bool isPropertyEnum(const PropertyName &propertyName) const;
-    QString propertyEnumScope(const PropertyName &propertyName) const;
     QStringList keysForEnum(const QString &enumName) const;
     bool cleverCheckType(const TypeName &otherType) const;
     QVariant::Type variantTypeId(const PropertyName &properyName) const;
@@ -547,7 +634,6 @@ public:
 
     QByteArray cppPackageName() const;
 
-    QString componentSource() const;
     QString componentFileName() const;
     QString importDirectoryPath() const;
 
@@ -556,9 +642,6 @@ public:
     QSet<QByteArray> &prototypeCachePositives();
     QSet<QByteArray> &prototypeCacheNegatives();
 
-    static void clearCache();
-
-
 private:
     NodeMetaInfoPrivate(Model *model, TypeName type, int maj = -1, int min = -1);
 
@@ -581,6 +664,7 @@ private:
     bool m_isFileComponent = false;
     PropertyNameList m_properties;
     PropertyNameList m_signals;
+    PropertyNameList m_slots;
     QList<TypeName> m_propertyTypes;
     PropertyNameList m_localProperties;
     PropertyName m_defaultPropertyName;
@@ -593,13 +677,10 @@ private:
     const Document *document() const;
 
     QPointer<Model> m_model;
-    static QHash<TypeName, Pointer> m_nodeMetaInfoCache;
     const ObjectValue *m_objectValue = nullptr;
     bool m_propertiesSetup = false;
 };
 
-QHash<TypeName, NodeMetaInfoPrivate::Pointer> NodeMetaInfoPrivate::m_nodeMetaInfoCache;
-
 bool NodeMetaInfoPrivate::isFileComponent() const
 {
     return m_isFileComponent;
@@ -625,6 +706,12 @@ PropertyNameList NodeMetaInfoPrivate::signalNames() const
     return m_signals;
 }
 
+PropertyNameList NodeMetaInfoPrivate::slotNames() const
+{
+    ensureProperties();
+    return m_slots;
+}
+
 QSet<QByteArray> &NodeMetaInfoPrivate::prototypeCachePositives()
 {
     return m_prototypeCachePositives;
@@ -635,11 +722,6 @@ QSet<QByteArray> &NodeMetaInfoPrivate::prototypeCacheNegatives()
     return m_prototypeCacheNegatives;
 }
 
-void NodeMetaInfoPrivate::clearCache()
-{
-    m_nodeMetaInfoCache.clear();
-}
-
 PropertyName NodeMetaInfoPrivate::defaultPropertyName() const
 {
     if (!m_defaultPropertyName.isEmpty())
@@ -649,30 +731,20 @@ PropertyName NodeMetaInfoPrivate::defaultPropertyName() const
 
 static inline TypeName stringIdentifier( const TypeName &type, int maj, int min)
 {
-    return type + QString::number(maj).toLatin1() + '_' + QString::number(min).toLatin1();
+    return type + QByteArray::number(maj) + '_' + QByteArray::number(min);
 }
 
 NodeMetaInfoPrivate::Pointer NodeMetaInfoPrivate::create(Model *model, const TypeName &type, int major, int minor)
 {
-    if (m_nodeMetaInfoCache.contains(stringIdentifier(type, major, minor))) {
-        const Pointer &info = m_nodeMetaInfoCache.value(stringIdentifier(type, major, minor));
-        if (info->model() == model)
-            return info;
-        else
-            m_nodeMetaInfoCache.clear();
-    }
+    if (model->d->m_nodeMetaInfoCache.contains(stringIdentifier(type, major, minor)))
+        return model->d->m_nodeMetaInfoCache.value(stringIdentifier(type, major, minor));
 
     Pointer newData(new NodeMetaInfoPrivate(model, type, major, minor));
     if (newData->isValid())
-        m_nodeMetaInfoCache.insert(stringIdentifier(type, major, minor), newData);
+        model->d->m_nodeMetaInfoCache.insert(stringIdentifier(type, major, minor), newData);
     return newData;
 }
 
-NodeMetaInfoPrivate::NodeMetaInfoPrivate() : m_isValid(false)
-{
-
-}
-
 NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, TypeName type, int maj, int min)
     : m_qualfiedTypeName(type)
     , m_majorVersion(maj)
@@ -708,7 +780,7 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, TypeName type, int maj, i
                 } else {
                     m_isFileComponent = true;
                     const Imports *imports = context()->imports(document());
-                    ImportInfo importInfo = imports->info(lookupNameComponent().last(), context().data());
+                    ImportInfo importInfo = imports->info(lookupNameComponent().constLast(), context().data());
                     if (importInfo.isValid() && importInfo.type() == ImportType::Library) {
                         m_majorVersion = importInfo.version().majorVersion();
                         m_minorVersion = importInfo.version().minorVersion();
@@ -727,8 +799,8 @@ const CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() const
 {
     const QList<TypeName> nameComponents = m_qualfiedTypeName.split('.');
     if (nameComponents.size() < 2)
-        return 0;
-    const TypeName type = nameComponents.last();
+        return nullptr;
+    const TypeName &type = nameComponents.constLast();
 
     TypeName module;
     for (int i = 0; i < nameComponents.size() - 1; ++i) {
@@ -738,16 +810,20 @@ const CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() const
     }
 
     // get the qml object value that's available in the document
-    foreach (const QmlJS::Import &import, context()->imports(document())->all()) {
-        if (import.info.path() != QString::fromUtf8(module))
-            continue;
-        const Value *lookupResult = import.object->lookupMember(QString::fromUtf8(type), context());
-        const CppComponentValue *cppValue = value_cast<CppComponentValue>(lookupResult);
-        if (cppValue
-                && (m_majorVersion == -1 || m_majorVersion == cppValue->componentVersion().majorVersion())
-                && (m_minorVersion == -1 || m_minorVersion == cppValue->componentVersion().minorVersion())
-                )
-            return cppValue;
+    const QmlJS::Imports *importsPtr = context()->imports(document());
+    if (importsPtr) {
+        const QList<QmlJS::Import> &imports = importsPtr->all();
+        foreach (const QmlJS::Import &import, imports) {
+            if (import.info.path() != QString::fromUtf8(module))
+                continue;
+            const Value *lookupResult = import.object->lookupMember(QString::fromUtf8(type), context());
+            const CppComponentValue *cppValue = value_cast<CppComponentValue>(lookupResult);
+            if (cppValue
+                    && (m_majorVersion == -1 || m_majorVersion == cppValue->componentVersion().majorVersion())
+                    && (m_minorVersion == -1 || m_minorVersion == cppValue->componentVersion().minorVersion())
+                    )
+                return cppValue;
+        }
     }
 
     const CppComponentValue *value = value_cast<CppComponentValue>(getObjectValue());
@@ -784,14 +860,14 @@ ContextPtr NodeMetaInfoPrivate::context() const
 {
     if (m_model && m_model->rewriterView() && m_model->rewriterView()->scopeChain())
         return m_model->rewriterView()->scopeChain()->context();
-    return ContextPtr(0);
+    return ContextPtr(nullptr);
 }
 
 const Document *NodeMetaInfoPrivate::document() const
 {
     if (m_model && m_model->rewriterView())
         return m_model->rewriterView()->document();
-    return 0;
+    return nullptr;
 }
 
 void NodeMetaInfoPrivate::setupLocalPropertyInfo(const QVector<PropertyInfo> &localPropertyInfos)
@@ -820,8 +896,8 @@ bool NodeMetaInfoPrivate::isPropertyWritable(const PropertyName &propertyName) c
 
     if (propertyName.contains('.')) {
         const PropertyNameList parts = propertyName.split('.');
-        const PropertyName objectName = parts.first();
-        const PropertyName rawPropertyName = parts.last();
+        const PropertyName &objectName = parts.constFirst();
+        const PropertyName &rawPropertyName = parts.constLast();
         const TypeName objectType = propertyType(objectName);
 
         if (isValueType(objectType))
@@ -853,8 +929,8 @@ bool NodeMetaInfoPrivate::isPropertyList(const PropertyName &propertyName) const
 
     if (propertyName.contains('.')) {
         const PropertyNameList parts = propertyName.split('.');
-        const PropertyName objectName = parts.first();
-        const PropertyName rawPropertyName = parts.last();
+        const PropertyName &objectName = parts.constFirst();
+        const PropertyName &rawPropertyName = parts.constLast();
         const TypeName objectType = propertyType(objectName);
 
         if (isValueType(objectType))
@@ -870,6 +946,12 @@ bool NodeMetaInfoPrivate::isPropertyList(const PropertyName &propertyName) const
     const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
+
+    if (!qmlObjectValue->hasProperty(QString::fromUtf8(propertyName))) {
+        const TypeName typeName = propertyType(propertyName);
+        return (typeName == "Item"  || typeName == "QtObject");
+    }
+
     return qmlObjectValue->isListProperty(QString::fromUtf8(propertyName));
 }
 
@@ -882,8 +964,8 @@ bool NodeMetaInfoPrivate::isPropertyPointer(const PropertyName &propertyName) co
 
     if (propertyName.contains('.')) {
         const PropertyNameList parts = propertyName.split('.');
-        const PropertyName objectName = parts.first();
-        const PropertyName rawPropertyName = parts.last();
+        const PropertyName &objectName = parts.constFirst();
+        const PropertyName &rawPropertyName = parts.constLast();
         const TypeName objectType = propertyType(objectName);
 
         if (isValueType(objectType))
@@ -914,8 +996,8 @@ bool NodeMetaInfoPrivate::isPropertyEnum(const PropertyName &propertyName) const
 
     if (propertyName.contains('.')) {
         const PropertyNameList parts = propertyName.split('.');
-        const PropertyName objectName = parts.first();
-        const PropertyName rawPropertyName = parts.last();
+        const PropertyName &objectName = parts.constFirst();
+        const PropertyName &rawPropertyName = parts.constLast();
         const TypeName objectType = propertyType(objectName);
 
         if (isValueType(objectType))
@@ -934,61 +1016,12 @@ bool NodeMetaInfoPrivate::isPropertyEnum(const PropertyName &propertyName) const
     return qmlObjectValue->getEnum(QString::fromUtf8(propertyType(propertyName))).isValid();
 }
 
-QString NodeMetaInfoPrivate::propertyEnumScope(const PropertyName &propertyName) const
-{
-    if (!isValid())
-        return QString();
-
-    ensureProperties();
-
-    if (propertyType(propertyName).contains("Qt::"))
-        return QStringLiteral("Qt");
-
-    if (propertyName.contains('.')) {
-        const PropertyNameList parts = propertyName.split('.');
-        const PropertyName objectName = parts.first();
-        const PropertyName rawPropertyName = parts.last();
-        const TypeName objectType = propertyType(objectName);
-
-        if (isValueType(objectType))
-            return QString();
-
-        QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
-        if (objectInfo->isValid())
-            return objectInfo->propertyEnumScope(rawPropertyName);
-        else
-            return QString();
-    }
-
-    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
-    if (!qmlObjectValue)
-        return QString();
-    const CppComponentValue *definedIn = 0;
-    qmlObjectValue->getEnum(QString::fromUtf8(propertyType(propertyName)), &definedIn);
-    if (definedIn) {
-        QString nonCppPackage;
-        foreach (const LanguageUtils::FakeMetaObject::Export &qmlExport, definedIn->metaObject()->exports()) {
-            if (qmlExport.package != QStringLiteral("<cpp>"))
-                nonCppPackage = qmlExport.package;
-        }
-
-        const LanguageUtils::FakeMetaObject::Export qmlExport =
-                definedIn->metaObject()->exportInPackage(nonCppPackage);
-        if (qmlExport.isValid())
-            return qmlExport.type;
-
-        return definedIn->className();
-    }
-
-    return QString();
-}
-
 static QByteArray getUnqualifiedName(const QByteArray &name)
 {
     const QList<QByteArray> nameComponents = name.split('.');
     if (nameComponents.size() < 2)
         return name;
-    return nameComponents.last();
+    return nameComponents.constLast();
 }
 
 static QByteArray getPackage(const QByteArray &name)
@@ -1001,6 +1034,14 @@ static QByteArray getPackage(const QByteArray &name)
     return nameComponents.join('.');
 }
 
+
+QList<TypeName> qtObjectTypes()
+{
+    static QList<TypeName> typeNames = {"QML.QtObject", "QtQml.QtObject", "<cpp>.QObject"};
+
+    return typeNames;
+}
+
 bool NodeMetaInfoPrivate::cleverCheckType(const TypeName &otherType) const
 {
     if (otherType == qualfiedTypeName())
@@ -1009,6 +1050,9 @@ bool NodeMetaInfoPrivate::cleverCheckType(const TypeName &otherType) const
     if (isFileComponent())
         return false;
 
+    if (qtObjectTypes().contains(qualfiedTypeName()) && qtObjectTypes().contains(otherType))
+        return true;
+
     const QByteArray typeName = getUnqualifiedName(otherType);
     const QByteArray package = getPackage(otherType);
 
@@ -1028,9 +1072,9 @@ bool NodeMetaInfoPrivate::cleverCheckType(const TypeName &otherType) const
     return typeName == convertedName.toUtf8();
 }
 
-QVariant::Type NodeMetaInfoPrivate::variantTypeId(const PropertyName &properyName) const
+QVariant::Type NodeMetaInfoPrivate::variantTypeId(const PropertyName &propertyName) const
 {
-    TypeName typeName = propertyType(properyName);
+    TypeName typeName = propertyType(propertyName);
     if (typeName == "string")
         return QVariant::String;
 
@@ -1061,6 +1105,15 @@ QVariant::Type NodeMetaInfoPrivate::variantTypeId(const PropertyName &properyNam
     if (typeName == "var")
         return QVariant::UserType;
 
+    if (typeName == "vector2d")
+        return QVariant::Vector2D;
+
+    if (typeName == "vector3d")
+        return QVariant::Vector3D;
+
+    if (typeName == "vector4d")
+        return QVariant::Vector4D;
+
     return QVariant::nameToType(typeName.data());
 }
 
@@ -1105,17 +1158,6 @@ QByteArray NodeMetaInfoPrivate::cppPackageName() const
     return QByteArray();
 }
 
-QString NodeMetaInfoPrivate::componentSource() const
-{
-    if (isFileComponent()) {
-        const ASTObjectValue * astObjectValue = value_cast<ASTObjectValue>(getObjectValue());
-        if (astObjectValue)
-            return astObjectValue->document()->source().mid(astObjectValue->typeName()->identifierToken.begin(),
-                                                            astObjectValue->initializer()->rbraceToken.end());
-    }
-    return QString();
-}
-
 QString NodeMetaInfoPrivate::componentFileName() const
 {
     if (isFileComponent()) {
@@ -1137,7 +1179,7 @@ QString NodeMetaInfoPrivate::importDirectoryPath() const
 
     if (isValid()) {
         const Imports *imports = context()->imports(document());
-        ImportInfo importInfo = imports->info(lookupNameComponent().last(), context().data());
+        ImportInfo importInfo = imports->info(lookupNameComponent().constLast(), context().data());
 
         if (importInfo.type() == ImportType::Directory) {
             return importInfo.path();
@@ -1222,7 +1264,21 @@ void NodeMetaInfoPrivate::setupPrototypes()
         description.className = ov->className();
         description.minorVersion = -1;
         description.majorVersion = -1;
-        if (const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov)) {
+        if (description.className == "QQuickItem") {
+            /* Ugly hack to recover from wrong prototypes for Item */
+            if (const CppComponentValue *qmlValue = value_cast<CppComponentValue>(
+                    context()->lookupType(document(), {"Item"}))) {
+                description.className = "QtQuick.Item";
+                description.minorVersion = qmlValue->componentVersion().minorVersion();
+                description.majorVersion = qmlValue->componentVersion().majorVersion();
+                m_prototypes.append(description);
+            } else {
+                qWarning() << Q_FUNC_INFO << "Lookup for Item failed";
+            }
+            continue;
+        }
+
+        if (const CppComponentValue *qmlValue = value_cast<CppComponentValue>(ov)) {
             description.minorVersion = qmlValue->componentVersion().minorVersion();
             description.majorVersion = qmlValue->componentVersion().majorVersion();
             LanguageUtils::FakeMetaObject::Export qtquickExport = qmlValue->metaObject()->exportInPackage(QLatin1String("QtQuick"));
@@ -1250,7 +1306,7 @@ void NodeMetaInfoPrivate::setupPrototypes()
             }
             m_prototypes.append(description);
         } else {
-            if (context()->lookupType(document(), QStringList() << ov->className())) {
+            if (context()->lookupType(document(), {ov->className()})) {
                 const Imports *allImports = context()->imports(document());
                 ImportInfo importInfo = allImports->info(description.className, context().data());
 
@@ -1308,6 +1364,7 @@ void NodeMetaInfoPrivate::initialiseProperties()
     setupLocalPropertyInfo(getTypes(m_objectValue, context(), true));
 
     m_signals = getSignals(m_objectValue, context());
+    m_slots = getSlots(m_objectValue, context());
 }
 
 } //namespace Internal
@@ -1317,19 +1374,14 @@ NodeMetaInfo::NodeMetaInfo() : m_privateData(new Internal::NodeMetaInfoPrivate()
 
 }
 
-NodeMetaInfo::NodeMetaInfo(Model *model, TypeName type, int maj, int min) : m_privateData(Internal::NodeMetaInfoPrivate::create(model, type, maj, min))
+NodeMetaInfo::NodeMetaInfo(Model *model, const TypeName &type, int maj, int min) : m_privateData(Internal::NodeMetaInfoPrivate::create(model, type, maj, min))
 {
 
 }
 
-NodeMetaInfo::~NodeMetaInfo()
-{
-}
+NodeMetaInfo::~NodeMetaInfo() = default;
 
-NodeMetaInfo::NodeMetaInfo(const NodeMetaInfo &other)
-    : m_privateData(other.m_privateData)
-{
-}
+NodeMetaInfo::NodeMetaInfo(const NodeMetaInfo &other) = default;
 
 NodeMetaInfo &NodeMetaInfo::operator=(const NodeMetaInfo &other)
 {
@@ -1364,6 +1416,11 @@ PropertyNameList NodeMetaInfo::signalNames() const
     return m_privateData->signalNames();
 }
 
+PropertyNameList NodeMetaInfo::slotNames() const
+{
+    return m_privateData->slotNames();
+}
+
 PropertyNameList NodeMetaInfo::directPropertyNames() const
 {
     return m_privateData->localProperties();
@@ -1404,9 +1461,9 @@ bool NodeMetaInfo::propertyIsPrivate(const PropertyName &propertyName) const
     return propertyName.startsWith("__");
 }
 
-QString NodeMetaInfo::propertyEnumScope(const PropertyName &propertyName) const
+bool NodeMetaInfo::propertyIsPointer(const PropertyName &propertyName) const
 {
-    return m_privateData->propertyEnumScope(propertyName);
+    return m_privateData->isPropertyPointer(propertyName);
 }
 
 QStringList NodeMetaInfo::propertyKeysForEnum(const PropertyName &propertyName) const
@@ -1442,6 +1499,14 @@ QVariant NodeMetaInfo::propertyCastedValue(const PropertyName &propertyName, con
     } else if (typeName == "alias") {
         // TODO: The QML compiler resolves the alias type. We probably should do the same.
         return variant;
+    } else if (typeName == "<cpp>.double") {
+        return variant.toDouble();
+    } else if (typeName == "<cpp>.float") {
+        return variant.toFloat();
+    } else if (typeName == "<cpp>.int") {
+        return variant.toInt();
+    } else if (typeName == "<cpp>.bool") {
+        return variant.toBool();
     } else if (copyVariant.convert(typeId)) {
         return copyVariant;
     }
@@ -1449,32 +1514,24 @@ QVariant NodeMetaInfo::propertyCastedValue(const PropertyName &propertyName, con
     return Internal::PropertyParser::variantFromString(variant.toString());
 }
 
+QList<NodeMetaInfo> NodeMetaInfo::classHierarchy() const
+{
+    QList<NodeMetaInfo> hierarchy = {*this};
+    hierarchy.append(superClasses());
+    return hierarchy;
+}
+
 QList<NodeMetaInfo> NodeMetaInfo::superClasses() const
 {
-    QList<NodeMetaInfo> list;
-
-    foreach (const Internal::TypeDescription &type,  m_privateData->prototypes()) {
-        list.append(NodeMetaInfo(m_privateData->model(), type.className.toUtf8(), type.majorVersion, type.minorVersion));
-    }
-    return list;
+    Model *model = m_privateData->model();
+    return Utils::transform(m_privateData->prototypes(), [model](const Internal::TypeDescription &type) {
+        return NodeMetaInfo(model, type.className.toUtf8(), type.majorVersion, type.minorVersion);
+    });
 }
 
 NodeMetaInfo NodeMetaInfo::directSuperClass() const
 {
-    QList<NodeMetaInfo> superClassesList = superClasses();
-    if (superClassesList.count() > 1)
-        return superClassesList.at(1);
-    return NodeMetaInfo();
-}
-
-QStringList NodeMetaInfo::superClassNames() const
-{
-    QStringList list;
-
-    foreach (const Internal::TypeDescription &type,  m_privateData->prototypes()) {
-        list.append(type.className);
-    }
-    return list;
+    return superClasses().value(1, NodeMetaInfo());
 }
 
 bool NodeMetaInfo::defaultPropertyIsComponent() const
@@ -1490,6 +1547,11 @@ TypeName NodeMetaInfo::typeName() const
     return m_privateData->qualfiedTypeName();
 }
 
+TypeName NodeMetaInfo::simplifiedTypeName() const
+{
+    return typeName().split('.').constLast();
+}
+
 int NodeMetaInfo::majorVersion() const
 {
     return m_privateData->majorVersion();
@@ -1500,11 +1562,6 @@ int NodeMetaInfo::minorVersion() const
     return m_privateData->minorVersion();
 }
 
-QString NodeMetaInfo::componentSource() const
-{
-    return m_privateData->componentSource();
-}
-
 QString NodeMetaInfo::componentFileName() const
 {
     return m_privateData->componentFileName();
@@ -1515,11 +1572,6 @@ QString NodeMetaInfo::importDirectoryPath() const
     return m_privateData->importDirectoryPath();
 }
 
-bool NodeMetaInfo::hasCustomParser() const
-{
-    return false;
-}
-
 bool NodeMetaInfo::availableInVersion(int majorVersion, int minorVersion) const
 {
     if (majorVersion == -1 && minorVersion == -1)
@@ -1532,7 +1584,7 @@ bool NodeMetaInfo::availableInVersion(int majorVersion, int minorVersion) const
 bool NodeMetaInfo::isSubclassOf(const TypeName &type, int majorVersion, int minorVersion) const
 {
     if (!isValid()) {
-        qWarning() << "NodeMetaInfo is invalid";
+        qWarning() << "NodeMetaInfo is invalid" << type;
         return false;
     }
 
@@ -1563,12 +1615,14 @@ bool NodeMetaInfo::isGraphicalItem() const
 {
     return isSubclassOf("QtQuick.Item")
             || isSubclassOf("QtQuick.Window.Window")
+            || isSubclassOf("QtQuick.Dialogs.Dialog")
             || isSubclassOf("QtQuick.Controls.Popup");
 }
 
-void NodeMetaInfo::clearCache()
+bool NodeMetaInfo::isQmlItem() const
 {
-    Internal::NodeMetaInfoPrivate::clearCache();
+    return isSubclassOf("QtQuick.QtObject")
+            || isSubclassOf("QtQml.QtObject");
 }
 
 bool NodeMetaInfo::isLayoutable() const
