@@ -52,6 +52,8 @@
 
 #include <QMessageBox>
 
+using namespace Utils;
+
 namespace QmlDesigner {
 
 Q_LOGGING_CATEGORY(documentManagerLog, "qtc.qtquickdesigner.documentmanager", QtWarningMsg)
@@ -302,12 +304,13 @@ bool DocumentManager::createFile(const QString &filePath, const QString &content
     textFileFormat.codec = Core::EditorManager::defaultTextCodec();
     QString errorMessage;
 
-    return textFileFormat.writeFile(filePath, contents, &errorMessage);
+    return textFileFormat.writeFile(Utils::FilePath::fromString(filePath), contents, &errorMessage);
 }
 
 void DocumentManager::addFileToVersionControl(const QString &directoryPath, const QString &newFilePath)
 {
-    Core::IVersionControl *versionControl = Core::VcsManager::findVersionControlForDirectory(directoryPath);
+    Core::IVersionControl *versionControl =
+        Core::VcsManager::findVersionControlForDirectory(FilePath::fromString(directoryPath));
     if (versionControl && versionControl->supportsOperation(Core::IVersionControl::AddOperation)) {
         const QMessageBox::StandardButton button
             = QMessageBox::question(Core::ICore::dialogParent(),
@@ -315,7 +318,7 @@ void DocumentManager::addFileToVersionControl(const QString &directoryPath, cons
                                     Core::VcsManager::msgPromptToAddToVcs(QStringList(newFilePath),
                                                                           versionControl),
                                     QMessageBox::Yes | QMessageBox::No);
-        if (button == QMessageBox::Yes && !versionControl->vcsAdd(newFilePath)) {
+        if (button == QMessageBox::Yes && !versionControl->vcsAdd(FilePath::fromString(newFilePath))) {
             Core::AsynchronousMessageBox::warning(Core::VcsManager::msgAddToVcsFailedTitle(),
                                                    Core::VcsManager::msgToAddToVcsFailed(QStringList(newFilePath), versionControl));
         }
@@ -474,7 +477,7 @@ bool DocumentManager::addResourceFileToIsoProject(const QString &resourceFilePro
     if (!projectNode)
         return false;
 
-    if (!projectNode->addFiles({resourceFilePath})) {
+    if (!projectNode->addFiles({Utils::FilePath::fromString(resourceFilePath)})) {
         qCWarning(documentManagerLog) << "Failed to add resource file to" << projectNode->displayName();
         return false;
     }
