@@ -30,9 +30,10 @@
 #include "callgrindcostitem.h"
 #include "callgrindfunction.h"
 
+#include <utils/filepath.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
-#include <QFileDevice>
 #include <QHash>
 #include <QVector>
 #include <QStringList>
@@ -40,7 +41,10 @@
 
 // #define DEBUG_PARSER
 
-namespace {
+using namespace Utils;
+
+namespace Valgrind {
+namespace Callgrind {
 
 static void skipSpace(const char **current, const char *end)
 {
@@ -129,10 +133,6 @@ static int parseNameShorthand(const char **current, const char *end)
     return -1; // invalid
 }
 
-}
-
-namespace Valgrind {
-namespace Callgrind {
 
 class Parser::Private
 {
@@ -149,7 +149,7 @@ public:
         delete data;
     }
 
-    void parse(QIODevice *device);
+    void parse(const FilePath &filePath);
     void parseHeader(QIODevice *device);
 
     using NamePair = QPair<qint64, QString>;
@@ -196,20 +196,22 @@ public:
     QSet<Function *> recursiveFunctions;
 };
 
-void Parser::Private::parse(QIODevice *device)
+void Parser::Private::parse(const FilePath &filePath)
 {
     // be sure to clean up existing data before re-allocating
     // the callee might not have taken the parse data
     delete data;
     data = nullptr;
 
-    QString file;
-    if (auto fileDevice = qobject_cast<QFileDevice *>(device))
-        file = fileDevice->fileName();
-    data = new ParseData(file);
-    parseHeader(device);
-    while (!device->atEnd()) {
-        QByteArray line = device->readLine();
+    const QString path = filePath.path(); // FIXME: Works only accidentally for docker
+    QFile file(path);
+    if (!file.open(QIODevice::ReadOnly))
+        qWarning() << "Could not open file for parsing:" << filePath.toUserOutput();
+
+    data = new ParseData(path);
+    parseHeader(&file);
+    while (!file.atEnd()) {
+        const QByteArray line = file.readLine();
         // empty lines actually have no meaning - only fn= starts a new function
         if (line.length() > 1)
             dispatchLine(line);
@@ -224,7 +226,7 @@ void Parser::Private::parse(QIODevice *device)
     // functions that need to accumulate their calees
     QSet<Function *> pendingFunctions;
     foreach (const CallData &callData, pendingCallees) {
-        Function *calledFunction = 0;
+        Function *calledFunction = nullptr;
         QTC_ASSERT(callData.call, continue);
         QTC_ASSERT(callData.call->caller(), continue);
         foreach (const Function *function, functionLookup.value(callData.calledFunction)) {
@@ -293,11 +295,11 @@ void Parser::Private::parseHeader(QIODevice *device)
             continue;
         } else if (line.startsWith("positions: ")) {
             QString values = getValue(line, 11);
-            data->setPositions(values.split(' ', QString::SkipEmptyParts));
+            data->setPositions(values.split(' ', Qt::SkipEmptyParts));
             addressValuesCount = data->positions().count();
         } else if (line.startsWith("events: ")) {
             QString values = getValue(line, 8);
-            data->setEvents(values.split(' ', QString::SkipEmptyParts));
+            data->setEvents(values.split(' ', Qt::SkipEmptyParts));
             costValuesCount = data->events().count();
         } else if (line.startsWith("version: ")) {
             QString value = getValue(line, 9);
@@ -320,7 +322,7 @@ void Parser::Private::parseHeader(QIODevice *device)
         } else if (line.startsWith("summary: ")) {
             QString values = getValue(line, 9);
             uint i = 0;
-            foreach (const QStringRef &value, values.splitRef(' ', QString::SkipEmptyParts))
+            foreach (const QString &value, values.split(' ', Qt::SkipEmptyParts))
                 data->setTotalCost(i++, value.toULongLong());
         } else if (!line.trimmed().isEmpty()) {
             // handle line and exit parseHeader
@@ -463,7 +465,7 @@ void Parser::Private::parseCostItem(const char *begin, const char *end)
         if (currentCallData.calledFile == -1) {
             currentCallData.calledFile = currentDifferingFile != -1 ? currentDifferingFile : lastFile;
             //HACK: workaround issue where sometimes fi=??? lines are prepended to function calls
-            if (unknownFiles.contains(currentCallData.calledFile))
+            if (unknownFiles.contains(quint64(currentCallData.calledFile)))
                 currentCallData.calledFile = lastFile;
         }
         if (currentCallData.calledObject == -1)
@@ -638,14 +640,13 @@ void Parser::Private::parseCalledObjectFile(const char *begin, const char *end)
 
 //BEGIN Parser
 
-void Parser::parse(QIODevice *device)
+void Parser::parse(const Utils::FilePath &filePath)
 {
-    d->parse(device);
+    d->parse(filePath);
 }
 
-Parser::Parser(QObject *parent)
-    : QObject(parent),
-      d(new Private(this))
+Parser::Parser()
+    : d(new Private(this))
 {
 }
 
