@@ -36,8 +36,11 @@
 #include <clangtools/clangtoolsdiagnostic.h>
 #include <debugger/analyzer/diagnosticlocation.h>
 #include <modelnode.h>
+#include <projectstorage/filestatus.h>
+#include <projectstorage/projectstoragepathwatchertypes.h>
 #include <projectstorage/projectstoragetypes.h>
 #include <projectstorage/sourcepathcachetypes.h>
+#include <sqlite.h>
 #include <sqlitesessionchangeset.h>
 #include <sqlitevalue.h>
 #include <tooltipinfo.h>
@@ -46,8 +49,6 @@
 #include <variantproperty.h>
 #include <qmldesigner/designercore/imagecache/imagecachestorageinterface.h>
 
-#include <sqlite.h>
-
 void PrintTo(const Utf8String &text, ::std::ostream *os)
 {
     *os << text;
@@ -933,6 +934,50 @@ std::ostream &operator<<(std::ostream &out, const Diagnostic &diag) {
 } // namespace ClangTools
 
 namespace QmlDesigner {
+namespace {
+const char *sourceTypeToText(SourceType sourceType)
+{
+    switch (sourceType) {
+    case SourceType::Qml:
+        return "Qml";
+    case SourceType::QmlUi:
+        return "QmlUi";
+    case SourceType::QmlDir:
+        return "QmlDir";
+    case SourceType::QmlTypes:
+        return "QmlTypes";
+    }
+
+    return "";
+}
+} // namespace
+
+std::ostream &operator<<(std::ostream &out, const FileStatus &fileStatus)
+{
+    return out << "(" << fileStatus.sourceId << ", " << fileStatus.size << ", "
+               << fileStatus.lastModified << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, SourceType sourceType)
+{
+    return out << sourceTypeToText(sourceType);
+}
+
+std::ostream &operator<<(std::ostream &out, const ProjectChunkId &id)
+{
+    return out << "(" << id.id << ", " << id.sourceType << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, const IdPaths &idPaths)
+{
+    return out << idPaths.id << ", " << idPaths.sourceIds << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, const WatcherEntry &entry)
+{
+    return out << "(" << entry.sourceId << ", " << entry.sourceContextId << ", " << entry.id << ", "
+               << entry.lastModified << ")";
+}
 
 std::ostream &operator<<(std::ostream &out, const ModelNode &node)
 {
@@ -971,8 +1016,8 @@ TypeAccessSemantics cleanFlags(TypeAccessSemantics accessSemantics)
 const char *typeAccessSemanticsToString(TypeAccessSemantics accessSemantics)
 {
     switch (cleanFlags(accessSemantics)) {
-    case TypeAccessSemantics::Invalid:
-        return "Invalid";
+    case TypeAccessSemantics::None:
+        return "None";
     case TypeAccessSemantics::Reference:
         return "Reference";
     case TypeAccessSemantics::Sequence:
@@ -991,9 +1036,7 @@ bool operator&(TypeAccessSemantics first, TypeAccessSemantics second)
     return static_cast<int>(first) & static_cast<int>(second);
 }
 
-} // namespace
-
-static const char *typeAccessSemanticsFlagsToString(TypeAccessSemantics accessSemantics)
+const char *typeAccessSemanticsFlagsToString(TypeAccessSemantics accessSemantics)
 {
     if (accessSemantics & TypeAccessSemantics::IsEnum)
         return "(IsEnum)";
@@ -1001,15 +1044,48 @@ static const char *typeAccessSemanticsFlagsToString(TypeAccessSemantics accessSe
     return "";
 }
 
+const char *isQualifiedToString(IsQualified isQualified)
+{
+    switch (isQualified) {
+    case IsQualified::No:
+        return "no";
+    case IsQualified::Yes:
+        return "yes";
+    }
+
+    return "";
+}
+
+const char *importKindToText(ImportKind kind)
+{
+    switch (kind) {
+    case ImportKind::Module:
+        return "Module";
+    case ImportKind::Directory:
+        return "Directory";
+    case ImportKind::QmlTypesDependency:
+        return "QmlTypesDependency";
+    }
+
+    return "";
+}
+
+} // namespace
+
 std::ostream &operator<<(std::ostream &out, TypeAccessSemantics accessSemantics)
 {
     return out << typeAccessSemanticsToString(accessSemantics)
                << typeAccessSemanticsFlagsToString(accessSemantics);
 }
 
+std::ostream &operator<<(std::ostream &out, IsQualified isQualified)
+{
+    return out << isQualifiedToString(isQualified);
+}
+
 std::ostream &operator<<(std::ostream &out, VersionNumber versionNumber)
 {
-    return out << versionNumber.version;
+    return out << versionNumber.value;
 }
 
 std::ostream &operator<<(std::ostream &out, Version version)
@@ -1019,12 +1095,8 @@ std::ostream &operator<<(std::ostream &out, Version version)
 
 std::ostream &operator<<(std::ostream &out, const ExportedType &exportedType)
 {
-    return out << "(\"" << exportedType.name << "\")";
-}
-
-std::ostream &operator<<(std::ostream &out, const ExplicitExportedType &exportedType)
-{
-    return out << "(\"" << exportedType.name << "\"" << exportedType.importId << ")";
+    return out << "(\"" << exportedType.name << "\"," << exportedType.moduleId << ", "
+               << exportedType.version << ")";
 }
 
 std::ostream &operator<<(std::ostream &out, const NativeType &nativeType)
@@ -1032,13 +1104,21 @@ std::ostream &operator<<(std::ostream &out, const NativeType &nativeType)
     return out << "(\"" << nativeType.name << "\")";
 }
 
+std::ostream &operator<<(std::ostream &out, const ImportedType &importedType)
+{
+    return out << "(\"" << importedType.name << ")";
+}
+std::ostream &operator<<(std::ostream &out, const QualifiedImportedType &importedType)
+{
+    return out << "(\"" << importedType.name << "\", " << importedType.import << ")";
+}
+
 std::ostream &operator<<(std::ostream &out, const Type &type)
 {
     using Utils::operator<<;
-    return out << "(import: " << type.importId << ", typename: \"" << type.typeName
-               << "\", prototype: \"" << type.prototype << "\", " << type.accessSemantics
-               << ", source: " << type.sourceId << ", exports: " << type.exportedTypes
-               << ", properties: " << type.propertyDeclarations
+    return out << "( typename: \"" << type.typeName << "\", prototype: " << type.prototype << ", "
+               << type.prototypeId << ", " << type.accessSemantics << ", source: " << type.sourceId
+               << ", exports: " << type.exportedTypes << ", properties: " << type.propertyDeclarations
                << ", functions: " << type.functionDeclarations
                << ", signals: " << type.signalDeclarations << ")";
 }
@@ -1046,8 +1126,10 @@ std::ostream &operator<<(std::ostream &out, const Type &type)
 std::ostream &operator<<(std::ostream &out, const PropertyDeclaration &propertyDeclaration)
 {
     using Utils::operator<<;
-    return out << "(\"" << propertyDeclaration.name << "\", \"" << propertyDeclaration.typeName
-               << "\", " << propertyDeclaration.traits << ", " << propertyDeclaration.typeId << ")";
+    return out << "(\"" << propertyDeclaration.name << "\", " << propertyDeclaration.typeName
+               << ", " << propertyDeclaration.typeId << ", " << propertyDeclaration.traits << ", "
+               << propertyDeclaration.typeId << ", \"" << propertyDeclaration.aliasPropertyName
+               << "\")";
 }
 
 std::ostream &operator<<(std::ostream &out, PropertyDeclarationTraits traits)
@@ -1104,15 +1186,14 @@ std::ostream &operator<<(std::ostream &out, const EnumerationDeclaration &enumer
                << enumerationDeclaration.enumeratorDeclarations << ")";
 }
 
-std::ostream &operator<<(std::ostream &out, const BasicImport &import)
+std::ostream &operator<<(std::ostream &out, const ImportKind &importKind)
 {
-    return out << "(" << import.name << ", " << import.version << ")";
+    return out << importKindToText(importKind);
 }
 
 std::ostream &operator<<(std::ostream &out, const Import &import)
 {
-    return out << "(" << import.name << ", " << import.version << ", " << import.sourceId << ", "
-               << import.importDependencies << ")";
+    return out << "(" << import.moduleId << ", " << import.version << ", " << import.sourceId << ")";
 }
 
 } // namespace Storage
